var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { loadData } from "./data/data";
import { constructNetwork } from "./networks/creation/constructNetwork";
import { constructScale } from "./scales/scale";
import { constructOrdering } from "./orderings/ordering";
import { select } from "./data/select";
import { constructNetworkComplex } from "./networks/creation/constructNetworkComplex";
import { computeLayout } from "./layouts/layout";
import { convertLinkPath } from "./vis/linkPath";
import { computeTableInduceLayout, labelTable } from "./vis/labelTable";
import { addAxes } from "./vis/axes";
import { groupFlattened, groupFlattenedComplete } from "./data/group";
import { constructExpressionFunction } from "./data/calculate";
import { importNetwork } from "./networks/creation/importNetwork";
import { applyNetworkTransform } from "./networks/transform";
import { constructDependencyGraph, determineEvaluationOrder } from "./dependencyGraph";
import { applySelectionTransform } from "./selectionTransforms/applySelectionTransform";
import { processMapSpec } from "./maps/maps";
import { groupNetwork } from "./networks/group";
import { addBorder } from "./vis/border";
const evaluateConditional = (valObj, datum, state) => {
    let conditions = [];
    if ("condition" in valObj) {
        conditions = [valObj.condition];
    }
    else {
        conditions = valObj.conditions;
    }
    // Loop through conditions until we find one that is true
    for (const condition of conditions) {
        let conditionIsSatisfied = false;
        if ("test" in condition) {
            const f = constructExpressionFunction({ calculate: condition.test });
            conditionIsSatisfied = !!f(datum, null, state); // TODO: only need to pass params
        }
        else if ("inSelection" in condition) {
            const selection = select(state, condition.inSelection);
            conditionIsSatisfied = selection.map(e => e.id).includes(datum.id);
        }
        else if ("selectionIsEmpty" in condition) {
            const selection = select(state, condition.selectionIsEmpty);
            conditionIsSatisfied = selection.length == 0;
        }
        else if ("selectionIsNotEmpty" in condition) {
            const selection = select(state, condition.selectionIsNotEmpty);
            conditionIsSatisfied = selection.length > 0;
        }
        if (conditionIsSatisfied) {
            if ("value" in condition) {
                // select the specific value
                return condition.value;
            }
            else {
                // select the object value, with the test/inSelection fields removed
                const { test, inSelection } = condition, rest = __rest(condition, ["test", "inSelection"]);
                return rest;
            }
        }
    }
    // If none matched, return the default values
    if ("value" in valObj) {
        return valObj.value;
    }
    else {
        // select the fragment of the spec with the condition removed
        const { condition } = valObj, rest = __rest(valObj, ["condition"]);
        return rest;
    }
};
const convertVal = (valObj, datum, state, defaultVal, bounds, index) => {
    let value;
    if (typeof (valObj) === "undefined") {
        return defaultVal;
    }
    if (typeof valObj === "object" && ("condition" in valObj || "conditions" in valObj)) {
        valObj = evaluateConditional(valObj, datum, state);
    }
    // first get the value
    if (typeof (valObj) === "number" || typeof (valObj) === "boolean") {
        value = valObj;
    }
    else if (typeof (valObj) === "string") {
        value = valObj;
    }
    else if ("field" in valObj) {
        value = select(datum, valObj.field);
    }
    else if ("ordering" in valObj) {
        // This assumes ordering is defined as mapping from id to position
        const ordering = state[valObj.ordering];
        value = ordering.get(datum.id);
    }
    else if ("expression" in valObj) {
        const f = constructExpressionFunction({ calculate: valObj.expression });
        value = f(datum, index, state, bounds);
    }
    else if ("value" in valObj) {
        value = valObj.value;
    }
    if (typeof (valObj) === "object" && "scale" in valObj) {
        const scale = state[valObj.scale];
        return scale(value);
    }
    else if (typeof (valObj) === "object" && "parameter" in valObj) {
        return state[valObj.parameter];
    }
    else {
        return value;
    }
};
const setFieldsOnItem = (sceneItem, spec, entry, state, bounds, index) => {
    // add concrete values to sceneItem, based on expressions in spec
    const vegaNames = {
        "area": "size"
    };
    for (const field in spec.mark) {
        if (field === "tooltip" && spec.mark.tooltip === true) {
            // "tooltip": true means the whole object whould be displayed in the tooltip
            sceneItem["tooltip"] = entry;
        }
        else if (!["type", "type", "actions"].includes(field)) {
            const fieldName = (field in vegaNames) ? vegaNames[field] : field;
            sceneItem[fieldName] = convertVal(spec.mark[field], entry, state, null, bounds, index);
        }
    }
    // add default black color for text
    if (spec.mark.type === "text" && !spec.mark.fill) {
        sceneItem.fill = "black";
    }
    else if (!("fill" in spec.mark)) {
        sceneItem.fill = "steelblue";
    }
    // cf. handling of x2 and y2 in adjustSpatial() function in vega-runtime
    if ("y2" in spec.mark) {
        if ("y" in spec.mark) {
            sceneItem.height = sceneItem.y2 - sceneItem.y;
        }
        else if ("height" in spec.mark) {
            sceneItem.y = sceneItem.y2 - (sceneItem.height || 0);
        }
    }
    if ("x2" in spec.mark) {
        if ("x" in spec.mark) {
            sceneItem.width = sceneItem.x2 - sceneItem.x;
        }
        else if ("width" in spec.mark) {
            sceneItem.x = sceneItem.x2 - (sceneItem.width || 0);
        }
    }
    if ("actions" in spec) {
        sceneItem.actions = spec.actions;
    }
};
const scaleLayoutToBounds = (layout, bounds) => {
    // rescale a layout to fit in specified bounds
    // Example where this is needed: a nested layout, where we've grouped edges by type, and want to draw small-multiples
    // using the same layout in each.
    // copying like this causes errors if ids are string
    const newLayout = layout; //JSON.parse(JSON.stringify(layout));
    let maxX = 0;
    let maxY = 0;
    let minX = Infinity;
    let minY = Infinity;
    for (const el of layout.keys()) {
        if (!layout.get(el)) {
            continue;
        }
        if (isNaN(layout.get(el).x) || isNaN(layout.get(el).y)) {
            continue;
        }
        maxX = Math.max(maxX, layout.get(el).x + layout.get(el).width);
        maxY = Math.max(maxY, layout.get(el).y + layout.get(el).height);
        minX = Math.min(minX, layout.get(el).x);
        minY = Math.min(minY, layout.get(el).y);
    }
    const xScaling = bounds.width / (maxX - minX);
    const yScaling = bounds.height / (maxY - minY);
    const scaling = Math.min(xScaling, yScaling);
    for (const el of newLayout.keys()) {
        const pos = newLayout.get(el);
        if (!pos) {
            continue;
        }
        pos.x = (pos.x - minX) * scaling;
        pos.y = (pos.y - minY) * scaling;
    }
    return newLayout;
};
const convertMark = (spec, entry, state, createMap, layout, bounds, index, extra) => __awaiter(void 0, void 0, void 0, function* () {
    // Converts from a mark object to a Vega scenegraph compatible object
    var _a;
    if (spec.mark.type === "linkpath") {
        return convertLinkPath(spec, entry, state, layout, extra, bounds);
    }
    else if (spec.mark.type === "map") {
        yield createMap(state[spec.mark.map + "#config"], spec.mark.map, state, bounds);
        return [];
    }
    else {
        // something that can pass through
        // accept circle/square/diamond as mark types, as a shorthand for setting mark type = "symbol" and shape
        const symbolNames = ["circle", "square", "diamond"];
        const marktype = symbolNames.includes(spec.mark.type) ? "symbol" : spec.mark.type;
        let item = {};
        const layout_name = ("layout" in spec) ? spec.layout : null;
        const actions = ("actions" in spec) ? spec.actions : null;
        if ("layout" in spec) {
            const pos = layout.get(entry.id);
            item = { x: pos.x, y: pos.y };
        }
        if (symbolNames.includes(spec.mark.type)) {
            item = Object.assign(Object.assign({}, item), { shape: spec.mark.type, area: (_a = spec.mark.area) !== null && _a !== void 0 ? _a : 50 });
        }
        const sceneItem = {
            marktype,
            items: [item],
            id: entry.id,
            layout_name,
            actions,
            dataset: spec.entries // so we can update correct selection
        };
        setFieldsOnItem(sceneItem.items[0], spec, entry, state, bounds, index);
        //  sceneItem.items[0].actions = actions;
        // console.log({sceneItem})
        return [sceneItem];
    }
});
function processNetworkSpec(networkSpec, state) {
    return __awaiter(this, void 0, void 0, function* () {
        if ("source_node" in networkSpec) {
            state[networkSpec.name] = constructNetwork(state, networkSpec);
        }
        else if ("format" in networkSpec) {
            state[networkSpec.name] = yield importNetwork(networkSpec);
        }
        else if ("source" in networkSpec) {
            // TODO: extract to separate function
            const oldNet = state[networkSpec.source];
            const newNet = { nodes: [], links: [] };
            const nodes = new Map();
            for (const n of oldNet.nodes) {
                const newNode = JSON.parse(JSON.stringify(n));
                nodes.set(n.id, newNode);
                newNet.nodes.push(n);
            }
            for (const l of oldNet.links) {
                newNet.links.push(Object.assign(Object.assign({}, JSON.parse(JSON.stringify(l))), { source: nodes.get(l.source.id), target: nodes.get(l.target.id) }));
            }
            state[networkSpec.name] = newNet;
        }
        else {
            state[networkSpec.name] = constructNetworkComplex(state, networkSpec);
        }
        // Ensure links have an id: if none is set, assign each link its array index as the id
        state[networkSpec.name].links = state[networkSpec.name].links.map((l, i) => (Object.assign({ id: i }, l)));
        for (const transform of (networkSpec.transform || [])) {
            const network = state[networkSpec.name];
            applyNetworkTransform(network, transform, state);
            state[networkSpec.name] = network;
        }
    });
}
const processVisSpec = (visSpec, scene, state, extra, bounds, createMap, fullSpec, funcs) => __awaiter(void 0, void 0, void 0, function* () {
    // If vis entry is defined using a table, modify to use corresponding computed layout and entries
    if ("table" in visSpec) {
        const { groupName } = state[visSpec.table];
        visSpec = Object.assign(Object.assign({}, visSpec), { entries: groupName, layout: visSpec.table + "#layout" });
    }
    let elementsInSelection = [];
    if ("ifInSelection" in visSpec) {
        elementsInSelection = select(state, visSpec.ifInSelection).map(e => e.id);
    }
    //////////////////////////////////////////////////////
    // each "vis" definition gets a separate group
    const group = {
        "marktype": "group",
        "items": [
            {
                x: 0,
                y: 0,
                //  "width": bounds.width + 2 * bounds.x,
                //  "height": bounds.height + 2 * bounds.y,
                "items": [] // array of contained mark instances
            }
        ]
    };
    scene.items[0].items.push(group);
    //scene = group; // causes parent--child1--child2--child3 chaining
    if ("table" in visSpec) {
        const { rowGroupIds, colGroupIds, distinctRows, distinctCols } = state[visSpec.table];
        const layout = scaleLayoutToBounds(state[visSpec.table + "#layout"], bounds);
        scene.items[0].items.push(...labelTable(distinctRows, distinctCols, state, visSpec.table, visSpec, rowGroupIds, colGroupIds, layout, extra));
        computeTableInduceLayout(distinctRows, distinctCols, state, visSpec.table, visSpec, rowGroupIds, colGroupIds, layout, extra, bounds);
    }
    // TODO: set bounds
    if ("vis" in visSpec) {
        // this is parent "vis" block, with child vis blocks
        // we need to loop over entries here, so things like locally-scoped layouts are calculated separately each time they are needed
        let entries = select(state, visSpec.entries);
        entries = Array.isArray(entries) ? entries : [entries];
        for (const entry of (entries || [])) {
            // create a new group for each sub-visualization
            let entryBounds = { x: bounds.x, y: bounds.y, width: bounds.width, height: bounds.height };
            if ("layout" in visSpec) {
                entryBounds = state[visSpec.layout].get(entry.id);
            }
            const group2 = {
                "marktype": "group",
                "items": [
                    Object.assign(Object.assign({}, entryBounds), { "items": [] // array of contained mark instances
                     })
                ]
            };
            group.items[0].items.push(group2);
            addAxes(visSpec, group2, state);
            let draw = false;
            if ("if" in visSpec) {
                const f = constructExpressionFunction({ calculate: visSpec.if });
                draw = !!f(entry, null, state); // TODO: only need to pass params
            }
            else if (!("ifInSelection" in visSpec) || elementsInSelection.includes(entry.id)) {
                draw = true;
            }
            if (draw) {
                addBorder(visSpec.border, group2, state, entryBounds);
                yield processSpec(visSpec, entryBounds, Object.assign(Object.assign({}, state), { entry }), extra, group2, createMap, fullSpec, funcs);
            }
            // for (const innerVisSpec of visSpec.vis) {
            //  await processSpec(innerVisSpec, bounds, { ...state, entry }, scene);
            // }
        }
    }
    else {
        // This is a "leaf" vis block
        yield processSpec(visSpec, bounds, state, extra, group, createMap, fullSpec, funcs);
    }
});
export const ensureInducedLayoutCreated = (extra, inducedLayoutName) => {
    if (!Object.keys(extra).includes("INDUCED_LAYOUTS")) {
        extra.INDUCED_LAYOUTS = [];
    }
    if (!extra.INDUCED_LAYOUTS.includes(inducedLayoutName)) {
        extra.INDUCED_LAYOUTS.push(inducedLayoutName);
    }
    if (!Object.keys(extra).includes(inducedLayoutName)) {
        extra[inducedLayoutName] = new Map();
    }
};
function processVisSpecLeaf(spec, parentState, extra, bounds, fullSpec, createMap) {
    var _a, _b, _c, _d, _e;
    return __awaiter(this, void 0, void 0, function* () {
        // the whole spec is a (leaf) vis block
        const state = Object.assign(Object.assign({}, parentState), extra);
        // If vis entry is defined using a table, modify to use corresponding computed layout and entries
        if ("table" in spec) {
            const { groupName } = state[spec.table];
            spec = Object.assign(Object.assign({}, spec), { entries: groupName, layout: spec.table + "#layout" });
        }
        let elementsInSelection = [];
        if ("ifInSelection" in spec) {
            elementsInSelection = select(state, spec.ifInSelection).map(e => e.id);
        }
        ///////////////////////////////////////////////////////////////////////
        let layout;
        if ("layout" in spec) {
            if (spec.layout.endsWith("#layout")) {
                // a table
                layout = scaleLayoutToBounds(state[spec.layout], bounds);
            }
            else {
                // TOOD: review implementation of this if statement
                // don't scale if the layout is from a map!
                // N.B. fullSpec.layouts may be empty if all layouts are locally scoped
                const layoutSpec = ((_a = fullSpec.layouts) !== null && _a !== void 0 ? _a : []).filter(l => l.name === spec.layout)[0];
                if (layoutSpec && "map" in layoutSpec) {
                    layout = state[spec.layout];
                }
                else if (state.INDUCED_LAYOUTS && state.INDUCED_LAYOUTS.includes(spec.layout)) {
                    layout = state[spec.layout];
                }
                else {
                    layout = scaleLayoutToBounds(state[spec.layout], bounds);
                }
            }
        }
        ////////////////////////////////////////////////////////////////////////////////
        let entries = select(state, spec.entries);
        entries = Array.isArray(entries) ? entries : [entries];
        const items = [];
        for (const entry of (entries || [])) {
            // TODO: auto-set special variables:
            // - bounds
            // I don't think we need to set `entry here: it is only needed when nesting
            // if using a table layout, bounds should be set to the bounds of the table cell, not the whole table
            let b = Object.assign({}, bounds);
            if ("table" in entry) {
                const { rowGroupIds, colGroupIds, distinctRows, distinctCols } = state[spec.table];
                items.push(...labelTable(distinctRows, distinctCols, state, spec.table, spec, rowGroupIds, colGroupIds, layout, extra));
                computeTableInduceLayout(distinctRows, distinctCols, state, spec.table, spec, rowGroupIds, colGroupIds, layout, extra, bounds);
            }
            if ("table" in spec) {
                const pos = layout.get(entry.id);
                b = { width: pos.width, height: pos.height, x: pos.x, y: pos.y };
            }
            let draw = false;
            if ("if" in spec) {
                const f = constructExpressionFunction({ calculate: spec.if });
                draw = !!f(entry, null, state); // TODO: only need to pass params
            }
            else if (!("ifInSelection" in spec) || elementsInSelection.includes(entry.id)) {
                draw = true;
            }
            if (draw) {
                const mark = yield convertMark(spec, entry, state, createMap, layout, b, items.length, extra);
                items.push(...mark);
            }
            if ("inducedLayout" in spec && !("table" in spec) && (spec.mark.type !== "linkpath")) {
                const inducedLayoutName = spec.inducedLayout.name;
                ensureInducedLayoutCreated(extra, inducedLayoutName);
                const xOffset = spec.x ? convertVal(spec.x, entry, state, 0, bounds, items.length) : 0;
                const yOffset = spec.y ? convertVal(spec.y, entry, state, 0, bounds, items.length) : 0;
                extra[inducedLayoutName].set(entry.id, {
                    x: bounds.x + layout.get(entry.id).x + xOffset,
                    y: bounds.y + layout.get(entry.id).y + yOffset,
                    width: (_c = (_b = spec.width) !== null && _b !== void 0 ? _b : layout.get(entry.id).width) !== null && _c !== void 0 ? _c : 100,
                    height: (_e = (_d = spec.height) !== null && _d !== void 0 ? _d : layout.get(entry.id).height) !== null && _e !== void 0 ? _e : 100
                });
            }
        }
        // this is an isolated vis item, such as a label
        if (!("entries" in spec)) {
            const mark = yield convertMark(spec, {}, state, createMap, layout, bounds, items.length, extra);
            items.push(...mark);
        }
        // group items into one mark
        if ((spec.mark.type === "line" || spec.mark.type === "area") && items.length > 0) {
            return [{
                    marktype: items[0].marktype,
                    items: items.map(it => it.items[0])
                }];
        }
        return items;
    });
}
function processSpec(spec, bounds, parentState, extra, scene, createMap, fullSpec, funcs) {
    return __awaiter(this, void 0, void 0, function* () {
        // state stores all named things - datasets, networks, layouts, scales, etc
        if (spec.vis) {
            // get the data
            let state;
            if (spec.data) {
                state = yield Object.assign(Object.assign({}, parentState), (yield loadData(spec.data, funcs)));
            }
            else {
                // There might not be a data block, if a network file is imported in network block
                state = Object.assign({}, parentState);
            }
            // const state = await loadData(spec.data)
            // do the join
            // addRelationships(state, spec.relationships);
            for (const param of (spec.parameters || [])) {
                if (param.type === "selection") {
                    state[param.name] = { nodes: [], links: [] };
                }
                if (param.value) {
                    state[param.name] = param.value;
                }
                else if (param.type === "selection" || "source" in param) {
                    state[param.name] = { nodes: [], links: [], network: param.network };
                }
                else {
                    state[param.name] = 0;
                }
            }
            // construct networks at top-level
            for (const networkSpec of (spec.networks || [])) {
                yield processNetworkSpec(networkSpec, state);
            }
            // compute groupings
            for (const grouping of (spec.groupings || [])) {
                if ("data" in grouping) {
                    state[grouping.name] = groupFlattened(state, grouping.data, grouping.common);
                }
                else {
                    const { groups, interGroupLinks } = groupNetwork(state, grouping.network, grouping.common);
                    state[grouping.name] = groups;
                    state[grouping.name + "_INTER_GROUP_LINKS"] = interGroupLinks;
                }
            }
            // construct scales
            for (const scaleSpec of (spec.scales || [])) {
                state[scaleSpec.name] = constructScale(scaleSpec, bounds, state);
            }
            // construct orderings
            const orderFields = new Map();
            for (const orderingSpec of (spec.orderings || [])) {
                state[orderingSpec.name] = constructOrdering(state, orderingSpec);
                if ("field" in orderingSpec) {
                    orderFields.set(orderingSpec.name, orderingSpec.field);
                }
            }
            // construct tables
            for (const tableSpec of (spec.tables || [])) {
                const { xOrderName, yOrderName, groupName, rowGroupIds, colGroupIds, distinctRows, distinctCols } = yield constructTable(tableSpec, state, bounds);
                state[tableSpec.name] = { xOrderName, yOrderName, groupName, rowGroupIds, colGroupIds, distinctRows, distinctCols };
            }
            // handle maps
            // need to do this before layout is computed using these
            for (const mapSpec of (spec.maps || [])) {
                state[mapSpec.name + "#config"] = processMapSpec(mapSpec);
            }
            // do layout
            const addToState = (name, val) => state[name] = val;
            for (const layoutSpec of (spec.layouts || [])) {
                const [layout, axes] = yield computeLayout(Object.assign(Object.assign({}, state), { _width: bounds.width, _height: bounds.height }), layoutSpec, bounds, addToState, funcs);
                state[layoutSpec.name] = layout;
                state[layoutSpec.name + "_AXES"] = axes;
            }
            // recurse downwards
            //let output = {};
            for (const visSpec of (spec.vis || [])) {
                // Note that processVisSpec can call processSpec, as vis spec can define scales/orders/layouts etc.
                yield processVisSpec(visSpec, scene, state, extra, bounds, createMap, fullSpec, funcs);
            }
            if (spec.vis.length === 0) {
                console.log({ state: state });
            }
            return state;
        }
        else {
            // the whole spec is a (leaf) vis block
            addAxes(spec, scene, parentState);
            const items = yield processVisSpecLeaf(spec, parentState, extra, bounds, fullSpec, createMap);
            Object.assign(parentState, extra); // TODO: check this fixed it
            scene.items[0].items = [...scene.items[0].items, ...items];
        }
        if (spec.vis && spec.vis.length === 0) {
            console.log({ state: parentState });
        }
        return parentState;
    });
}
function processFullSpec(spec, bounds, state, createMap, funcs) {
    return __awaiter(this, void 0, void 0, function* () {
        const scene = {
            "marktype": "group",
            "items": [
                {
                    "x": bounds.x,
                    "y": bounds.y,
                    "width": bounds.width + 2 * bounds.x,
                    "height": bounds.height + 2 * bounds.y,
                    "items": [] // array of contained mark instances
                }
            ]
        };
        if (!spec.vis) {
            spec.vis = [];
        }
        console.log("Dependency graph:", constructDependencyGraph(spec));
        // const renderer = new SVGStringRenderer();
        const extra = {};
        const newState = yield processSpec(spec, bounds, state, extra, scene, createMap, spec, funcs);
        return { sceneJSON: scene, state: Object.assign(Object.assign({}, newState), extra) };
    });
}
const getBoundsFromSpec = (specObj) => {
    var _a, _b;
    const paddingLeft = (_a = specObj.x) !== null && _a !== void 0 ? _a : 25;
    const paddingTop = (_b = specObj.y) !== null && _b !== void 0 ? _b : 25;
    const innerWidth = specObj.width || 1000;
    const innerHeight = specObj.height || 1000;
    return { x: paddingLeft, y: paddingTop, width: innerWidth, height: innerHeight };
};
const constructTable = (tableSpec, state, bounds) => __awaiter(void 0, void 0, void 0, function* () {
    const fields = [];
    let xOrderName;
    let yOrderName;
    let groupName;
    // find fields to group on  - need to do this before we can compute the grouping
    if ("order" in tableSpec.colOrder) {
        // fields.push(orderFields.get(tableSpec.colOrder.field));
        fields.push(tableSpec.colOrder.field);
    }
    else {
        fields.push(tableSpec.colOrder.field);
    }
    if ("order" in tableSpec.rowOrder) {
        fields.push(tableSpec.rowOrder.field);
    }
    else {
        fields.push(tableSpec.rowOrder.field);
    }
    // group data
    if ("group" in tableSpec) {
        groupName = tableSpec.group;
    }
    else {
        // TODO: if user create group manually, it should be complete - need way to specify this
        groupName = tableSpec.name + "#grouping";
        state[groupName] = groupFlattenedComplete(state, tableSpec.data, fields, tableSpec.symmetric, tableSpec.fullDataset);
    }
    // find distinct row/col values
    const colIds = [];
    const distinctCols = [];
    const colGroupIds = [];
    const rowIds = [];
    const distinctRows = [];
    const rowGroupIds = [];
    for (const g of state[tableSpec.name + "#grouping"]) {
        let col = g.key[0];
        if (typeof col !== "object") {
            col = { id: col };
        }
        if (!colIds.includes(col.id)) {
            distinctCols.push(col);
            colIds.push(col.id);
            colGroupIds.push(g.id);
        }
        let row = g.key[1];
        if (typeof row !== "object") {
            row = { id: row };
        }
        if (!rowIds.includes(row.id)) {
            distinctRows.push(row);
            rowIds.push(row.id);
            rowGroupIds.push(g.id);
        }
    }
    // compute orderings - can't do this until we've grouped the data
    if ("order" in tableSpec.colOrder) {
        xOrderName = tableSpec.colOrder.order;
    }
    else {
        state[tableSpec.name + "#colData"] = distinctCols;
        const orderConfig = {
            data: tableSpec.name + "#colData",
            name: tableSpec.name,
            orderBy: Object.assign(Object.assign({}, tableSpec.colOrder), { field: tableSpec.colOrder.field })
        };
        xOrderName = tableSpec.name + "#colOrdering";
        state[xOrderName] = constructOrdering(state, orderConfig);
        fields.push(tableSpec.colOrder.field);
    }
    if ("order" in tableSpec.rowOrder) {
        yOrderName = tableSpec.rowOrder.order;
    }
    else {
        state[tableSpec.name + "#rowData"] = distinctRows;
        const orderConfig = {
            data: tableSpec.name + "#rowData",
            name: tableSpec.name,
            orderBy: Object.assign(Object.assign({}, tableSpec.rowOrder), { field: tableSpec.rowOrder.field })
        };
        yOrderName = tableSpec.name + "#rowOrdering";
        state[yOrderName] = constructOrdering(state, orderConfig);
    }
    const layout = new Map();
    const cellWidth = bounds.width / distinctCols.length;
    const cellHeight = bounds.height / distinctRows.length;
    for (const g of state[tableSpec.name + "#grouping"]) {
        let col = g.key[0];
        if (typeof col !== "object") {
            col = { id: col };
        }
        let row = g.key[1];
        if (typeof row !== "object") {
            row = { id: row };
        }
        layout.set(g.id, {
            x: state[xOrderName].get(col.id) * cellWidth,
            y: state[yOrderName].get(row.id) * cellHeight,
            width: cellWidth,
            height: cellHeight
        });
    }
    state[tableSpec.name + "#layout"] = layout;
    return { xOrderName, yOrderName, groupName, distinctRows, distinctCols, rowGroupIds, colGroupIds };
});
function updateSpec(spec, state, modifiedItems, createMap, funcs) {
    return __awaiter(this, void 0, void 0, function* () {
        const dependencyGraph = constructDependencyGraph(spec);
        // const bounds = {width: spec.width || 1000, height: spec.height || 1000, x: 25, y: 25}; // TODO: FIXME
        const bounds = getBoundsFromSpec(spec);
        // compute evaluation order, but skip the recomputation of the modified items (as they have already been adjusted)
        const reevaluationOrder = determineEvaluationOrder(dependencyGraph, modifiedItems).filter(item => !modifiedItems.includes(item.name));
        for (const step of reevaluationOrder) {
            if (step.type === "data") {
                // @ts-ignore
                state = yield Object.assign(Object.assign({}, state), (yield loadData(step.spec, funcs)));
            }
            else if (step.type === "grouping") {
                // todo: check if includeEmptyGroups is true
                state[step.spec.name] = groupFlattened(state, step.spec.data, step.spec.common);
            }
            else if (step.type === "network") {
                yield processNetworkSpec(step.spec, state);
            }
            else if (step.type === "layout") {
                const addToState = (name, val) => state[name] = val;
                const [layout, axes] = yield computeLayout(state, step.spec, bounds, addToState, funcs);
                //   const [layout, axes] = await computeLayout({...state, _width: bounds.width, _height: bounds.height}, layoutSpec, bounds);
                state[step.spec.name] = layout;
                state[step.spec.name + "_AXES"] = axes;
            }
            else if (step.type === "scale") {
                // @ts-ignore
                state[step.spec.name] = constructScale(step.spec, bounds, state);
            }
            else if (step.type === "ordering") {
                state[step.spec.name] = constructOrdering(state, step.spec);
            }
            else if (step.type === "parameter") {
                // a param will only be re-evaluated if it is a DerivedParameter, but TS wants us to check this
                if ("source" in step.spec) {
                    state[step.spec.name] = applySelectionTransform(state, step.spec, spec);
                }
            }
            else if (step.type === "table") {
                const { xOrderName, yOrderName, groupName, rowGroupIds, colGroupIds, distinctRows, distinctCols } = yield constructTable(step.spec, state, bounds);
                state[step.spec.name] = { xOrderName, yOrderName, groupName, rowGroupIds, colGroupIds, distinctRows, distinctCols };
            }
            //else if (step.type === "vis"){
            //
            //}
        }
        // handle vis
        const extra = {};
        const scene = {
            "marktype": "group",
            "items": [
                {
                    "x": bounds.x,
                    "y": bounds.y,
                    "width": bounds.width + 2 * bounds.x,
                    "height": bounds.height + 2 * bounds.y,
                    "items": [] // array of contained mark instances
                }
            ]
        };
        // recurse downwards
        for (const visSpec of (spec.vis || [])) {
            yield processVisSpec(visSpec, scene, state, extra, bounds, createMap, spec, funcs); // TODO: check state is fllstate
        }
        if (spec.vis.length === 0) {
            console.log({ state: state });
        }
        //
        return { sceneJSON: scene, state: Object.assign(Object.assign({}, state), extra) };
    });
}
export { processFullSpec, setFieldsOnItem, updateSpec, convertVal, getBoundsFromSpec };
//# sourceMappingURL=complete.js.map