import { deviation, max, mean, median, min, mode, sum, variance, quantile, minIndex, maxIndex } from "d3-array";
import { group } from "./group";
import { select } from "./select";
const uncorrectedVariance = (xs) => variance(xs) * (xs.length - 1) / (xs.length);
const aggregationFunctions = {
    count: (xs) => xs.length,
    valid: (xs) => xs.filter(x => x !== null && typeof (x) !== "undefined" && !(typeof (x) === "number" && Number.isNaN(x))).length,
    missing: (xs) => xs.filter(x => x === null || typeof (x) === "undefined" || x === "").length,
    distinct: (xs) => new Set(xs).size,
    sum,
    product: (xs) => xs.reduce((res, item) => res * item, 1),
    mean,
    average: mean,
    variance: uncorrectedVariance,
    variancep: variance,
    stdev: (xs) => Math.sqrt(uncorrectedVariance(xs)),
    stdevp: deviation,
    median,
    q1: (xs) => quantile(xs, 0.25),
    q3: (xs) => quantile(xs, 0.75),
    min,
    max,
    argmin: (xs) => xs[minIndex(xs)],
    argmax: (xs) => xs[maxIndex(xs)],
    mode
};
// not implemented: stderr, ci0, ci1
// new addition not in Vega: mode
// unlike Vega, we don't provide a "values" aggregation; instead use a grouping
// performs a single aggregation operation
const computeAggregation = (data, spec) => {
    // group data
    var _a, _b, _c;
    const grouped = spec.groupby ? group({ data }, "data", spec.groupby) : { all: data };
    const result = [];
    if (!spec.fields || spec.fields.length === 0) {
        // no fields specified, so default to just doing a COUNT aggregation
        for (const groupName of Object.keys(grouped)) {
            const groupData = grouped[groupName];
            const resultFieldName = ((_a = spec.as) === null || _a === void 0 ? void 0 : _a[0]) || `count`;
            const aggregatedGroup = {};
            for (const field of spec.groupby) {
                aggregatedGroup[field] = select(groupData[0], field); // groupData[0][field];
            }
            aggregatedGroup[resultFieldName] = aggregationFunctions.count(groupData);
            result.push(aggregatedGroup);
        }
    }
    else {
        // do the specified aggregation
        for (const groupName of Object.keys(grouped)) {
            const groupData = grouped[groupName];
            const aggregatedGroup = {};
            for (const field of spec.groupby || []) {
                aggregatedGroup[field] = select(groupData[0], field); // groupData[0][field];
            }
            for (const i in spec.fields) {
                const field = spec.fields[i];
                const agFuncName = ((_b = spec.ops) === null || _b === void 0 ? void 0 : _b[i]) || "count";
                const aggregationFunction = aggregationFunctions[agFuncName];
                const resultFieldName = ((_c = spec.as) === null || _c === void 0 ? void 0 : _c[i]) || agFuncName;
                aggregatedGroup[resultFieldName] = aggregationFunction(groupData.map(d => d[field]));
            }
            // const aggregatedGroup = spec.fields.map( (field, i) => aggregationFunctions[] )
            result.push(aggregatedGroup);
        }
    }
    return result;
};
export { computeAggregation, aggregationFunctions };
//# sourceMappingURL=aggregation.js.map