import { codegenExpression, parseExpression, functions } from "vega-expression";
import { functionContext } from "vega-functions";
import { utcFormat, timeFormat, utcParse, timeParse } from "d3-time-format";
const constructExpressionFunction = (spec) => {
    const thisPrefix = "this."; // function context prefix
    // This function is copied from https://github.com/vega/vega/blob/bde41b2cb0bdf282fb9d526462428d1a0e73832d/packages/vega-functions/src/codegen.js#L309
    function buildFunctions(codegen) {
        const fn = functions(codegen);
        for (const name in functionContext) {
            fn[name] = thisPrefix + name;
        }
        // This is a bit of a hack: vega-functions provides versions of these functions, but they don't work for us as they
        // try to identify the locale from a Vega dataflow, which we don't have
        const dateFns = { timeFormat, timeParse, utcFormat, utcParse };
        for (const fnName of Object.keys(dateFns)) {
            // @ts-ignore
            fn[fnName] = thisPrefix + fnName;
            functionContext[fnName] = (value, specifier) => dateFns[fnName](specifier)(value);
        }
        return fn;
    }
    const codegen = codegenExpression({
        globalvar: "globalVar",
        functions: buildFunctions,
    });
    const funcCode = codegen(parseExpression(spec.calculate));
    // First argument means that the function argument will be interpreted as having name "globalVar"
    // so we can avoid using an actual global variable
    const fn = Function("globalVar", "\"use strict\"; return (" + funcCode.code + ")");
    return (d, index, params, bounds) => fn.apply(functionContext, [{ datum: d, index, params, bounds }]);
};
const calculateCalculate = (data, spec) => {
    const f = constructExpressionFunction(spec);
    // spec.as may be an expression referring to a nested field (eg, data.Name):
    // need to save results to datum.data.Name rather than datum["data.Name"]
    for (let i = 0; i < data.length; i++) {
        let t = data[i];
        const parts = spec.as.split(".");
        while (parts.length > 1) {
            t = t[parts.shift()];
        }
        t[parts[0]] = f(data[i], i);
    }
    return data;
};
export { calculateCalculate, constructExpressionFunction };
//# sourceMappingURL=calculate.js.map