var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// see vega-parsers/src/parsers/data.js
// @ts-ignore
import { loader, read } from "vega-loader";
import { applyTransform } from "./transform";
import { select } from "./select";
function loadData(fullDataSpec, funcs) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = {};
        for (const dataset of fullDataSpec) {
            const data = yield loadSingleDataset(dataset, allData, funcs);
            if (data) {
                allData[data.name] = data.values;
                if ("transform" in dataset) {
                    for (const transformSpec of dataset.transform) {
                        data.values = yield applyTransform(allData, data.values, transformSpec, funcs);
                    }
                }
                allData[data.name] = data.values;
            }
        }
        return allData;
    });
}
function loadSingleDataset(dataSpec, allData, funcs) {
    return __awaiter(this, void 0, void 0, function* () {
        if ("values" in dataSpec) {
            return { name: dataSpec.name, values: dataSpec.values };
        }
        else if ("url" in dataSpec) {
            const dataText = yield loader().load(dataSpec.url);
            return { name: dataSpec.name, values: read(dataText, dataSpec.format) };
        }
        else if ("source" in dataSpec) {
            const data = JSON.parse(JSON.stringify(select(allData, dataSpec.source)));
            return { name: dataSpec.name, values: data };
        }
        else if ("localStorage" in dataSpec) {
            const dataText = window.localStorage.getItem(dataSpec.localStorage);
            return { name: dataSpec.name, values: read(dataText, dataSpec.format) };
        }
        else if ("function" in dataSpec) {
            return { name: dataSpec.name, values: funcs[dataSpec.function](dataSpec) };
        }
        // we don't allow named datasets
    });
}
export { loadData };
//# sourceMappingURL=data.js.map