var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// https://positionstack.com/documentation
// UGH: batch requests requires professional plan or higher :-(
// We use https://geocode.maps.co/
// which is powered by OSM and Nominatim
const sleep = (ms) => new Promise(resolve => setTimeout(resolve, ms));
const geocode = (data, spec) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e;
    const locations = new Map();
    // read cached locations from LocalStorage
    let cachedLocations;
    if (window) {
        cachedLocations = JSON.parse((_a = window.localStorage.getItem("locations")) !== null && _a !== void 0 ? _a : "{}");
        for (const locationName of Object.keys(cachedLocations)) {
            locations.set(locationName, cachedLocations[locationName]);
        }
    }
    for (const datum of data) {
        let placeQueryString;
        if ("locationField" in spec) {
            placeQueryString = `q=${datum[spec.locationField]}`;
        }
        else {
            const terms = [];
            const f = n => encodeURIComponent(datum[spec[n]]);
            if ("streetField" in spec) {
                terms.push(`street=${f("streetField")}`);
            }
            if ("cityField" in spec) {
                terms.push(`city=${f("cityField")}`);
            }
            if ("stateField" in spec) {
                terms.push(`state=${f("stateField")}`);
            }
            if ("countryField" in spec) {
                terms.push(`country=${f("countryField")}`);
            }
            if ("postalCodeField" in spec) {
                terms.push(`postalcode=${f("postalCodeField")}`);
            }
            placeQueryString = terms.join("&");
        }
        if (locations.has(placeQueryString)) {
            const location = locations.get(placeQueryString);
            datum[(_b = spec.lonAs) !== null && _b !== void 0 ? _b : "lon"] = location.lon;
            datum[(_c = spec.latAs) !== null && _c !== void 0 ? _c : "lat"] = location.lat;
            if ("nameAs" in spec) {
                datum[spec.nameAs] = location.display_name;
            }
        }
        else {
            // Respect the limit of 2 requests/s listed at (https://geocode.maps.co/)
            yield sleep(800);
            const url = `https://geocode.maps.co/search?${placeQueryString}`;
            const response = yield fetch(url, {
                mode: 'cors',
                headers: {
                    'User-Agent': 'ANYTHING_WILL_WORK_HERE'
                }
            }).then(r => r.json());
            if (response.length > 0) {
                datum[(_d = spec.lonAs) !== null && _d !== void 0 ? _d : "lon"] = response[0].lon;
                datum[(_e = spec.latAs) !== null && _e !== void 0 ? _e : "lat"] = response[0].lat;
                if ("nameAs" in spec) {
                    datum[spec.nameAs] = response[0].display_name;
                }
                locations.set(placeQueryString, response[0]);
            }
            else {
                locations.set(placeQueryString, { lon: undefined, lat: undefined }); // otherwise a re-run will retry the query
            }
        }
    }
    // write fetched locations back to LocalStorage
    if (window) {
        for (const locationName of locations.keys()) {
            cachedLocations[locationName] = locations.get(locationName);
        }
        window.localStorage.setItem("locations", JSON.stringify(cachedLocations));
    }
    return data;
});
export { geocode };
//# sourceMappingURL=geocoding.js.map