// N.B.
// - currently doesn't handle binning, which will be needed for continuous variables
// - do we need to keep track of the names of the fields used for grouping at each level?
// - should we be returning an object with keys, or a list of lists?
// - should handle a predicate/key function for grouping, which would allow arbitrary things (e.g., splitting into even/odd ids)
import { cross } from "d3-array";
import { select } from "./select";
const group = (data, datasetName, fields) => {
    const grouped = {};
    const dataset = data[datasetName];
    for (const d of dataset) {
        let container = grouped;
        for (let i = 0; i < fields.length; i++) {
            const field = fields[i];
            const v = select(d, field);
            // @ts-ignore
            if (!container[v]) {
                // @ts-ignore
                container[v] = (i < fields.length - 1) ? {} : [];
            }
            // @ts-ignore
            container = container[v];
        }
        // @ts-ignore
        container.push(d);
    }
    return grouped;
};
const groupFlattened = (data, datasetName, fields) => {
    const grouped = new Map();
    //const dataset = data[datasetName];
    const dataset = select(data, datasetName);
    let id = 0;
    for (const d of dataset) {
        // we stringify as otherwise different arrays containing same values would be considered different keys
        const key = JSON.stringify(fields.map(field => select(d, field)));
        if (grouped.has(key)) {
            grouped.get(key).data.push(d);
            grouped.get(key).size += 1;
        }
        else {
            grouped.set(key, { id, data: [d], size: 1, key: JSON.parse(key) });
            id++;
        }
    }
    return Array.from(grouped.values());
};
const groupFlattenedComplete = (data, datasetName, fields, symmetric, fullDatasetName) => {
    // need to include keys (can't read them from first data element for empty lists)
    // record list of values as we fi
    const vals = new Map();
    const grouped = new Map();
    //const dataset = data[datasetName];
    const dataset = select(data, datasetName);
    let id = 0;
    const keys = [];
    for (const d of dataset) {
        // update list of all vals for all fields
        for (const field of fields) {
            if (!vals.has(field)) {
                vals.set(field, []);
            }
            const fieldVal = select(d, field);
            const fieldVals = vals.get(field);
            if (!fieldVals.includes(fieldVal)) {
                fieldVals.push(fieldVal);
            }
        }
        // we stringify as otherwise different arrays containing same values would be considered different keys
        const key = JSON.stringify(fields.map(field => select(d, field)));
        keys.push(key);
        if (grouped.has(key)) {
            grouped.get(key).data.push(d);
            grouped.get(key).size += 1;
        }
        else {
            grouped.set(key, { id, data: [d], size: 1, key: JSON.parse(key) });
            id++;
        }
    }
    // fill in gaps - find all combinations, and if not in keys then add
    let combinations;
    if (symmetric) {
        // If symmetric is true, we include an entry for every element in field1 or field2, corssed with every element in field1 or field2
        // this is useful for ajacency matrices of directed networks, where a source/sink node may otherwise appear as a column but not a row (or vice-versa)
        let allVals = [].concat.apply([], ...vals.values());
        if (fullDatasetName) {
            const fullDataset = select(data, fullDatasetName);
            for (const d of fullDataset) {
                allVals.push(select(d, fields[0]));
                allVals.push(select(d, fields[1]));
            }
        }
        /*
        if (rows) {
          allVals = [...allVals, ...rows];
        }*/
        const uniqueVals = Array.from(new Set(allVals));
        combinations = cross(uniqueVals, uniqueVals);
    }
    else {
        // @ts-ignore
        combinations = cross(...vals.values());
    }
    for (const combination of combinations) {
        const key = JSON.stringify(combination);
        if (!keys.includes(key)) {
            grouped.set(key, { id, data: [], size: 0, key: JSON.parse(key) });
            id++;
        }
    }
    return Array.from(grouped.values());
};
export { group, groupFlattened, groupFlattenedComplete };
//# sourceMappingURL=group.js.map