var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { select } from "../data/select";
import { computeTopologicalLayout } from "../topologicalLayouts/layout";
import { computeMapLayout } from "../maps/mapLayout";
import { transformLayout } from "./layoutTransforms";
import { computeExplicitLayout } from "./explicitLayout";
import { computeDerivedLayout } from "./derivedLayout";
import { getPos, layoutRings } from "./oneDimensionalLayouts";
import { layoutPolar, layoutPolarMultipleAngleScales, layoutPolarMultipleRadiusScales } from "./polarLayouts";
import { layoutCartesian, layoutCartesianMultipleXScales, layoutCartesianMultipleYScales } from "./layoutCartesian";
import { rescaleLayout, scale } from "./utils";
// TODO: rather than rescaling, pass down bounds
// scale complicates things...
// Individual layout methods
const computeAttributeLayout = (state, spec, bounds) => {
    var _a, _b;
    let layout = new Map();
    let axes = []; // IDEA is to push this through the same sclaing transformation as individual coordinates
    // but this doesn't quite work for polar coordinates
    // this is a bit of a hack around type checking
    if (!spec.pattern && "x" in spec && "y" in spec) {
        spec.pattern = "cartesian";
    }
    else if (!spec.pattern) {
        spec.pattern = "row-by-row";
    }
    let data;
    if ("network" in spec) {
        data = select(state, spec.network).nodes;
    }
    else {
        data = select(state, spec.data);
    }
    if (spec.pattern === "explicit") {
        return computeExplicitLayout(data, spec, state, bounds);
    }
    else if (spec.pattern === "rings") {
        [layout, axes] = layoutRings(data, spec, state);
    }
    else if (spec.pattern === "polar") {
        if (spec.angle.length > 1 && spec.radius.length > 1) {
            console.log("Error - can't have multiple scales or orders for both radius and angle dimensions in a polar layout");
        }
        else if (spec.angle.length > 1 && spec.radius.length === 1) {
            // this has a separate scale for each concentric ring
            [layout, axes] = layoutPolarMultipleAngleScales(data, spec, state);
        }
        else if (spec.angle.length === 1 && spec.radius.length > 1) {
            [layout, axes] = layoutPolarMultipleRadiusScales(data, spec, state);
        }
        else if (spec.angle.length === 1 && spec.radius.length === 1) {
            // this is the old "polar" plot
            [layout, axes] = layoutPolar(data, spec, state);
        }
    }
    else if (spec.pattern === "cartesian") {
        if (!spec.x || spec.x.length === 0) {
            spec.x = [{ value: 0 }];
        }
        if (!spec.y || spec.y.length === 0) {
            spec.y = [{ value: 0 }];
        }
        if (spec.x.length > 1 && spec.y.length > 1) {
            console.log("Error - can't have multiple scales or orders for both x- and y- dimensions in a layout");
        }
        else if (spec.x.length > 1 && spec.y.length === 1) {
            [layout, axes] = layoutCartesianMultipleXScales(data, spec, state);
        }
        else if (spec.x.length === 1 && spec.y.length > 1) {
            [layout, axes] = layoutCartesianMultipleYScales(data, spec, state);
        }
        else if (spec.x.length === 1 && spec.y.length === 1) {
            [layout, axes] = layoutCartesian(data, spec, state);
        }
        const xIsOrder = "order" in spec.x[0];
        const yIsOrder = "order" in spec.y[0];
        if (xIsOrder && !yIsOrder) {
            scale(layout, "x", "y", true, spec.positioning);
        }
        if (yIsOrder && !xIsOrder) {
            scale(layout, "y", "x", true, spec.positioning);
        }
    }
    else {
        // handle grid and linear layouts
        let maxX = 0;
        let maxY = 0;
        for (const datum of data) {
            const pos = getPos(datum, spec, state, data.length);
            layout.set(datum.id, pos);
            maxX = Math.max(maxX, pos.x);
            maxY = Math.max(maxY, pos.y);
            // TODO: construct bounds by subdividing space
        }
        const w = bounds.width / (maxX || 1);
        const h = bounds.height / (maxY || 1);
        // add corners, so rescaling doesn't trim off any rows or columns
        if (spec.pattern !== "linear" && spec.pattern !== "spiral" && spec.pattern !== "circle") {
            layout.set("#TOP_LEFT_CORNER", { x: 0, y: 0, width: w, height: h });
            layout.set("#BOTTOM_RIGHT_CORNER", {
                x: w * ((_a = spec.numCols) !== null && _a !== void 0 ? _a : Math.ceil(Math.sqrt(data.length))),
                y: h * ((_b = spec.numRows) !== null && _b !== void 0 ? _b : Math.ceil(Math.sqrt(data.length))),
                width: w, height: h
            });
        }
        for (const datum of data) {
            layout.set(datum.id, {
                x: w * layout.get(datum.id).x,
                y: h * layout.get(datum.id).y,
                width: w,
                height: h
            });
        }
    }
    rescaleLayout(spec, state, data, layout, bounds, axes);
    return [layout, axes];
};
const computeLayout = (state, spec, bounds, addToState, funcs) => __awaiter(void 0, void 0, void 0, function* () {
    let layout;
    // N.B. checking if spec.type is in a list of topological layout types would be more concise, but confuse the TypeScript compiler
    if (("type" in spec) && spec.type === "function") {
        layout = funcs[spec.function](state, spec, bounds);
    }
    else if ("type" in spec && (spec.type === "d3-force" || spec.type === "webcola" || spec.type === "cytoscape" || spec.type === "tulip" || spec.type === "bioFabric" || spec.type === "setcola" || spec.type === "bipartite")) {
        layout = yield computeTopologicalLayout(state, spec, bounds, addToState);
    }
    else if ("map" in spec) {
        layout = computeMapLayout(state, spec, bounds);
    }
    else if ("source" in spec) {
        layout = computeDerivedLayout(state, spec, bounds, funcs);
    }
    else {
        layout = computeAttributeLayout(state, spec, bounds);
    }
    // If "source" is in spec, this is a derived layout, and the transforms were already applied to create this layout.
    // Otherwise, the transforms should be applied now.
    if ("transform" in spec && !("source" in spec)) {
        let l = layout[0];
        for (const layoutTransform of ((spec === null || spec === void 0 ? void 0 : spec.transform) || [])) {
            l = transformLayout(layoutTransform, spec, l, state, funcs); // layout is an array of [layout, axes]
        }
        layout[0] = l;
    }
    return layout;
});
export { computeLayout };
//# sourceMappingURL=layout.js.map