import { applyOrderOrScale, getTicks, scale } from "./utils";
export const layoutCartesianMultipleXScales = (data, spec, state) => {
    const layout = new Map();
    const axes = [];
    const processedAxes = [];
    // One kind of parallel coordinates plot
    for (const datum of data) {
        const ySpec = ("order" in spec.y[0]) ? { order: spec.y[0].order } : spec.y[0];
        const y = applyOrderOrScale(datum, ySpec, state);
        const xn = spec.x[y];
        const xSpec = ("order" in xn) ? { order: xn.order } : xn;
        const x = applyOrderOrScale(datum, xSpec, state);
        layout.set(datum.id, { x, y });
        if (!processedAxes.includes(xn)) {
            processedAxes.push(xn);
            axes.push({
                "type": "cartesian",
                "x": x,
                "x2": x,
                "y": 0,
                "y2": 0,
                "extend": "y",
                ticks: getTicks(ySpec, state).map(t => ({ x, y: t[0], label: t[1] }))
            });
        }
    }
    scale(layout, "x", "y", ("order" in spec.x[0]), spec.positioning);
    return [layout, axes];
};
export const layoutCartesianMultipleYScales = (data, spec, state) => {
    // One kind of parallel coordinates plot
    const layout = new Map();
    const axes = [];
    const processedAxes = [];
    for (const datum of data) {
        const xSpec = ("order" in spec.x[0]) ? { order: spec.x[0].order } : spec.x[0];
        const x = applyOrderOrScale(datum, xSpec, state);
        const yn = spec.y[x];
        const ySpec = ("order" in yn) ? { order: yn.order } : yn;
        const y = applyOrderOrScale(datum, ySpec, state);
        layout.set(datum.id, { x, y });
        if (!processedAxes.includes(yn)) {
            processedAxes.push(yn);
            axes.push({
                "type": "cartesian",
                "x": 0,
                "x2": 0,
                "y": y,
                "y2": y,
                "extend": "x",
                ticks: getTicks(ySpec, state).map(t => ({ x: t[0], y, label: t[1] }))
            });
        }
    }
    scale(layout, "y", "x", ("order" in spec.y[0]), spec.positioning);
    return [layout, axes];
};
export const layoutCartesian = (data, spec, state) => {
    // formerly the "cartesian" layout
    const layout = new Map();
    const axes = [];
    // this caused problems for one of the layouts - jigsaw
    /*
      const xSpec = ("order" in spec.x[0]) ? {  order: {...spec.x[0]} } : spec.x[0];
    const ySpec = ("order" in spec.y[0]) ? { order: {...spec.y[0]} } : spec.y[0];
  
     */
    // this caused problems for tables
    //  const xSpec = ("order" in spec.x[0]) ? { order: spec.x[0].order } : spec.x[0];
    //const ySpec = ("order" in spec.y[0]) ? { order: spec.y[0].order } : spec.y[0];
    // convert
    let xSpec;
    if ("order" in spec.x[0] && "field" in spec.x[0]) {
        xSpec = { order: { order: spec.x[0].order, field: spec.x[0].field } };
    }
    else if ("order" in spec.x[0]) {
        xSpec = { order: spec.x[0].order };
    }
    else {
        xSpec = spec.x[0];
    }
    let ySpec;
    if ("order" in spec.y[0] && "field" in spec.x[0]) {
        ySpec = { order: { order: spec.y[0].order, field: spec.y[0].field } };
    }
    else if ("order" in spec.y[0]) { // assumes this is a string
        ySpec = { order: spec.y[0].order };
    }
    else {
        ySpec = spec.y[0];
    }
    // const xSpec = {order: spec.x[0]};
    //  const ySpec = {spec.y[0]};
    for (const datum of data) {
        const x = applyOrderOrScale(datum, xSpec, state);
        const y = applyOrderOrScale(datum, ySpec, state);
        layout.set(datum.id, { x, y, width: 1, height: 1 });
    }
    const y0 = ("scale" in ySpec) ? state[ySpec.scale](0) : 0;
    axes.push({
        "type": "cartesian",
        "x": 0,
        "x2": 0,
        "y": y0,
        "y2": y0,
        "extend": "x",
        ticks: getTicks(xSpec, state).map(t => ({ x: t[0], y: y0, label: t[1] }))
    });
    const x0 = ("scale" in xSpec) ? state[xSpec.scale](0) : 0;
    axes.push({
        "type": "cartesian",
        "x": x0,
        "x2": x0,
        "y": 0,
        "y2": 0,
        "extend": "y",
        ticks: getTicks(ySpec, state).map(t => ({ x: x0, y: t[0], label: t[1] }))
    });
    return [layout, axes];
};
//# sourceMappingURL=layoutCartesian.js.map