import { aggregationFunctions } from "../data/aggregation";
const defaultAggregateLinks = (edgeData) => edgeData;
const aggregateEdges = (links, spec) => {
    var _a;
    let aggregateLinks = defaultAggregateLinks;
    let directed = false;
    if (spec) {
        aggregateLinks = spec.operation ? aggregationFunctions[spec.operation] : aggregateLinks;
        directed = (_a = spec.directed) !== null && _a !== void 0 ? _a : directed;
    }
    const groupedLinks = new Map();
    for (const link of links) {
        let sourceId = link.source.id;
        let targetId = link.target.id;
        if (!directed) {
            const s = (sourceId <= targetId) ? sourceId : targetId;
            const t = (sourceId <= targetId) ? targetId : sourceId;
            sourceId = s;
            targetId = t;
        }
        if (!groupedLinks.has(sourceId)) {
            groupedLinks.set(sourceId, new Map());
        }
        if (!groupedLinks.get(sourceId).has(targetId)) {
            groupedLinks.get(sourceId).set(targetId, []);
        }
        groupedLinks.get(sourceId).get(targetId).push(link);
    }
    const outputLinks = [];
    for (const [, linksWithSource] of groupedLinks) {
        if (!linksWithSource) {
            continue;
        }
        for (const [, linksWithSourceAndTarget] of linksWithSource) {
            if (!linksWithSourceAndTarget) {
                continue;
            }
            outputLinks.push({
                source: linksWithSourceAndTarget[0].source,
                target: linksWithSourceAndTarget[0].target,
                id: linksWithSourceAndTarget[0].id,
                data: aggregateLinks(linksWithSourceAndTarget.map(link => link.data))
            });
        }
    }
    return outputLinks;
};
export { aggregateEdges };
//# sourceMappingURL=aggregateEdges.js.map