import { select } from "../data/select";
import { aggregationFunctions } from "../data/aggregation";
const computeConnectivityBasedAttribute = (nodes, links, spec) => {
    // N.B. selector can refer to either node or link data
    var _a, _b;
    const attributeSelector = spec.attributeSelector;
    const direction = (_a = spec.direction) !== null && _a !== void 0 ? _a : "both";
    const aggregationOperation = (_b = spec.operation) !== null && _b !== void 0 ? _b : "count";
    const computedAttributeName = spec.as;
    const values = new Map();
    for (const link of links) {
        if (direction !== "out") {
            const nodeWithData = link.source;
            const otherNode = link.target;
            const value = select({ link, node: nodeWithData }, attributeSelector);
            if (!values.has(otherNode)) {
                values.set(otherNode, []);
            }
            values.get(otherNode).push(value);
        }
        if (direction !== "in") {
            const nodeWithData = link.target;
            const otherNode = link.source;
            const value = select({ link, node: nodeWithData }, attributeSelector);
            if (!values.has(otherNode)) {
                values.set(otherNode, []);
            }
            values.get(otherNode).push(value);
        }
    }
    const aggregationFn = aggregationFunctions[aggregationOperation];
    for (const node of nodes) {
        if (typeof (node.data) === "undefined") {
            node.data = {};
        }
        node.data[computedAttributeName] = aggregationFn(values.get(node));
    }
};
export { computeConnectivityBasedAttribute };
//# sourceMappingURL=connectivityBasedAttribute.js.map