import { select } from "../../data/select";
import { calculateMetrics } from "../metrics";
const constructNetworks = (data, specs) => {
    for (const spec of specs) {
        constructNetwork(data, spec);
    }
};
// TODO: guess names of nodes/links fields if not specified
// TODO: allow linking on implicit index (rather than an explicit id)
const constructFromLinkTableOnly = (data, spec) => {
    var _a, _b;
    const links = JSON.parse(JSON.stringify(select(data, spec.links)));
    const reverse_links = [];
    const nodes = [];
    const node_with_id = new Map();
    for (const link of links) {
        const id_of_source = select(link, spec.source_node[1]);
        const id_of_target = select(link, spec.target_node[1]);
        if (!node_with_id.has(id_of_source)) {
            const node = { id: id_of_source };
            nodes.push(node);
            node_with_id.set(id_of_source, node);
        }
        link.source = node_with_id.get(id_of_source);
        if (!node_with_id.has(id_of_target)) {
            const node = { id: id_of_target };
            nodes.push(node);
            node_with_id.set(id_of_target, node);
        }
        link.target = node_with_id.get(id_of_target);
        link.directed = (_a = spec.directed) !== null && _a !== void 0 ? _a : true;
        if (spec.addReverseLinks) {
            reverse_links.push(Object.assign(Object.assign({}, link), { source: link.target, target: link.source, id: link.id + "_R", directed: (_b = spec.directed) !== null && _b !== void 0 ? _b : true }));
        }
    }
    if (spec.metrics) {
        calculateMetrics(links, nodes, spec.metrics);
    }
    let linksToAdd;
    if ("ignoreDanglingLinks" in spec && !spec.ignoreDanglingLinks) {
        linksToAdd = [...links, ...reverse_links];
    }
    else {
        linksToAdd = [...links, ...reverse_links].filter(l => typeof l.source !== "undefined" && typeof l.target !== "undefined");
    }
    return { nodes, links: linksToAdd };
};
const constructNetwork = (data, spec) => {
    var _a, _b;
    if (!("nodes" in spec)) {
        return constructFromLinkTableOnly(data, spec);
    }
    const links = JSON.parse(JSON.stringify(select(data, spec.links)));
    const nodes = JSON.parse(JSON.stringify(select(data, spec.nodes)));
    // TODO: handle cases where there is not exactly one match
    const reverse_links = [];
    const node_with_id = new Map();
    for (const n of nodes) {
        node_with_id.set(select(n, spec.source_node[0]), n);
    }
    for (const link of links) {
        const id_of_source = select(link, spec.source_node[1]);
        const id_of_target = select(link, spec.target_node[1]);
        link.source = node_with_id.get(id_of_source);
        link.target = node_with_id.get(id_of_target);
        link.directed = (_a = spec.directed) !== null && _a !== void 0 ? _a : true;
        if (spec.addReverseLinks) {
            reverse_links.push(Object.assign(Object.assign({}, link), { source: link.target, target: link.source, id: link.id + "_R", directed: (_b = spec.directed) !== null && _b !== void 0 ? _b : true }));
        }
    }
    if (spec.metrics) {
        calculateMetrics(links, nodes, spec.metrics);
    }
    let linksToAdd;
    if ("ignoreDanglingLinks" in spec && !spec.ignoreDanglingLinks) {
        linksToAdd = [...links, ...reverse_links];
    }
    else {
        linksToAdd = [...links, ...reverse_links].filter(l => typeof l.source !== "undefined" && typeof l.target !== "undefined");
    }
    return { nodes, links: linksToAdd };
};
export { constructNetwork, constructNetworks };
//# sourceMappingURL=constructNetwork.js.map