import { aggregateEdges } from "./aggregateEdges";
import { calculateClusters } from "./cluster";
import { computeConnectivityBasedAttribute } from "./connectivityBasedAttribute";
import { calculateMetrics } from "./metrics";
import { projectNetwork } from "./project";
import { calculateCalculate } from "../data/calculate";
import { removeIsolatedNodes } from "./removeIsolatedNodes";
import { filterEdges } from "./filterEdges";
import { filterNodes } from "./filterNodes";
import { aggregateNodes } from "./aggregateNodes";
import { connectNodes } from "./connectNodes";
import { swapNodesEdges } from "./swapNodesEdges";
import { reverseEdges } from "./reverseEdges";
import { setDirectedness } from "./setDirectedness";
import { promoteField } from "./promoteField";
const applyNetworkTransform = (network, spec, state) => {
    const nodes = network.nodes;
    const links = network.links;
    // these transforms modify in-place, rather than returning modified network object
    if (spec.type === "metric") {
        calculateMetrics(links, nodes, [spec]);
    }
    else if (spec.type === "cluster") {
        calculateClusters(links, nodes, spec);
    }
    else if (spec.type === "aggregateEdges") {
        network.links = aggregateEdges(links, spec);
    }
    else if (spec.type === "connectivityBasedAttribute") {
        computeConnectivityBasedAttribute(nodes, links, spec);
    }
    else if (spec.type === "project") {
        const { nodes: n, links: l } = projectNetwork(nodes, links, spec);
        network.nodes = n;
        network.links = l;
    }
    else if (spec.type === "calculate") {
        if (spec.for === "links") {
            network.links = calculateCalculate(links, spec);
        }
        else {
            network.nodes = calculateCalculate(nodes, spec);
        }
    }
    else if (spec.type === "removeIsolated") {
        removeIsolatedNodes(network);
    }
    else if (spec.type === "filterNodes") {
        filterNodes(network, spec, state);
    }
    else if (spec.type === "filterEdges") {
        network.links = filterEdges(links, spec, state);
    }
    else if (spec.type === "aggregateNodes") {
        aggregateNodes(network, spec);
    }
    else if (spec.type === "connect") {
        network.links = connectNodes(nodes, links, spec, state);
    }
    else if (spec.type === "swapNodesAndEdges") {
        const { nodes: n, links: l } = swapNodesEdges(nodes, links, spec);
        network.nodes = n;
        network.links = l;
    }
    else if (spec.type === "reverseEdges") {
        network.links = reverseEdges(nodes, links, spec, state);
    }
    else if (spec.type === "setDirectness") {
        network.links = setDirectedness(nodes, links, spec, state);
    }
    else if (spec.type === "promote") {
        const { nodes: n, links: l } = promoteField(nodes, links, spec, state);
        network.nodes = n;
        network.links = l;
    }
};
export { applyNetworkTransform };
//# sourceMappingURL=transform.js.map