// compute an ordering based on an attribute or expression
import { select } from "../data/select";
import { constructExpressionFunction } from "../data/calculate";
const generateCompareFn = (criteria) => {
    return (a, b) => {
        for (const criterion of criteria) {
            const direction = criterion.direction;
            let aVal;
            let bVal;
            if ("expression" in criterion) {
                const fn = criterion.fn;
                aVal = fn(a);
                bVal = fn(b);
            }
            else {
                aVal = select(a, criterion.field);
                bVal = select(b, criterion.field);
            }
            if (aVal > bVal) {
                return direction === "descending" ? -1 : +1;
            }
            else if (aVal < bVal) {
                return direction === "descending" ? +1 : -1;
            }
        }
        return 0;
    };
};
export const getCompareFn = (orderBy) => {
    // ensure spec is an array of sorting criteria
    if (typeof (orderBy) === "string" || !("length" in orderBy)) {
        orderBy = [orderBy];
    }
    for (let i = 0; i < orderBy.length; i++) {
        const criterion = orderBy[i];
        if (typeof (criterion) === "string") {
            orderBy[i] = { field: criterion };
        }
        else if ("expression" in criterion && criterion.expression) {
            criterion.fn = constructExpressionFunction({ calculate: criterion.expression });
        }
    }
    return generateCompareFn(orderBy);
};
const attributeOrdering = (data, spec) => {
    const compareFn = getCompareFn(spec.orderBy);
    const toSort = data.sort(compareFn);
    const order = new Map();
    if (spec.allowTies) {
        let distinctVals = 0;
        for (let i = 0; i < toSort.length; i++) {
            const different = (i === 0) || (compareFn(toSort[i], toSort[i - 1]) !== 0);
            if ((i > 0) && different) {
                distinctVals++;
            }
            order.set(toSort[i].id, distinctVals);
        }
    }
    else {
        for (const i in toSort) {
            order.set(toSort[i].id, +i);
        }
    }
    return order;
};
export { attributeOrdering };
//# sourceMappingURL=attributeOrdering.js.map