import { attributeOrdering } from "./attributeOrdering";
import { constructSeriation } from "./seriation";
import { select } from "../data/select";
const constructOrdering = (state, config) => {
    if ("orderBy" in config) {
        const data = select(state, config.data);
        return attributeOrdering(data, config);
    }
    else {
        const network = select(state, config.network);
        // no longer assume that ids are sequential and start at 0
        const indexOfId = new Map();
        const idOfIndex = new Map();
        for (const i in network.nodes) {
            const node = network.nodes[i];
            indexOfId.set(node.id, i);
            idOfIndex.set(+i, node.id);
        }
        const adjMat = [];
        for (const node1 of network.nodes) {
            const i = indexOfId.get(node1.id);
            adjMat[i] = [];
            for (const node2 of network.nodes) {
                const j = indexOfId.get(node2.id);
                adjMat[i][j] = 0;
            }
        }
        for (const link of network.links) {
            const i = indexOfId.get(link.source.id);
            const j = indexOfId.get(link.target.id);
            const weightField = ("weightField" in config) ? config.weightField : "value";
            const weight = select(link, weightField);
            adjMat[i][j] += weight;
            adjMat[j][i] += weight;
            adjMat[i][i] += weight;
            adjMat[j][j] += weight;
        }
        let method = config.method;
        if (typeof (method) === "object" && "parameter" in method) {
            method = state[method.parameter];
        }
        const seriation = constructSeriation(adjMat, Object.assign(Object.assign({}, config), { method }));
        const ordering = new Map();
        for (const ind of seriation.keys()) {
            ordering.set(idOfIndex.get(ind), seriation.get(ind));
        }
        return ordering;
    }
};
export { constructOrdering };
//# sourceMappingURL=ordering.js.map