// by field, sorted alphabetically or numerically
import * as reorder from "reorder.js";
function inverse_permutation(perm) {
    const inv = new Map();
    for (let i = 0; i < perm.length; i++) {
        inv.set(perm[i], i);
    }
    return inv;
}
const constructSeriation = (mat, config) => {
    // These methods assume we are acting on anetwork
    // TODO: construct matrix properly
    // TODO: rather than permutation, return map from id to position
    // (the i'th element of permutation is the array undex of element in original list that will be in i'th place of result)
    // reorder.inverse_permutation() should do this
    const distances = ["euclidean", "manhattan", "minkowski", "chebyshev", "hamming", "jaccard", "braycurtis"];
    if (config.method === "optimal-leaf-order") {
        if (distances.includes(config.distance)) {
            // @ts-ignore
            const graph = reorder.mat2graph(mat);
            // @ts-ignore
            const dist_adjacency = reorder.graph2valuemats(graph);
            const leafOrder = reorder.optimal_leaf_order()
                .distance(reorder.distance.manhattan);
            // @ts-ignore
            const permutation = reorder.valuemats_reorder(dist_adjacency, leafOrder);
            //   console.log("in construct order:", permutation)
            // @ts-ignore
            return inverse_permutation(permutation);
        }
        else {
            // @ts-ignore
            const permutation = reorder.optimal_leaf_order()(mat);
            //                console.log("in construct order:", permutation)
            // @ts-ignore
            //return permutation;
            return inverse_permutation(permutation);
        }
        // var permuted_mat = reorder.stablepermute(mat, perm);
    }
    else if (config.method === "barycentre") {
        // barycenter/crossing reduction
        // @ts-ignore
        const graph = reorder.mat2graph(mat);
        // @ts-ignore
        const barycenter = reorder.barycenter_order(graph);
        // @ts-ignore
        const improved = reorder.adjacent_exchange(graph, barycenter[0], barycenter[1]);
        const perm = barycenter[0];
        // @ts-ignore
        return inverse_permutation(perm);
    }
    else if (config.method === "bandwidth-reduction") {
        // bandwidth reduction/rcm
        // @ts-ignore
        const graph = reorder.mat2graph(mat);
        // @ts-ignore
        const perm = reorder.reverse_cuthill_mckee_order(graph);
        // @ts-ignore
        return inverse_permutation(perm);
    }
    else if (config.method === "spectral") {
        // @ts-ignore
        const graph = reorder.mat2graph(mat);
        // @ts-ignore
        const perm = reorder.spectral_order(graph);
        // @ts-ignore
        return inverse_permutation(perm);
    }
    else if (config.method === "pca") {
        // @ts-ignore
        const perm = reorder.pca_order(mat);
        // @ts-ignore
        return inverse_permutation(perm);
        // permuted_mat = reorder.stablepermute(mat, perm);
        //  return perm;
    }
};
export { constructSeriation };
// Object.keys(reorder.distances)
//const reorder = require("reorder.js")
//leaf order over distance matrix
// valuemats
/*
    reorder.valuemats_reorder - reorder an undirected graph using its value matrix and a matrix ordering algorithm.
    reorder.barycenter_order - reorder a graph using the barycenter heuristic.
        reorder.adjacent_exchanges - improves a barycenter_order by swapping adjacent vertices when that decreases the number of crossings.
    reorder.bfs_order - reorder an undirected graph using a Breadth-First order.
    reorder.cuthill_mckee_order - reorder an undirected graph using the Cuthill-McKee bandwith reduction algorithm (bad).
    reorder.reverse_cuthill_mckee - reorder an undirected graph using the Reverse Cuthill-McKee bandwith reduction algorithm (bad).

    reorder.spectral_order - reorder a graph according to its Fiedler vector.

    reorder.optimal_leaf_order - reorder a table using the Optimal Leaf Ordering algorithm.
        optimal_leaf_order.distance - get or set the distance function for the optimal leaf ordering algorithm.
        optimal_leaf_order.linkage - get or set the linkage for the hierarchical clustering used by the optimal leaf ordering algorithm.
        optimal_leaf_order.distance_matrix - get of set the distance matrix used for computing the optimal leaf order.
        reorder.order - Manipulation and control of optimal leaf ordering parameters to interact with the reordering.
        order.distance - get or set the distance function for the optimal leaf ordering.
        order.linkage - get or set the linkage for the hierarchical clustering used by the optimal leaf ordering algorithm.
        order.limits - get or set the min/max indices of the matrix manipulated.
        order.except - get or set ranges of indices the ordering algorithm should not change.
    reorder.pca_order - reorder a matrix using PCA.
    reorder.sort_order - reorder a matrix according to a sort function.

   const g = reorder.mat2graph(mat)
 */ 
//# sourceMappingURL=seriation.js.map