// This is based on the scale() function in vega-encode/test/scale-test.js
// vega-scale doesn't accept the JSON scale spec; this has to be handled by vega-encode instead
// @ts-ignore
import { scale } from "vega-encode";
// @ts-ignore
import { Dataflow } from "vega-dataflow";
import { extent } from "d3-array";
import { constructExpressionFunction } from "./data/calculate";
import { select } from "./data/select";
const constructScale = (config, bounds, state) => {
    let domain;
    if (typeof config.domain === "object" && "data" in config.domain) {
        const dataTable = select(state, config.domain.data);
        const values = dataTable.map(d => select(d, config.domain.field));
        const discreteScales = ["ordinal", "band", "point"];
        if (discreteScales.includes(config.type)) {
            domain = values;
        }
        else {
            domain = extent(values);
        }
    }
    if (bounds && "rangeExpressions" in config) {
        const range = [];
        for (const i in config.rangeExpressions) {
            const expr = config.rangeExpressions[i];
            if (typeof expr === "string") {
                const fn = constructExpressionFunction({ calculate: expr });
                range.push(fn({}, +i, state || {}, bounds));
            }
            else {
                range.push(expr);
            }
        }
        config = Object.assign(Object.assign({}, config), { range });
    }
    else if (bounds && "rangeMapFunction" in config) {
        const fn = constructExpressionFunction({ calculate: config.rangeMapFunction });
        const range = config.range.map((d, i) => fn(d, i, state, bounds));
        config = Object.assign(Object.assign({}, config), { range });
    }
    else if (bounds && ("domain" in config)) {
        for (const i in config.range) {
            const val = config.range[i];
            if (["width", "height", "x", "y"].includes(val)) {
                config.range[i] = bounds[val];
            }
            if ("area" === val) {
                config.range[i] = bounds.width * bounds.height;
            }
        }
    }
    const df = new Dataflow();
    const s = df.add(scale, Object.assign(Object.assign({}, config), { domain: domain !== null && domain !== void 0 ? domain : config.domain }));
    let e = false;
    df.error = ((_) => e = _);
    df.run();
    return e ? "ERROR" : s.value;
};
export { constructScale };
//# sourceMappingURL=scale.js.map