export const constructAdjacencyList = (nodes, links, treatAllLinksAsUndirected) => {
    const inLinks = new Map();
    const outLinks = new Map();
    const undirectedLinks = new Map();
    for (const link of links) {
        const source_id = link.source.id;
        const target_id = link.target.id;
        if (!inLinks.has(source_id)) {
            inLinks.set(source_id, []);
            outLinks.set(source_id, []);
            undirectedLinks.set(source_id, []);
        }
        if (!inLinks.has(target_id)) {
            inLinks.set(target_id, []);
            outLinks.set(target_id, []);
            undirectedLinks.set(target_id, []);
        }
        if (link.directed && !treatAllLinksAsUndirected) {
            if (!inLinks.has(target_id)) {
                inLinks.set(target_id, []);
            }
            inLinks.get(target_id).push(link);
            if (!outLinks.has(source_id)) {
                outLinks.set(source_id, []);
            }
            outLinks.get(source_id).push(link);
        }
        else {
            if (!undirectedLinks.has(source_id)) {
                undirectedLinks.set(source_id, []);
            }
            undirectedLinks.get(source_id).push(link);
            if (!undirectedLinks.has(target_id)) {
                undirectedLinks.set(target_id, []);
            }
            // don't add self-links twice
            if (source_id !== target_id) {
                undirectedLinks.get(target_id).push(link);
            }
        }
    }
    return { inLinks, outLinks, undirectedLinks };
};
//# sourceMappingURL=constructAdjacencyList.js.map