import { select } from "../data/select";
const shortestPath = (state, selection, adjacency, spec) => {
    const network = state[selection.network];
    const nodes = network.nodes;
    if (selection.nodes.length < 2) {
        console.log("Can't compute path, as fewer than 2 nodes selected");
        return { nodes: [], links: [] };
    }
    const startNodeId = selection.nodes[1].id;
    const endNodeId = selection.nodes[0].id;
    let weightFunc; // function from link object to weight
    if (!("weight" in spec)) {
        weightFunc = () => 1;
    }
    else {
        weightFunc = (link) => select(link, spec.weight);
    }
    const dist = [];
    const prev = [];
    const edge = [];
    let Q = [];
    for (const n of nodes) {
        dist[n.id] = Infinity;
        prev[n.id] = undefined;
        Q.push(n.id);
    }
    dist[startNodeId] = 0;
    while (Q.length > 0) {
        // find vertex in Q with min distance
        let minVal = Infinity;
        let minId = undefined;
        for (const id of Q) {
            if (dist[id] < minVal) {
                minId = id;
                minVal = dist[id];
            }
        }
        Q = Q.filter(id => id !== minId);
        const u = minId;
        // check if we can terminate early
        if (u === endNodeId) {
            break;
        }
        // iterate over neighbours of v still in u
        // reverse direction of unordered links if necessary
        const ul = adjacency.undirectedLinks.get(minId).map(l => {
            if (l.target.id === minId) {
                return l;
            }
            else {
                return Object.assign(Object.assign({}, l), { target: l.source, source: l.target });
            }
        });
        const links = [...adjacency.inLinks.get(minId), ...ul];
        for (const l of links) {
            const v = l.source.id;
            if (!Q.includes(v)) {
                continue;
            }
            const alt = dist[u] + weightFunc(l);
            if (alt < dist[v]) {
                dist[v] = alt;
                prev[v] = u;
                edge[v] = l;
            }
        }
    }
    const pathNodes = [];
    const pathEdges = [];
    let u = endNodeId;
    if (typeof prev[u] !== "undefined") {
        while (typeof u !== "undefined") {
            pathNodes.push(nodes[u]); // TODO: check this goes from node ID to node object
            if (edge[u]) {
                pathEdges.push(edge[u]);
            }
            u = prev[u];
        }
    }
    return { nodes: pathNodes, links: pathEdges };
};
export { shortestPath };
//# sourceMappingURL=shortestPath.js.map