import cytoscape from 'cytoscape';
// @ts-ignore
import avsdf from 'cytoscape-avsdf';
import dagre from 'cytoscape-dagre';
// @ts-ignore
//import elk from 'cytoscape-elk';
// @ts-ignore
import klay from 'cytoscape-klay';
// import fcose from 'cytoscape-fcose';
// @ts-ignore
import coseBilkent from 'cytoscape-cose-bilkent';
// @ts-ignore
import cola from 'cytoscape-cola';
// @ts-ignore
import cise from 'cytoscape-cise';
// @ts-ignore
import euler from 'cytoscape-euler';
// @ts-ignore
import spread from 'cytoscape-spread';
import { constructExpressionFunction } from "../data/calculate";
const functionOptions = {
    "concentric": ["concentric", "levelWidth"],
    "cose": ["nodeRepulsion", "idealEdgeLength", "edgeElasticity"],
    "dagre": ["minLen", "edgeWeight"],
    "fcose": ["nodeRepulsion", "idealEdgeLength", "edgeElasticity", "tilingPaddingVertical", "tilingPaddingHorizontal"],
    "euler": ["springLength", "springCoeff", "mass"],
    "Klay": ["priority"],
    "cose-bilkent": ["tilingPaddingVertical", "tilingPaddingHorizontal"],
};
export const getVal = (valSpec) => {
    // Some parameters for a force may be accessor functions of either the nodes or links,
    // rather than numbers that are th same for all nodes/links
    //  N.B. expression should refer to "datum"
    if (typeof (valSpec) === "number") {
        return () => valSpec;
    }
    else if ("expression" in valSpec) {
        return constructExpressionFunction({ calculate: valSpec.expression });
    }
    else if ("field" in valSpec) {
        return (n) => n[valSpec.field];
    }
};
const layoutCytoscape = (nodes, links, spec) => {
    const elements = [
        ...nodes.map((n) => ({ data: n })),
        ...links.map((n) => ({ data: Object.assign(Object.assign({}, n), { source: n.source.id, target: n.target.id }) }))
    ];
    if (spec.algorithm in functionOptions) {
        for (const option of functionOptions[spec.algorithm]) {
            if (option in spec) {
                if (typeof (spec[option]) === "object" && "expression" in spec[option]) {
                    // This is a potentially unsafe hack
                    // the functions need to access node data with calls like `node.data('fieldName')`, which the vega expression parser doesn't allow
                    spec[option] = eval("(n) =>" + spec[option].expression);
                }
                else {
                    spec[option] = getVal(spec[option]);
                }
            }
        }
    }
    cytoscape.use(avsdf);
    cytoscape.use(dagre);
    // cytoscape.use(elk);
    cytoscape.use(klay);
    // cytoscape.use(fcose)
    cytoscape.use(coseBilkent);
    cytoscape.use(cola);
    cytoscape.use(cise);
    cytoscape.use(euler);
    cytoscape.use(spread);
    const cy = cytoscape({
        elements
    });
    spec.animate = false;
    // the grid is the default within the Cytoscape app:
    // "the default layout and is always available as part of the Cytoscape core"
    // https://manual.cytoscape.org/en/stable/Navigation_and_Layout.html
    cy.layout(Object.assign(Object.assign({}, spec), { name: spec.algorithm || 'grid' })).run();
    return cy.nodes().map((n) => n._private.position);
};
export { layoutCytoscape };
//# sourceMappingURL=layoutCytoscape.js.map