import { convertVal } from "../complete";
import { max } from "d3-array";
import { select } from "../data/select";
const setTextFieldsOnItem = (sceneItem, spec, datum, state) => {
    for (const fieldName in spec) {
        if (!["type", "type", "actions"].includes(fieldName)) {
            sceneItem[fieldName] = convertVal(spec[fieldName], datum, state);
        }
    }
    if ("actions" in spec) {
        sceneItem.actions = spec.actions;
    }
};
export const computeTableInduceLayout = (distinctRows, distinctCols, state, tableName, spec, rowGroupIds, colGroupIds, layout, extra, bounds) => {
    if ("inducedLayout" in spec) {
        const inducedLayoutName = spec.inducedLayout.name;
        if (!Object.keys(extra).includes("INDUCED_LAYOUTS")) {
            extra.INDUCED_LAYOUTS = [];
        }
        if (!extra.INDUCED_LAYOUTS.includes(inducedLayoutName)) {
            extra.INDUCED_LAYOUTS.push(inducedLayoutName);
        }
        if (!Object.keys(extra).includes(inducedLayoutName)) {
            extra[inducedLayoutName] = new Map();
        }
        if (spec.inducedLayout.rowId) {
            for (const i in distinctRows) {
                const idForLayout = rowGroupIds[i];
                const d = state[tableName + "#grouping"][idForLayout].data[0];
                const xOffset = spec.inducedLayout.x ? convertVal(spec.inducedLayout.x, {}, state, 0, bounds) : 0;
                const yOffset = spec.inducedLayout.y ? convertVal(spec.inducedLayout.y, {}, state, 0, bounds) : 0;
                const idToSave = select(d, spec.inducedLayout.rowId);
                extra[inducedLayoutName].set(idToSave, {
                    x: bounds.x + xOffset,
                    y: bounds.y + layout.get(idForLayout).y + yOffset
                });
            }
        }
        if (spec.inducedLayout.colId) {
            for (const i in distinctCols) {
                const idForLayout = colGroupIds[i];
                const d = state[tableName + "#grouping"][idForLayout].data[0];
                const xOffset = spec.inducedLayout.x ? convertVal(spec.inducedLayout.x, {}, state, 0, bounds) : 0;
                const yOffset = spec.inducedLayout.y ? convertVal(spec.inducedLayout.y, {}, state, 0, bounds) : 0;
                const idToSave = select(d, spec.inducedLayout.colId);
                extra[inducedLayoutName].set(idToSave, {
                    x: bounds.x + layout.get(idForLayout).x + xOffset,
                    y: bounds.y + yOffset
                });
            }
        }
    }
};
export const labelTable = (distinctRows, distinctCols, state, tableName, spec, rowGroupIds, colGroupIds, layout, extra) => {
    var _a, _b;
    const items = [];
    if ("rowLabels" in spec) {
        for (const i in distinctRows) {
            const row = distinctRows[i];
            const idForLayout = rowGroupIds[i];
            // layout = state[tableName + "#layout"];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const itemBase = {
                fill: "black",
                align: "left",
                text: convertVal(spec.rowLabels, d, state),
                x: 0,
                y: layout.get(idForLayout).y + layout.get(idForLayout).height / 2,
                angle: 0,
                limit: (_a = spec.rowLabels.width) !== null && _a !== void 0 ? _a : 100
            };
            const itemDraggable = {
                // these fields are needed for the drag-to-reorder interaction
                idForLayout: distinctRows[i].id,
                layoutName: spec.layout,
                role: "rowLabel"
            };
            const item = spec.dragToReorder ? Object.assign(Object.assign({}, itemBase), itemDraggable) : itemBase;
            setTextFieldsOnItem(item, spec.rowLabels, d, state);
            items.push(item);
        }
    }
    if ("colLabels" in spec) {
        for (const i in distinctCols) {
            const col = distinctCols[i];
            const idForLayout = colGroupIds[i];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const itemBase = {
                fill: "black",
                text: convertVal(spec.colLabels, d, state),
                x: layout.get(idForLayout).x + layout.get(idForLayout).width / 2,
                y: 0,
                angle: 270,
                limit: (_b = spec.colLabels.width) !== null && _b !== void 0 ? _b : 100,
            };
            const itemDraggable = {
                // these fields are needed for the drag-to-reorder interaction
                idForLayout: distinctCols[i].id,
                layoutName: spec.layout,
                role: "colLabel"
            };
            const item = spec.dragToReorder ? Object.assign(Object.assign({}, itemBase), itemDraggable) : itemBase;
            setTextFieldsOnItem(item, spec.colLabels, d, state);
            items.push(item);
        }
    }
    const ruleItems = [];
    const width = max(distinctCols.map((r, i) => layout.get(colGroupIds[i]).x + layout.get(colGroupIds[i]).width));
    const height = max(distinctRows.map((r, i) => layout.get(rowGroupIds[i]).y + layout.get(rowGroupIds[i]).height));
    if ("rowLines" in spec) {
        const lineTop = {
            stroke: "black",
            x: 0,
            x2: width,
            y: height,
            y2: height
        };
        setTextFieldsOnItem(lineTop, spec.rowLines, {}, state);
        ruleItems.push(lineTop);
        for (const i in distinctRows) {
            const idForLayout = rowGroupIds[i];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const lineBottom = {
                stroke: "black",
                x: 0,
                x2: width,
                y: layout.get(idForLayout).y,
                y2: layout.get(idForLayout).y //+ layout.get(idForLayout).height
            };
            setTextFieldsOnItem(lineBottom, spec.rowLines, d, state);
            ruleItems.push(lineBottom);
        }
    }
    if ("rowMidLines" in spec) {
        for (const i in distinctRows) {
            const idForLayout = rowGroupIds[i];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const lineBottom = {
                stroke: "black",
                x: 0,
                x2: width,
                y: layout.get(idForLayout).y + layout.get(idForLayout).height / 2,
                y2: layout.get(idForLayout).y + layout.get(idForLayout).height / 2
            };
            setTextFieldsOnItem(lineBottom, spec.rowMidLines, d, state);
            ruleItems.push(lineBottom);
        }
    }
    if ("colLines" in spec) {
        const lineLeft = {
            stroke: "black",
            x: 0,
            x2: 0,
            y: 0,
            y2: height
        };
        setTextFieldsOnItem(lineLeft, spec.colLines, {}, state);
        ruleItems.push(lineLeft);
        for (const i in distinctCols) {
            const idForLayout = colGroupIds[i];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const lineRight = {
                stroke: "black",
                x: layout.get(idForLayout).x + layout.get(idForLayout).width,
                x2: layout.get(idForLayout).x + layout.get(idForLayout).width,
                y: 0,
                y2: height
            };
            setTextFieldsOnItem(lineRight, spec.colLines, d, state);
            ruleItems.push(lineRight);
        }
    }
    if ("colMidLines" in spec) {
        for (const i in distinctCols) {
            const idForLayout = colGroupIds[i];
            const d = state[tableName + "#grouping"][idForLayout].data[0];
            const lineRight = {
                stroke: "black",
                x: layout.get(idForLayout).x + layout.get(idForLayout).width / 2,
                x2: layout.get(idForLayout).x + layout.get(idForLayout).width / 2,
                y: 0,
                y2: height
            };
            setTextFieldsOnItem(lineRight, spec.colMidLines, d, state);
            ruleItems.push(lineRight);
        }
    }
    return [
        { marktype: "text", items },
        { marktype: "rule", items: ruleItems }
    ];
};
//# sourceMappingURL=labelTable.js.map