import { processFullSpec } from "./complete";
import { renderSceneJSONToString } from "./vis/renderSceneToString";

const fs = require("fs");

test("handles les-mis example", () => {
  const spec = {
    data: [
      {
        "name": "nodes",
        "values": [
          {
            "name": "Myriel",
            "group": 1,
            "id": 0
          },
          {
            "name": "Napoleon",
            "group": 1,
            "id": 1
          },
          {
            "name": "Mlle.Baptistine",
            "group": 1,
            "id": 2
          },
          {
            "name": "Mme.Magloire",
            "group": 1,
            "id": 3
          },
          {
            "name": "CountessdeLo",
            "group": 1,
            "id": 4
          },
          {
            "name": "Geborand",
            "group": 1,
            "id": 5
          },
          {
            "name": "Champtercier",
            "group": 1,
            "id": 6
          },
          {
            "name": "Cravatte",
            "group": 1,
            "id": 7
          },
          {
            "name": "Count",
            "group": 1,
            "id": 8
          },
          {
            "name": "OldMan",
            "group": 1,
            "id": 9
          },
          {
            "name": "Labarre",
            "group": 2,
            "id": 10
          },
          {
            "name": "Valjean",
            "group": 2,
            "id": 11
          },
          {
            "name": "Marguerite",
            "group": 3,
            "id": 12
          },
          {
            "name": "Mme.deR",
            "group": 2,
            "id": 13
          },
          {
            "name": "Isabeau",
            "group": 2,
            "id": 14
          },
          {
            "name": "Gervais",
            "group": 2,
            "id": 15
          },
          {
            "name": "Tholomyes",
            "group": 3,
            "id": 16
          },
          {
            "name": "Listolier",
            "group": 3,
            "id": 17
          },
          {
            "name": "Fameuil",
            "group": 3,
            "id": 18
          },
          {
            "name": "Blacheville",
            "group": 3,
            "id": 19
          },
          {
            "name": "Favourite",
            "group": 3,
            "id": 20
          },
          {
            "name": "Dahlia",
            "group": 3,
            "id": 21
          },
          {
            "name": "Zephine",
            "group": 3,
            "id": 22
          },
          {
            "name": "Fantine",
            "group": 3,
            "id": 23
          },
          {
            "name": "Mme.Thenardier",
            "group": 4,
            "id": 24
          },
          {
            "name": "Thenardier",
            "group": 4,
            "id": 25
          },
          {
            "name": "Cosette",
            "group": 5,
            "id": 26
          },
          {
            "name": "Javert",
            "group": 4,
            "id": 27
          },
          {
            "name": "Fauchelevent",
            "group": 0,
            "id": 28
          },
          {
            "name": "Bamatabois",
            "group": 2,
            "id": 29
          },
          {
            "name": "Perpetue",
            "group": 3,
            "id": 30
          },
          {
            "name": "Simplice",
            "group": 2,
            "id": 31
          },
          {
            "name": "Scaufflaire",
            "group": 2,
            "id": 32
          },
          {
            "name": "Woman1",
            "group": 2,
            "id": 33
          },
          {
            "name": "Judge",
            "group": 2,
            "id": 34
          },
          {
            "name": "Champmathieu",
            "group": 2,
            "id": 35
          },
          {
            "name": "Brevet",
            "group": 2,
            "id": 36
          },
          {
            "name": "Chenildieu",
            "group": 2,
            "id": 37
          },
          {
            "name": "Cochepaille",
            "group": 2,
            "id": 38
          },
          {
            "name": "Pontmercy",
            "group": 4,
            "id": 39
          },
          {
            "name": "Boulatruelle",
            "group": 6,
            "id": 40
          },
          {
            "name": "Eponine",
            "group": 4,
            "id": 41
          },
          {
            "name": "Anzelma",
            "group": 4,
            "id": 42
          },
          {
            "name": "Woman2",
            "group": 5,
            "id": 43
          },
          {
            "name": "MotherInnocent",
            "group": 0,
            "id": 44
          },
          {
            "name": "Gribier",
            "group": 0,
            "id": 45
          },
          {
            "name": "Jondrette",
            "group": 7,
            "id": 46
          },
          {
            "name": "Mme.Burgon",
            "group": 7,
            "id": 47
          },
          {
            "name": "Gavroche",
            "group": 8,
            "id": 48
          },
          {
            "name": "Gillenormand",
            "group": 5,
            "id": 49
          },
          {
            "name": "Magnon",
            "group": 5,
            "id": 50
          },
          {
            "name": "Mlle.Gillenormand",
            "group": 5,
            "id": 51
          },
          {
            "name": "Mme.Pontmercy",
            "group": 5,
            "id": 52
          },
          {
            "name": "Mlle.Vaubois",
            "group": 5,
            "id": 53
          },
          {
            "name": "Lt.Gillenormand",
            "group": 5,
            "id": 54
          },
          {
            "name": "Marius",
            "group": 8,
            "id": 55
          },
          {
            "name": "BaronessT",
            "group": 5,
            "id": 56
          },
          {
            "name": "Mabeuf",
            "group": 8,
            "id": 57
          },
          {
            "name": "Enjolras",
            "group": 8,
            "id": 58
          },
          {
            "name": "Combeferre",
            "group": 8,
            "id": 59
          },
          {
            "name": "Prouvaire",
            "group": 8,
            "id": 60
          },
          {
            "name": "Feuilly",
            "group": 8,
            "id": 61
          },
          {
            "name": "Courfeyrac",
            "group": 8,
            "id": 62
          },
          {
            "name": "Bahorel",
            "group": 8,
            "id": 63
          },
          {
            "name": "Bossuet",
            "group": 8,
            "id": 64
          },
          {
            "name": "Joly",
            "group": 8,
            "id": 65
          },
          {
            "name": "Grantaire",
            "group": 8,
            "id": 66
          },
          {
            "name": "MotherPlutarch",
            "group": 9,
            "id": 67
          },
          {
            "name": "Gueulemer",
            "group": 4,
            "id": 68
          },
          {
            "name": "Babet",
            "group": 4,
            "id": 69
          },
          {
            "name": "Claquesous",
            "group": 4,
            "id": 70
          },
          {
            "name": "Montparnasse",
            "group": 4,
            "id": 71
          },
          {
            "name": "Toussaint",
            "group": 5,
            "id": 72
          },
          {
            "name": "Child1",
            "group": 10,
            "id": 73
          },
          {
            "name": "Child2",
            "group": 10,
            "id": 74
          },
          {
            "name": "Brujon",
            "group": 4,
            "id": 75
          },
          {
            "name": "Mme.Hucheloup",
            "group": 8,
            "id": 76
          }
        ]

      },
      {
        "name": "links",
        "values": [
          {
            "source": 1,
            "target": 0,
            "value": 1
          },
          {
            "source": 2,
            "target": 0,
            "value": 8
          },
          {
            "source": 3,
            "target": 0,
            "value": 10
          },
          {
            "source": 3,
            "target": 2,
            "value": 6
          },
          {
            "source": 4,
            "target": 0,
            "value": 1
          },
          {
            "source": 5,
            "target": 0,
            "value": 1
          },
          {
            "source": 6,
            "target": 0,
            "value": 1
          },
          {
            "source": 7,
            "target": 0,
            "value": 1
          },
          {
            "source": 8,
            "target": 0,
            "value": 2
          },
          {
            "source": 9,
            "target": 0,
            "value": 1
          },
          {
            "source": 11,
            "target": 10,
            "value": 1
          },
          {
            "source": 11,
            "target": 3,
            "value": 3
          },
          {
            "source": 11,
            "target": 2,
            "value": 3
          },
          {
            "source": 11,
            "target": 0,
            "value": 5
          },
          {
            "source": 12,
            "target": 11,
            "value": 1
          },
          {
            "source": 13,
            "target": 11,
            "value": 1
          },
          {
            "source": 14,
            "target": 11,
            "value": 1
          },
          {
            "source": 15,
            "target": 11,
            "value": 1
          },
          {
            "source": 17,
            "target": 16,
            "value": 4
          },
          {
            "source": 18,
            "target": 16,
            "value": 4
          },
          {
            "source": 18,
            "target": 17,
            "value": 4
          },
          {
            "source": 19,
            "target": 16,
            "value": 4
          },
          {
            "source": 19,
            "target": 17,
            "value": 4
          },
          {
            "source": 19,
            "target": 18,
            "value": 4
          },
          {
            "source": 20,
            "target": 16,
            "value": 3
          },
          {
            "source": 20,
            "target": 17,
            "value": 3
          },
          {
            "source": 20,
            "target": 18,
            "value": 3
          },
          {
            "source": 20,
            "target": 19,
            "value": 4
          },
          {
            "source": 21,
            "target": 16,
            "value": 3
          },
          {
            "source": 21,
            "target": 17,
            "value": 3
          },
          {
            "source": 21,
            "target": 18,
            "value": 3
          },
          {
            "source": 21,
            "target": 19,
            "value": 3
          },
          {
            "source": 21,
            "target": 20,
            "value": 5
          },
          {
            "source": 22,
            "target": 16,
            "value": 3
          },
          {
            "source": 22,
            "target": 17,
            "value": 3
          },
          {
            "source": 22,
            "target": 18,
            "value": 3
          },
          {
            "source": 22,
            "target": 19,
            "value": 3
          },
          {
            "source": 22,
            "target": 20,
            "value": 4
          },
          {
            "source": 22,
            "target": 21,
            "value": 4
          },
          {
            "source": 23,
            "target": 16,
            "value": 3
          },
          {
            "source": 23,
            "target": 17,
            "value": 3
          },
          {
            "source": 23,
            "target": 18,
            "value": 3
          },
          {
            "source": 23,
            "target": 19,
            "value": 3
          },
          {
            "source": 23,
            "target": 20,
            "value": 4
          },
          {
            "source": 23,
            "target": 21,
            "value": 4
          },
          {
            "source": 23,
            "target": 22,
            "value": 4
          },
          {
            "source": 23,
            "target": 12,
            "value": 2
          },
          {
            "source": 23,
            "target": 11,
            "value": 9
          },
          {
            "source": 24,
            "target": 23,
            "value": 2
          },
          {
            "source": 24,
            "target": 11,
            "value": 7
          },
          {
            "source": 25,
            "target": 24,
            "value": 13
          },
          {
            "source": 25,
            "target": 23,
            "value": 1
          },
          {
            "source": 25,
            "target": 11,
            "value": 12
          },
          {
            "source": 26,
            "target": 24,
            "value": 4
          },
          {
            "source": 26,
            "target": 11,
            "value": 31
          },
          {
            "source": 26,
            "target": 16,
            "value": 1
          },
          {
            "source": 26,
            "target": 25,
            "value": 1
          },
          {
            "source": 27,
            "target": 11,
            "value": 17
          },
          {
            "source": 27,
            "target": 23,
            "value": 5
          },
          {
            "source": 27,
            "target": 25,
            "value": 5
          },
          {
            "source": 27,
            "target": 24,
            "value": 1
          },
          {
            "source": 27,
            "target": 26,
            "value": 1
          },
          {
            "source": 28,
            "target": 11,
            "value": 8
          },
          {
            "source": 28,
            "target": 27,
            "value": 1
          },
          {
            "source": 29,
            "target": 23,
            "value": 1
          },
          {
            "source": 29,
            "target": 27,
            "value": 1
          },
          {
            "source": 29,
            "target": 11,
            "value": 2
          },
          {
            "source": 30,
            "target": 23,
            "value": 1
          },
          {
            "source": 31,
            "target": 30,
            "value": 2
          },
          {
            "source": 31,
            "target": 11,
            "value": 3
          },
          {
            "source": 31,
            "target": 23,
            "value": 2
          },
          {
            "source": 31,
            "target": 27,
            "value": 1
          },
          {
            "source": 32,
            "target": 11,
            "value": 1
          },
          {
            "source": 33,
            "target": 11,
            "value": 2
          },
          {
            "source": 33,
            "target": 27,
            "value": 1
          },
          {
            "source": 34,
            "target": 11,
            "value": 3
          },
          {
            "source": 34,
            "target": 29,
            "value": 2
          },
          {
            "source": 35,
            "target": 11,
            "value": 3
          },
          {
            "source": 35,
            "target": 34,
            "value": 3
          },
          {
            "source": 35,
            "target": 29,
            "value": 2
          },
          {
            "source": 36,
            "target": 34,
            "value": 2
          },
          {
            "source": 36,
            "target": 35,
            "value": 2
          },
          {
            "source": 36,
            "target": 11,
            "value": 2
          },
          {
            "source": 36,
            "target": 29,
            "value": 1
          },
          {
            "source": 37,
            "target": 34,
            "value": 2
          },
          {
            "source": 37,
            "target": 35,
            "value": 2
          },
          {
            "source": 37,
            "target": 36,
            "value": 2
          },
          {
            "source": 37,
            "target": 11,
            "value": 2
          },
          {
            "source": 37,
            "target": 29,
            "value": 1
          },
          {
            "source": 38,
            "target": 34,
            "value": 2
          },
          {
            "source": 38,
            "target": 35,
            "value": 2
          },
          {
            "source": 38,
            "target": 36,
            "value": 2
          },
          {
            "source": 38,
            "target": 37,
            "value": 2
          },
          {
            "source": 38,
            "target": 11,
            "value": 2
          },
          {
            "source": 38,
            "target": 29,
            "value": 1
          },
          {
            "source": 39,
            "target": 25,
            "value": 1
          },
          {
            "source": 40,
            "target": 25,
            "value": 1
          },
          {
            "source": 41,
            "target": 24,
            "value": 2
          },
          {
            "source": 41,
            "target": 25,
            "value": 3
          },
          {
            "source": 42,
            "target": 41,
            "value": 2
          },
          {
            "source": 42,
            "target": 25,
            "value": 2
          },
          {
            "source": 42,
            "target": 24,
            "value": 1
          },
          {
            "source": 43,
            "target": 11,
            "value": 3
          },
          {
            "source": 43,
            "target": 26,
            "value": 1
          },
          {
            "source": 43,
            "target": 27,
            "value": 1
          },
          {
            "source": 44,
            "target": 28,
            "value": 3
          },
          {
            "source": 44,
            "target": 11,
            "value": 1
          },
          {
            "source": 45,
            "target": 28,
            "value": 2
          },
          {
            "source": 47,
            "target": 46,
            "value": 1
          },
          {
            "source": 48,
            "target": 47,
            "value": 2
          },
          {
            "source": 48,
            "target": 25,
            "value": 1
          },
          {
            "source": 48,
            "target": 27,
            "value": 1
          },
          {
            "source": 48,
            "target": 11,
            "value": 1
          },
          {
            "source": 49,
            "target": 26,
            "value": 3
          },
          {
            "source": 49,
            "target": 11,
            "value": 2
          },
          {
            "source": 50,
            "target": 49,
            "value": 1
          },
          {
            "source": 50,
            "target": 24,
            "value": 1
          },
          {
            "source": 51,
            "target": 49,
            "value": 9
          },
          {
            "source": 51,
            "target": 26,
            "value": 2
          },
          {
            "source": 51,
            "target": 11,
            "value": 2
          },
          {
            "source": 52,
            "target": 51,
            "value": 1
          },
          {
            "source": 52,
            "target": 39,
            "value": 1
          },
          {
            "source": 53,
            "target": 51,
            "value": 1
          },
          {
            "source": 54,
            "target": 51,
            "value": 2
          },
          {
            "source": 54,
            "target": 49,
            "value": 1
          },
          {
            "source": 54,
            "target": 26,
            "value": 1
          },
          {
            "source": 55,
            "target": 51,
            "value": 6
          },
          {
            "source": 55,
            "target": 49,
            "value": 12
          },
          {
            "source": 55,
            "target": 39,
            "value": 1
          },
          {
            "source": 55,
            "target": 54,
            "value": 1
          },
          {
            "source": 55,
            "target": 26,
            "value": 21
          },
          {
            "source": 55,
            "target": 11,
            "value": 19
          },
          {
            "source": 55,
            "target": 16,
            "value": 1
          },
          {
            "source": 55,
            "target": 25,
            "value": 2
          },
          {
            "source": 55,
            "target": 41,
            "value": 5
          },
          {
            "source": 55,
            "target": 48,
            "value": 4
          },
          {
            "source": 56,
            "target": 49,
            "value": 1
          },
          {
            "source": 56,
            "target": 55,
            "value": 1
          },
          {
            "source": 57,
            "target": 55,
            "value": 1
          },
          {
            "source": 57,
            "target": 41,
            "value": 1
          },
          {
            "source": 57,
            "target": 48,
            "value": 1
          },
          {
            "source": 58,
            "target": 55,
            "value": 7
          },
          {
            "source": 58,
            "target": 48,
            "value": 7
          },
          {
            "source": 58,
            "target": 27,
            "value": 6
          },
          {
            "source": 58,
            "target": 57,
            "value": 1
          },
          {
            "source": 58,
            "target": 11,
            "value": 4
          },
          {
            "source": 59,
            "target": 58,
            "value": 15
          },
          {
            "source": 59,
            "target": 55,
            "value": 5
          },
          {
            "source": 59,
            "target": 48,
            "value": 6
          },
          {
            "source": 59,
            "target": 57,
            "value": 2
          },
          {
            "source": 60,
            "target": 48,
            "value": 1
          },
          {
            "source": 60,
            "target": 58,
            "value": 4
          },
          {
            "source": 60,
            "target": 59,
            "value": 2
          },
          {
            "source": 61,
            "target": 48,
            "value": 2
          },
          {
            "source": 61,
            "target": 58,
            "value": 6
          },
          {
            "source": 61,
            "target": 60,
            "value": 2
          },
          {
            "source": 61,
            "target": 59,
            "value": 5
          },
          {
            "source": 61,
            "target": 57,
            "value": 1
          },
          {
            "source": 61,
            "target": 55,
            "value": 1
          },
          {
            "source": 62,
            "target": 55,
            "value": 9
          },
          {
            "source": 62,
            "target": 58,
            "value": 17
          },
          {
            "source": 62,
            "target": 59,
            "value": 13
          },
          {
            "source": 62,
            "target": 48,
            "value": 7
          },
          {
            "source": 62,
            "target": 57,
            "value": 2
          },
          {
            "source": 62,
            "target": 41,
            "value": 1
          },
          {
            "source": 62,
            "target": 61,
            "value": 6
          },
          {
            "source": 62,
            "target": 60,
            "value": 3
          },
          {
            "source": 63,
            "target": 59,
            "value": 5
          },
          {
            "source": 63,
            "target": 48,
            "value": 5
          },
          {
            "source": 63,
            "target": 62,
            "value": 6
          },
          {
            "source": 63,
            "target": 57,
            "value": 2
          },
          {
            "source": 63,
            "target": 58,
            "value": 4
          },
          {
            "source": 63,
            "target": 61,
            "value": 3
          },
          {
            "source": 63,
            "target": 60,
            "value": 2
          },
          {
            "source": 63,
            "target": 55,
            "value": 1
          },
          {
            "source": 64,
            "target": 55,
            "value": 5
          },
          {
            "source": 64,
            "target": 62,
            "value": 12
          },
          {
            "source": 64,
            "target": 48,
            "value": 5
          },
          {
            "source": 64,
            "target": 63,
            "value": 4
          },
          {
            "source": 64,
            "target": 58,
            "value": 10
          },
          {
            "source": 64,
            "target": 61,
            "value": 6
          },
          {
            "source": 64,
            "target": 60,
            "value": 2
          },
          {
            "source": 64,
            "target": 59,
            "value": 9
          },
          {
            "source": 64,
            "target": 57,
            "value": 1
          },
          {
            "source": 64,
            "target": 11,
            "value": 1
          },
          {
            "source": 65,
            "target": 63,
            "value": 5
          },
          {
            "source": 65,
            "target": 64,
            "value": 7
          },
          {
            "source": 65,
            "target": 48,
            "value": 3
          },
          {
            "source": 65,
            "target": 62,
            "value": 5
          },
          {
            "source": 65,
            "target": 58,
            "value": 5
          },
          {
            "source": 65,
            "target": 61,
            "value": 5
          },
          {
            "source": 65,
            "target": 60,
            "value": 2
          },
          {
            "source": 65,
            "target": 59,
            "value": 5
          },
          {
            "source": 65,
            "target": 57,
            "value": 1
          },
          {
            "source": 65,
            "target": 55,
            "value": 2
          },
          {
            "source": 66,
            "target": 64,
            "value": 3
          },
          {
            "source": 66,
            "target": 58,
            "value": 3
          },
          {
            "source": 66,
            "target": 59,
            "value": 1
          },
          {
            "source": 66,
            "target": 62,
            "value": 2
          },
          {
            "source": 66,
            "target": 65,
            "value": 2
          },
          {
            "source": 66,
            "target": 48,
            "value": 1
          },
          {
            "source": 66,
            "target": 63,
            "value": 1
          },
          {
            "source": 66,
            "target": 61,
            "value": 1
          },
          {
            "source": 66,
            "target": 60,
            "value": 1
          },
          {
            "source": 67,
            "target": 57,
            "value": 3
          },
          {
            "source": 68,
            "target": 25,
            "value": 5
          },
          {
            "source": 68,
            "target": 11,
            "value": 1
          },
          {
            "source": 68,
            "target": 24,
            "value": 1
          },
          {
            "source": 68,
            "target": 27,
            "value": 1
          },
          {
            "source": 68,
            "target": 48,
            "value": 1
          },
          {
            "source": 68,
            "target": 41,
            "value": 1
          },
          {
            "source": 69,
            "target": 25,
            "value": 6
          },
          {
            "source": 69,
            "target": 68,
            "value": 6
          },
          {
            "source": 69,
            "target": 11,
            "value": 1
          },
          {
            "source": 69,
            "target": 24,
            "value": 1
          },
          {
            "source": 69,
            "target": 27,
            "value": 2
          },
          {
            "source": 69,
            "target": 48,
            "value": 1
          },
          {
            "source": 69,
            "target": 41,
            "value": 1
          },
          {
            "source": 70,
            "target": 25,
            "value": 4
          },
          {
            "source": 70,
            "target": 69,
            "value": 4
          },
          {
            "source": 70,
            "target": 68,
            "value": 4
          },
          {
            "source": 70,
            "target": 11,
            "value": 1
          },
          {
            "source": 70,
            "target": 24,
            "value": 1
          },
          {
            "source": 70,
            "target": 27,
            "value": 1
          },
          {
            "source": 70,
            "target": 41,
            "value": 1
          },
          {
            "source": 70,
            "target": 58,
            "value": 1
          },
          {
            "source": 71,
            "target": 27,
            "value": 1
          },
          {
            "source": 71,
            "target": 69,
            "value": 2
          },
          {
            "source": 71,
            "target": 68,
            "value": 2
          },
          {
            "source": 71,
            "target": 70,
            "value": 2
          },
          {
            "source": 71,
            "target": 11,
            "value": 1
          },
          {
            "source": 71,
            "target": 48,
            "value": 1
          },
          {
            "source": 71,
            "target": 41,
            "value": 1
          },
          {
            "source": 71,
            "target": 25,
            "value": 1
          },
          {
            "source": 72,
            "target": 26,
            "value": 2
          },
          {
            "source": 72,
            "target": 27,
            "value": 1
          },
          {
            "source": 72,
            "target": 11,
            "value": 1
          },
          {
            "source": 73,
            "target": 48,
            "value": 2
          },
          {
            "source": 74,
            "target": 48,
            "value": 2
          },
          {
            "source": 74,
            "target": 73,
            "value": 3
          },
          {
            "source": 75,
            "target": 69,
            "value": 3
          },
          {
            "source": 75,
            "target": 68,
            "value": 3
          },
          {
            "source": 75,
            "target": 25,
            "value": 3
          },
          {
            "source": 75,
            "target": 48,
            "value": 1
          },
          {
            "source": 75,
            "target": 41,
            "value": 1
          },
          {
            "source": 75,
            "target": 70,
            "value": 1
          },
          {
            "source": 75,
            "target": 71,
            "value": 1
          },
          {
            "source": 76,
            "target": 64,
            "value": 1
          },
          {
            "source": 76,
            "target": 65,
            "value": 1
          },
          {
            "source": 76,
            "target": 66,
            "value": 1
          },
          {
            "source": 76,
            "target": 63,
            "value": 1
          },
          {
            "source": 76,
            "target": 62,
            "value": 1
          },
          {
            "source": 76,
            "target": 48,
            "value": 1
          },
          {
            "source": 76,
            "target": 58,
            "value": 1
          }
        ]

      }
    ],

    networks: [
      {
        name: "le-mis-network",
        nodes: "nodes",
        links: "links",
        directed: true,
        source_node: ["id", "source"], // node.id == link.source
        target_node: ["id", "target"],
        metrics: [{ metric: "degree" }, { metric: "closeness" }]

      }
    ],

    layouts: [
      {
        name: "le-mis-layout",
        network: "le-mis-network",

        // type: "d3-force",
        //type: "webcola"

        "type": "cytoscape",
        "algorithm": "circle",
        "animate": false,


        "normalizeX": [0, 500],
        "normalizeY": [0, 500]
      }
    ],

    scales: [

      {
        name: "radius",
        type: "sqrt",
        range: [1, 10],
        domain: [0, 40]
      },
      {
        name: "color",
        type: "linear",
        scheme: "reds",
        domain: [0, 40]
      }
    ],

    // applying a layout implicity sets
    vis: [

      {
        entries: "le-mis-network.links",
        layout: "le-mis-layout",
        mark: {
          type: "linkpath",

          // This option is simple but limited - e.g., can't offset lines from point
          start: "source", // i.e., take .source for each edge, apply the layout, and position start of line there
          end: "target" // i.e., take .target for each edge, apply the layout, and position end of line there

          // This is ambiguous - does it mean layout(link.source).x or layout(link.source.x)?
          //xSource: {field: "source.x"}, /// this means apply the layout to .source, then get .x - ambiguous with applying layout to .source.x
          //ySource: {field: "source.y"},
          //xTarget: {field: "target.x"},
          //yTarget: {field: "target.y"}

          // This is explicit, but requires parsing
          // xSource: {field: "le-mis-layout(source).x"}, /// this means apply the layout to .source, then get .x - ambiguous with applying layout to .source.x
          // ySource: {field: "le-mis-layout(source).y"},
          // xTarget: {field: "le-mis-layout(target).x"},
          // yTarget: {field: "le-mis-layout(target).y"}
        }
      },
      {
        entries: "le-mis-network.nodes",
        layout: "le-mis-layout",
        mark: {
          type: "circle",
          radius: { field: "degree", scale: "radius" },
          fill: { field: "degree", scale: "color" }
        }
      }
      // labels, and offsetting their position ?
    ]
  };


  /*
          relationships: [
          ["source_node", "links.source", "nodes.id",],
          ["target_node", "links.target", "nodes.id",]
      ],

   */
  processFullSpec(spec, {}).then(scene => {

 //     console.log(JSON.stringify(scene, null, 4));
      fs.writeFile("test-scenegraph.json", JSON.stringify(scene, null, 4), () => null);


      const svgString = renderSceneJSONToString(scene.sceneJSON, 1000, 1000);
//      console.log(svgString)
      fs.writeFile("test-scenegraph.svg",svgString, () => null);

      //  fs.writeFile("test-auto.svg", renderer.getSVGString(), ()=> null);

      // console.log(renderer.getSVGString());


      /*
    expect(d.nodes.length).toBe(77);
    expect(d.links.length).toBe(254);


    // check the join worked
    expect(d.links[0].source_node).toEqual( {
                    "name": "Napoleon",
                    "group": 1,
                    "id": 1
                })

      //  console.log(d);

       */

    }
  );


});