import { computeAggregation } from "./aggregation";

test("simple aggregation works", () => {

  // example from https://vega.github.io/vega/docs/transforms/aggregate/

  const x = [
    { "foo": 1, "bar": 1 },
    { "foo": 1, "bar": 2 },
    { "foo": null, "bar": 3 }
  ];

  const spec = {
    "type": "aggregate",
    "fields": ["foo", "bar", "bar"],
    "ops": ["valid", "sum", "median"],
    "as": ["v", "s", "m"]
  };

  expect(computeAggregation(x, spec)).toEqual(
    [{ "v": 2, "s": 6, "m": 2 }]
  );

});


test("usage with groupby", () => {

  // example from https://vega.github.io/vega/docs/transforms/aggregate/

  const x = [
    { "foo": "a", "bar": 1 },
    { "foo": "a", "bar": 2 },
    { "foo": "b", "bar": 3 }
  ];

  const spec = {
    "type": "aggregate",
    "groupby": ["foo"]
  };


  const result = computeAggregation(x, spec);



  expect(result).toEqual(
    [
      { "foo": "a", "count": 2 },
      { "foo": "b", "count": 1 }
    ]
  );


});


test("usage with nested fields", () => {

  // example from https://vega.github.io/vega/docs/transforms/aggregate/

  const x = [
    { "foo": { "baz": "a" }, "bar": 1 },
    { "foo": { "baz": "a" }, "bar": 2 },
    { "foo": { "baz": "b" }, "bar": 3 }
  ];

  const spec = {
    "type": "aggregate",
    "groupby": ["foo.baz"]
  };

  const result = computeAggregation(x, spec);

  expect(result).toEqual(
    [
      { "foo.baz": "a", "count": 2 },
      { "foo.baz": "b", "count": 1 }
    ]
  );

});

