import {loadData} from "./data"

// We have marked several tests as to be skipped to limit network requests to other people's servers during development


test("it loads literal data", () => {
    const spec = [
        {
            "name": "foo",
            "values": [
                {"a": "A", "b": 28}, {"a": "B", "b": 55}, {"a": "C", "b": 43},
                {"a": "D", "b": 91}, {"a": "E", "b": 81}, {"a": "F", "b": 53},
                {"a": "G", "b": 19}, {"a": "H", "b": 87}, {"a": "I", "b": 52}
            ]
        }
    ];

    loadData(spec).then(r => {
        expect(r.foo.length).toBe(9)
        expect(r.foo).toBe(spec[0].values)
    })
});

test.skip("it loads JSON from URL", () => {
    const spec = [
        {
            "name": "cars",
            "url": "https://vega.github.io/vega-lite/data/cars.json"
        },
    ];

    loadData(spec).then(r => {
        expect(r.cars).toHaveLength(406);
        expect(r.cars[0]).toEqual({
            "Acceleration": 12,
            "Cylinders": 8,
            "Displacement": 307,
            "Horsepower": 130,
            "Miles_per_Gallon": 18,
            "Name": "chevrolet chevelle malibu",
            "Origin": "USA",
            "Weight_in_lbs": 3504,
            "Year": "1970-01-01",
        })
    });


});

test.skip("it loads CSV from URL", () => {
    const spec = [
        {
            "name": "stocks",
            "url": "https://vega.github.io/datalib/data/stocks.csv",
            "format": {
                "type": "csv",
                "parse": {"date": "date", "price": "number"}
            }
        },
    ];

    loadData(spec).then(r => {
        expect(r.stocks).toHaveLength(560);
        expect(r.stocks[0]).toEqual({"date": 946684800000, "price": 39.81, "symbol": "MSFT"})
    })
});

test.skip("it loads multiple data sources at once", () => {
        const spec = [
            {
                "name": "foo",
                "values": [
                    {"a": "A", "b": 28}, {"a": "B", "b": 55}, {"a": "C", "b": 43},
                    {"a": "D", "b": 91}, {"a": "E", "b": 81}, {"a": "F", "b": 53},
                    {"a": "G", "b": 19}, {"a": "H", "b": 87}, {"a": "I", "b": 52}
                ]
            },
            {
                "name": "cars",
                "url": "https://vega.github.io/vega-lite/data/cars.json"
            },
            {
                "name": "stocks",
                "url": "https://vega.github.io/datalib/data/stocks.csv",
                "format": {
                    "type": "csv",
                    "parse": {"date": "date", "price": "number"}
                }
            }
        ];

        loadData(spec).then(r => {
            expect(r.foo).toHaveLength(9)
            expect(r.foo).toBe(spec[0].values)

            expect(r.stocks).toHaveLength(560);
            expect(r.stocks[0]).toEqual({"date": 946684800000, "price": 39.81, "symbol": "MSFT"})

            expect(r.stocks).toHaveLength(560);
            expect(r.stocks[0]).toEqual({"date": 946684800000, "price": 39.81, "symbol": "MSFT"})
        });
    }
)


test("it makes transformed copy of existing data", () => {
    const spec = [
        {
            "name": "foo",
            "values": [
                {"a": "A", "b": 28}, {"a": "B", "b": 55}, {"a": "C", "b": 43},
            ]
        },
      {
        "name": "bar",
        "source": "foo",
        "transform": [
            {
                "type": "calculate",
                "calculate": "datum.b * 10",
                "as": "c"
            }
        ]
    }
    ];

    loadData(spec).then(r => {
        expect(r.foo).toEqual([
            { "a": "A", "b": 28 },
            { "a": "B", "b": 55 },
            { "a": "C", "b": 43 },
        ]);


        expect(r.bar).toEqual([
          { "a": "A", "b": 28, "c": 280 },
          { "a": "B", "b": 55, "c": 550 },
          { "a": "C", "b": 43, "c":  430 },
          ]
        );

    })


});


test("can laod data from a function", () => {

    const getData = () => [
                {"a": "A", "b": 28}, {"a": "B", "b": 55}, {"a": "C", "b": 43},
                {"a": "D", "b": 91}, {"a": "E", "b": 81}, {"a": "F", "b": 53},
                {"a": "G", "b": 19}, {"a": "H", "b": 87}, {"a": "I", "b": 52}
            ];

    loadData([{ name: "foo", function: "getData" }], { getData }).then(r => {
        expect(r.foo.length).toBe(9)
        expect(r.foo).toEqual(getData())
    })
});
