// see vega-parsers/src/parsers/data.js
// @ts-ignore
import {loader, read} from "vega-loader";

import { applyTransform } from "./transform";
import type { GaGraphDataSpec } from "../types/spec";
import { select } from "./select";

type DataType =  {
    [name: string]: any[];
}

async function loadData(fullDataSpec: GaGraphDataSpec[], funcs?: any) {
    const allData: DataType = {};
    for (const dataset of fullDataSpec) {
        const data = await loadSingleDataset(dataset, allData, funcs);

        if (data) {
            allData[data.name] = data.values;

            if ("transform" in dataset) {
                for (const transformSpec of dataset.transform) {
                    data.values = await applyTransform(allData, data.values, transformSpec, funcs);
                }
            }

            allData[data.name] = data.values;
        }
    }
    return allData;
}

async function loadSingleDataset(dataSpec: GaGraphDataSpec, allData: any, funcs?: any) {
    if ("values" in dataSpec) {
        return {name: dataSpec.name, values: dataSpec.values};
    } else if ("url" in dataSpec) {
        const dataText = await loader().load(dataSpec.url);
        return {name: dataSpec.name, values: read(dataText, dataSpec.format)}
    } else if ("source" in dataSpec){
        const data = JSON.parse(JSON.stringify(select(allData, dataSpec.source)));
        return {name: dataSpec.name, values: data}
    } else if ("localStorage" in dataSpec){
        const dataText = window.localStorage.getItem(dataSpec.localStorage);
        return {name: dataSpec.name, values: read(dataText, dataSpec.format)}
    } else if ("function" in dataSpec){
        return {name: dataSpec.name, values: funcs[dataSpec.function](dataSpec) }
    }
    // we don't allow named datasets
}

export {loadData}