import type {FilterSpec} from "../types/data-transforms"

// https://vega.github.io/vega-lite/docs/filter.html

import { codegenExpression, parseExpression } from "vega-expression";

const filter = (data, spec: FilterSpec) => {

  const codegen = codegenExpression({
    globalvar: "globalVar"
  });

  const funcCode = codegen(parseExpression(spec.expr));

  // First argument means that the function argument will be interpreted as having name "globalVar"
  // so we can avoid using an actual global variable
  const fn = Function("globalVar", "\"use strict\"; return (" + funcCode.code + ")");

  return data.filter((d: any) => fn({datum: d}));
};

export {filter};
