import { cross, max, zip } from "d3-array";
import type { FlattenSpec } from "../types/data-transforms";

const flatten = (data, spec: FlattenSpec) => {

  const output = [];
  for (const row of data) {
    const iterables = spec.flatten.map((fieldName: string) => row[fieldName]);

    const maxIterableLength = max(iterables.map(l => l.length)) as unknown as number; // this is a kludge: I think the d3 function has the wrong type signature

    for (let i = 0; i < maxIterableLength; i++) {
      const flattenedVals = iterables.map(iter => iter[i]);

      const flattenedValObj = Object.fromEntries(zip(spec.flatten, flattenedVals));
      output.push({ ...row, ...flattenedValObj });
    }
  }
  return output;
};

const flattenCartesian = (data, spec) => {

  const output = [];

  for (const row of data) {
    const iterables = spec.flatten.map((fieldName: string) => row[fieldName]);

    const crossed = cross.apply(null, iterables);
    for (const combination of crossed) {
      const combinationVals = Object.fromEntries(zip(spec.flatten, combination));
      output.push({ ...row, ...combinationVals });
    }
  }
  return output;

};

export { flatten };