import { group, groupFlattened, groupFlattenedComplete } from "./group";

const data = {};
data.edges = [
    {source: 1, target: 2, id: 1},
    {source: 1, target: 2, id: 2},
    {source: 1, target: 3, id: 3},
    {source: 1, target: 4, id: 4},
    {source: 1, target: 5, id: 5},

    {source: 2, target: 2, id: 6},
    {source: 2, target: 2, id: 7},
    {source: 3, target: 3, id: 8},
    {source: 3, target: 4, id: 9},
    {source: 4, target: 5, id: 10},
];


test("can group by a single attribute", () => {
    const grouped = group(data, 'edges', ['source']);

    expect(grouped).toEqual(
        {
            "1": [
                {"source": 1, "target": 2, "id": 1},
                {"source": 1, "target": 2, "id": 2},
                {"source": 1, "target": 3, "id": 3 },
                {"source": 1, "target": 4, "id": 4},
                {"source": 1, "target": 5, "id": 5}
            ],
            "2": [
                {"source": 2, "target": 2, "id": 6},
                {"source": 2, "target": 2, "id": 7}
            ],
            "3": [
                {"source": 3, "target": 3, "id": 8},
                {"source": 3, "target": 4, "id": 9}
            ],
            "4": [{"source": 4, "target": 5, "id": 10}]
        }
    )




    const groupedF = groupFlattened(data, 'edges', ['source']);
    expect(groupedF).toEqual(
      [
        {
          id: 0,
          size: 5,
          key: [1],
          data: [
            { "source": 1, "target": 2, "id": 1 },
            { "source": 1, "target": 2, "id": 2 },
            { "source": 1, "target": 3, "id": 3 },
            { "source": 1, "target": 4, "id": 4 },
            { "source": 1, "target": 5, "id": 5 }
          ]
        },
        {
          id: 1,
          size: 2,
          key: [2],
          data: [
            { "source": 2, "target": 2, "id": 6 },
            { "source": 2, "target": 2, "id": 7 }
          ]
        },
        {
          id: 2,
          size: 2,
          key: [3],
          data: [
            { "source": 3, "target": 3, "id": 8 },
            { "source": 3, "target": 4, "id": 9 }
          ]
        },
        {
          id: 3,
          size: 1,
          key: [4],
          data: [{ "source": 4, "target": 5, "id": 10 }]
        }
      ]
    )
})


test("can group by two attributes", () => {
    const grouped = group(data, 'edges', ['source', 'target']);

    expect(grouped).toEqual(
        {
            "1": {
                "2": [{"source": 1, "target": 2, "id": 1}, {"source": 1, "target": 2, "id": 2}],
                "3": [{"source": 1, "target": 3, "id": 3}],
                "4": [{"source": 1, "target": 4, "id": 4}],
                "5": [{"source": 1, "target": 5, "id": 5}]
            },
            "2": {"2": [{"source": 2, "target": 2, "id": 6}, {"source": 2, "target": 2, "id": 7}]},
            "3": {"3": [{"source": 3, "target": 3, "id": 8}], "4": [{"source": 3, "target": 4, "id": 9}]},
            "4": {"5": [{"source": 4, "target": 5, "id": 10}]}
        }
    );

  const groupedF = groupFlattened(data, "edges", ["source", "target"]);

  expect(groupedF).toEqual(
    [
      {
        id: 0,
        size: 2,
        key: [1, 2],
        data: [{ "source": 1, "target": 2, "id": 1 }, { "source": 1, "target": 2, "id": 2 }]
      },
      { id: 1, size: 1, key: [1, 3], data: [{ "source": 1, "target": 3, "id": 3 }] },
      { id: 2, size: 1, key: [1, 4],  data: [{ "source": 1, "target": 4, "id": 4 }] },
      { id: 3, size: 1, key: [1, 5], data: [{ "source": 1, "target": 5, "id": 5 }] },
      {
        id: 4, size: 2, key: [2, 2], data: [{ "source": 2, "target": 2, "id": 6 }, { "source": 2, "target": 2, "id": 7 }]
      },
      { id: 5, size: 1, key: [3, 3], data: [{ "source": 3, "target": 3, "id": 8 }] },
      { id: 6, size: 1, key: [3, 4], data: [{ "source": 3, "target": 4, "id": 9 }] },
      { id: 7, size: 1, key: [4, 5], data: [{ "source": 4, "target": 5, "id": 10 }] }
    ]
  );
})

test("can group by a nested expression", () => {

    const nestedData = [
    { "foo": { "baz": "a" }, "bar": 1 },
    { "foo": { "baz": "a" }, "bar": 2 },
    { "foo": { "baz": "b" }, "bar": 3 }
  ];

const grouped = group({ data: nestedData }, "data", ["foo.baz"]);

expect(grouped).toEqual({
  a: [ { "foo": { "baz": "a" }, "bar": 1 }, { "foo": { "baz": "a" }, "bar": 2 }],
  b: [{ "foo": { "baz": "b" }, "bar": 3 }]
})


  const groupedF = groupFlattened({ data: nestedData }, "data", ["foo.baz"]);

  expect(groupedF).toEqual([
    { id: 0, size: 2, key: ["a"], data: [{ "foo": { "baz": "a" }, "bar": 1 }, { "foo": { "baz": "a" }, "bar": 2 }]
},
    {id: 1, size: 1, key: ["b"], data: [{ "foo": { "baz": "b" }, "bar": 3 }] }
  ]);

//console.log(grouped);


})




test("can group, returning the empty groups too", () => {

  const d = [
    { x: 1, y: 2, id: 1 },
    { x: 1, y: 3, id: 2 },
    { x: 2, y: 4, id: 3 },
    { x: 5, y: 1, id: 4 },
  ];


  const grouped = groupFlattenedComplete({ data: d }, 'data', ['x', 'y']);

  const expected = [
      {"data": [], "id": 5, "key": [1,1], "size": 0},
      {"data": [{"id": 1, "x": 1, "y": 2}], "id": 0, "key": [1, 2], "size": 1},
      {"data": [{"id": 2, "x": 1, "y": 3}], "id": 1, "key": [1, 3], "size": 1},
      {"data": [], "id": 4, "key": [1,4], "size": 0},

      {"data": [], "id": 8, "key": [2,1], "size": 0},
      {"data": [], "id": 6, "key": [2,2], "size": 0},
      {"data": [], "id": 7, "key": [2,3], "size": 0},
      {"data": [{"id": 3, "x": 2, "y": 4}], "id": 2, "key": [2, 4], "size": 1},

      {"data": [{"id": 4, "x": 5, "y": 1}], "id": 3, "key": [5, 1], "size": 1},
      {"data": [], "id": 9, "key": [5,2], "size": 0},
      {"data": [], "id": 10, "key": [5,3], "size": 0},
      {"data": [], "id": 11, "key": [5,4], "size": 0}
  ].sort( (a,b) => a.id - b.id );

  expect(grouped).toEqual(expected);

})