import { join } from "./join";

test("can merge two arrays", () => {

  const array1 = [
    { id: 1, name: "Foo" },
    { id: 2, name: "Bar" },
    { id: 3, name: "Baz" }
  ];

  const array2 = [
    { Name: "Foo", age: 18 },
    { Name: "Bar", age: 21 },
    { Name: "Baz", age: 16 }
  ];

  const merged = join(array1, array2, "name", "Name");

  expect(merged).toEqual([
      { id: 1, name: "Foo", Name: "Foo", age: 18 },
      { id: 2, name: "Bar", Name: "Bar", age: 21 },
      { id: 3, name: "Baz", Name: "Baz", age: 16 }
    ]
  );

});

test("only adds requested fields", () => {

  const array1 = [
    { id: 1, name: "Foo" },
    { id: 2, name: "Bar" },
    { id: 3, name: "Baz" }
  ];

  const array2 = [
    { Name: "Foo", age: 18 },
    { Name: "Bar", age: 21 },
    { Name: "Baz", age: 16 }
  ];

  const merged = join(array1, array2, "name", "Name", ["age"]);

  expect(merged).toEqual([
      { id: 1, name: "Foo", age: 18 },
      { id: 2, name: "Bar", age: 21 },
      { id: 3, name: "Baz", age: 16 }
    ]
  );

});



test("can cope with missing values in array2", () => {
    const array1 = [
    { id: 1, name: "Foo" },
    { id: 2, name: "Bar" },
    { id: 3, name: "Baz" }
  ];

  const array2 = [
    { Name: "Foo", age: 18 },
    { Name: "Bar", age: 21 },
  ];

  const merged = join(array1, array2, "name", "Name");

  expect(merged).toEqual([
      { id: 1, name: "Foo", Name: "Foo", age: 18 },
      { id: 2, name: "Bar", Name: "Bar", age: 21 },
      { id: 3, name: "Baz"}
    ]
  );
})


test("can cope with empty array2", () => {
    const array1 = [
    { id: 1, name: "Foo" },
    { id: 2, name: "Bar" },
    { id: 3, name: "Baz" }
  ];

  const merged = join(array1, [], "name", "Name");

  expect(merged).toEqual([
      { id: 1, name: "Foo"},
      { id: 2, name: "Bar" },
      { id: 3, name: "Baz"}
    ]
  );
})