import { lookup } from "./lookup";

// example based on https://vega.github.io/vega/docs/transforms/lookup/

test("can do a lookup that attaches whole objects", () => {

  const data = {
    names: [
      { "id": "A", "name": "label A" },
      { "id": "B", "name": "label B" },
      { "id": "C", "name": "label C" }
    ],
    values: [
      { "foo": "A", "bar": 28 },
      { "foo": "B", "bar": 55 },
      { "foo": "C", "bar": 43 },
      { "foo": "C", "bar": 91 },
      { "foo": "D", "bar": 81 }
    ]
  };

  const spec = {
    "type": "lookup",
    "from": "names",
    "key": "id",
    "fields": ["foo"],
    "as": ["obj"]
  };

  const res = lookup(data.values, data, spec);


  expect(res).toEqual([
    { "foo": "A", "bar": 28, "obj": { "id": "A", "name": "label A" } },
    { "foo": "B", "bar": 55, "obj": { "id": "B", "name": "label B" } },
    { "foo": "C", "bar": 43, "obj": { "id": "C", "name": "label C" } },
    { "foo": "C", "bar": 91, "obj": { "id": "C", "name": "label C" } },
    { "foo": "D", "bar": 81, "obj": undefined } // null rather than undefined in vega example
  ]);

  expect(data.values).toEqual([
      { "foo": "A", "bar": 28 },
      { "foo": "B", "bar": 55 },
      { "foo": "C", "bar": 43 },
      { "foo": "C", "bar": 91 },
      { "foo": "D", "bar": 81 }
    ]);

});

test("can do a lookup that attaches specific fields", () => {

  const data = {
    names: [
      { "id": "A", "name": "label A" },
      { "id": "B", "name": "label B" },
      { "id": "C", "name": "label C" }
    ],
    values: [
      { "foo": "A", "bar": 28 },
      { "foo": "B", "bar": 55 },
      { "foo": "C", "bar": 43 },
      { "foo": "C", "bar": 91 },
      { "foo": "D", "bar": 81 }
    ]
  };

  const spec = {
    "type": "lookup",
    "from": "names",
    "key": "id",
    "fields": ["foo"],
    "values": ["name"],
    "as": ["obj"],
    default: "some label"
  };

  const res = lookup(data.values, data, spec);

  expect(res).toEqual([
{"foo": "A", "bar": 28, "obj": "label A"},
{"foo": "B", "bar": 55, "obj": "label B"},
{"foo": "C", "bar": 43, "obj": "label C"},
{"foo": "C", "bar": 91, "obj": "label C"},
{"foo": "D", "bar": 81, "obj": "some label"}
  ]);

  expect(data.values).toEqual([
      { "foo": "A", "bar": 28 },
      { "foo": "B", "bar": 55 },
      { "foo": "C", "bar": 43 },
      { "foo": "C", "bar": 91 },
      { "foo": "D", "bar": 81 }
    ]);


});