import { pivot } from "./pivot";

test("can pivot", () => {

  // example from https://vega.github.io/vega-lite/docs/pivot.html

  const data = [
    { "country": "Norway", "type": "gold", "count": 14 },
    { "country": "Norway", "type": "silver", "count": 14 },
    { "country": "Norway", "type": "bronze", "count": 11 },
    { "country": "Germany", "type": "gold", "count": 14 },
    { "country": "Germany", "type": "silver", "count": 10 },
    { "country": "Germany", "type": "bronze", "count": 7 },
    { "country": "Canada", "type": "gold", "count": 11 },
    { "country": "Canada", "type": "silver", "count": 8 },
    { "country": "Canada", "type": "bronze", "count": 10 }
  ];

  const spec = {
    "pivot": "type",
    "groupby": ["country"],
    "value": "count"
  };

  const pivoted = pivot(data, spec);

  expect(pivoted).toEqual([
    { "country": "Norway", "gold": 14, "silver": 14, "bronze": 11 },
    { "country": "Germany", "gold": 14, "silver": 10, "bronze": 7 },
    { "country": "Canada", "gold": 11, "silver": 8, "bronze": 10 }
  ]);

});

test("can pivot with aggregation operation", () => {

  // example from https://vega.github.io/vega-lite/docs/pivot.html

  const data = [
    { "country": "Norway", "type": "gold", "count": 14 },
    { "country": "Norway", "type": "gold", "count": 10 },

    { "country": "Norway", "type": "silver", "count": 14 },
    { "country": "Norway", "type": "silver", "count": 6 },

    { "country": "Norway", "type": "bronze", "count": 11 },
    { "country": "Germany", "type": "gold", "count": 14 },
    { "country": "Germany", "type": "silver", "count": 10 },
    { "country": "Germany", "type": "bronze", "count": 7 },
    { "country": "Canada", "type": "gold", "count": 11 },
    { "country": "Canada", "type": "silver", "count": 8 },
    { "country": "Canada", "type": "bronze", "count": 10 }
  ];

  const spec = {
    "pivot": "type",
    "groupby": ["country"],
    "value": "count",
    "op": "mean"
  };

  const pivoted = pivot(data, spec);

  expect(pivoted).toEqual([
    { "country": "Norway", "gold": 12, "silver": 10, "bronze": 11 },
    { "country": "Germany", "gold": 14, "silver": 10, "bronze": 7 },
    { "country": "Canada", "gold": 11, "silver": 8, "bronze": 10 }
  ]);

});