import {addRelationship, addRelationships} from "./relationships";


test("handles source/target relationship", () => {

    let data = new Map();

    data.nodes = [
        {id: 0, name: 'foo'},
        {id: 1, name: 'bar'},
        {id: 2, name: 'foo-bar'}
    ];

    data.edges = [
        {source_id: 0, target_id: 1},
        {source_id: 1, target_id: 2},
        {source_id: 1, target_id: 20}

    ]

    addRelationship(data, ["source", "edges.source_id", "nodes.id",]);
    addRelationship(data, ["target", "edges.target_id", "nodes.id",]);

    expect(data["edges"][0].source).toEqual({id: 0, name: 'foo'});
    expect(data["edges"][1].source).toEqual({id: 1, name: 'bar'});
    expect(data["edges"][2].source).toEqual({id: 1, name: 'bar'});

    expect(data["edges"][0].target).toEqual({id: 1, name: 'bar'});
    expect(data["edges"][1].target).toEqual({id: 2, name: 'foo-bar'});
    expect(data["edges"][2].target).toEqual(null);
})


test("can add multiple relationships at once", () => {

    let data = new Map();

    data.nodes = [
        {id: 0, name: 'foo'},
        {id: 1, name: 'bar'},
        {id: 2, name: 'foo-bar'}
    ];

    data.edges = [
        {source_id: 0, target_id: 1},
        {source_id: 1, target_id: 2},
        {source_id: 1, target_id: 20}

    ]

    addRelationships(data, [["source", "edges.source_id", "nodes.id",], ["target", "edges.target_id", "nodes.id",]]);

    expect(data["edges"][0].source).toEqual({id: 0, name: 'foo'});
    expect(data["edges"][1].source).toEqual({id: 1, name: 'bar'});
    expect(data["edges"][2].source).toEqual({id: 1, name: 'bar'});

    expect(data["edges"][0].target).toEqual({id: 1, name: 'bar'});
    expect(data["edges"][1].target).toEqual({id: 2, name: 'foo-bar'});
    expect(data["edges"][2].target).toEqual(null);
})

