import type { TransformSpec } from "../types/data-transforms";
import { computeAggregation } from "./aggregation";
import { computeBins } from "./bin";
import { calculateCalculate } from "./calculate";
import { filter } from "./filter";
import { flatten } from "./flatten";
import { foldData } from "./fold";
import { geocode } from "./geocoding";
import { pivot } from "./pivot";
import { sample } from "./sample";
import { lookup } from "./lookup";
import { unroll } from "./unroll";
import { addIndex } from "./addIndex";


const applyTransform = async (state, data, spec: TransformSpec, funcs?) => {
  if (spec.type === "aggregation") {
    return computeAggregation(data, spec);
  } else if (spec.type === "bin") {
    return computeBins(data, spec);
  } else if (spec.type === "calculate") {
    return calculateCalculate(data, spec);
  } else if (spec.type === "filter") {
    return filter(data, spec);
  } else if (spec.type === "flatten") {
    return flatten(data, spec);
  } else if (spec.type === "fold") {
    return foldData(data, spec);
  } else if (spec.type === "pivot"){
    return pivot(data, spec);
  } else if (spec.type === "sample"){
    return sample(data, spec);
  } else if (spec.type === "unroll"){
    return unroll(data, spec);
  } else if (spec.type === "lookup"){
    return lookup(data, state, spec);
  } else if (spec.type === "geocode"){
    return await geocode(data, spec);
  } else if (spec.type === "addIndex"){
    return addIndex(data, spec);
  } else if (spec.type === "function"){
    return funcs[spec.function](data, spec);
  }
}


export { applyTransform };