import {range} from "d3-array";

import {grid} from "./grid";

test("handles row-by-row grid", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'row-by-row'));
    expect(pattern).toEqual(
        [
            [0, 0], [1, 0], [2, 0], [3, 0], [4, 0],
            [0, 1], [1, 1], [2, 1], [3, 1], [4, 1],
            [0, 2], [1, 2], [2, 2], [3, 2], [4, 2],
            [0, 3], [1, 3], [2, 3], [3, 3], [4, 3],
            [0, 4], [1, 4], [2, 4], [3, 4], [4, 4]
        ]
    );
})

test("handles col-by-col grid", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'col-by-col'));
    expect(pattern).toEqual(
        [
            [0, 0], [0, 1], [0, 2], [0, 3], [0, 4],
            [1, 0], [1, 1], [1, 2], [1, 3], [1, 4],
            [2, 0], [2, 1], [2, 2], [2, 3], [2, 4],
            [3, 0], [3, 1], [3, 2], [3, 3], [3, 4],
            [4, 0], [4, 1], [4, 2], [4, 3], [4, 4]
        ]
    );
})

test("handles row-snake grid", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'row-snake'));
    expect(pattern).toEqual(
        [
            [0, 0], [1, 0], [2, 0], [3, 0], [4, 0],
            [4, 1], [3, 1], [2, 1], [1, 1], [0, 1],
            [0, 2], [1, 2], [2, 2], [3, 2], [4, 2],
            [4, 3], [3, 3], [2, 3], [1, 3], [0, 3],
            [0, 4], [1, 4], [2, 4], [3, 4], [4, 4]
        ]
    );
})

test("handles col-snake grid", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'col-snake'));
    expect(pattern).toEqual(
        [
            [0, 0], [0, 1], [0, 2], [0, 3], [0, 4],
            [1, 4], [1, 3], [1, 2], [1, 1], [1, 0],
            [2, 0], [2, 1], [2, 2], [2, 3], [2, 4],
            [3, 4], [3, 3], [3, 2], [3, 1], [3, 0],
            [4, 0], [4, 1], [4, 2], [4, 3], [4, 4]
        ]
    );
})

test("handles diagonal", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'diagonal'));
    expect(pattern).toEqual(
        [
            [0, 0],
            [0, 1], [1, 0],
            [0, 2], [1, 1], [2, 0],
            [0, 3], [1, 2], [2, 1], [3, 0],
            [0, 4], [1, 3], [2, 2], [3, 1], [4, 0],
            [0, 5], [1, 4], [2, 3], [3, 2], [4, 1], [5, 0],
            [0, 6], [1, 5], [2, 4], [3, 3]
        ]
    );
})

test("handles diagonal-snake", () => {
    const pattern = range(25).map(i => grid(i, 5, 5, 'diagonal-snake'));
    expect(pattern).toEqual([
            [0, 0],
            [0, 1], [1, 0],
            [2, 0], [1, 1], [0, 2],
            [0, 3], [1, 2], [2, 1], [3, 0],
            [4, 0], [3, 1], [2, 2], [1, 3], [0, 4],
            [0, 5], [1, 4], [2, 3], [3, 2], [4, 1], [5, 0],
            [6, 0], [5, 1], [4, 2], [3, 3]
        ]
    );
})

test("handles z-curve", () => {
    const pattern = range(64).map(i => grid(i, 8, 8, 'z-curve'));
    expect(pattern).toEqual([

        [0, 0], [1, 0], [0, 1], [1, 1],
        [2, 0], [3, 0], [2, 1], [3, 1],
        [0, 2], [1, 2], [0, 3], [1, 3],
        [2, 2], [3, 2], [2, 3], [3, 3],

        [4, 0], [5, 0], [4, 1], [5, 1],
        [6, 0], [7, 0], [6, 1], [7, 1],
        [4, 2], [5, 2], [4, 3], [5, 3],
        [6, 2], [7, 2], [6, 3], [7, 3],

        [0, 4], [1, 4], [0, 5], [1, 5],
        [2, 4], [3, 4], [2, 5], [3, 5],
        [0, 6], [1, 6], [0, 7], [1, 7],
        [2, 6], [3, 6], [2, 7], [3, 7],

        [4, 4], [5, 4], [4, 5], [5, 5],
        [6, 4], [7, 4], [6, 5], [7, 5],
        [4, 6], [5, 6], [4, 7], [5, 7],
        [6, 6], [7, 6], [6, 7], [7, 7]]
    );
})

test("handles Hilbert curve", () => {
    const pattern = range(64).map(i => grid(i, 8, 8, 'hilbert'));
    expect(pattern).toEqual(
        [
            [0, 0], [0, 1], [1, 1], [1, 0], [2, 0], [3, 0], [3, 1], [2, 1], [2, 2], [3, 2], [3, 3], [2, 3], [1, 3],
            [1, 2], [0, 2], [0, 3], [0, 4], [1, 4], [1, 5], [0, 5], [0, 6], [0, 7], [1, 7], [1, 6], [2, 6], [2, 7],
            [3, 7], [3, 6], [3, 5], [2, 5], [2, 4], [3, 4], [4, 4], [5, 4], [5, 5], [4, 5], [4, 6], [4, 7], [5, 7],
            [5, 6], [6, 6], [6, 7], [7, 7], [7, 6], [7, 5], [6, 5], [6, 4], [7, 4], [7, 3], [7, 2], [6, 2], [6, 3],
            [5, 3], [4, 3], [4, 2], [5, 2], [5, 1], [4, 1], [4, 0], [5, 0], [6, 0], [6, 1], [7, 1], [7, 0]
        ]
    );
})
