import type { GridType } from "../types/layout";

const grid = (i, rows, cols, layout: GridType) => {

    // (0,0) is top-left, with coordinates increasing downwards and rightwards

    let row;
    let col;
    if (layout === "row-by-row") {
        row = Math.floor(i / cols);
        col = i - (row * cols)
        return [col, row];
    }
    if (layout === "col-by-col") {
        col = Math.floor(i / rows);
        row = i - (col * rows)
        return [col, row]
    }

    if (layout === "row-snake") {
        row = Math.floor(i / cols);

        col = i - (row * cols);

        if (row % 2 === 1) {
            col = (cols - 1) - col
        }

        return [col, row];
    }

    if (layout === "col-snake") {
        col = Math.floor(i / rows);
        row = i - (col * rows)

        if (col % 2 === 1) {
            row = (rows - 1) - row
        }
        return [col, row]
    }

    if (layout === "diagonal") {
        // first, work out which stripe we are on
        let stripe;

        let remainder = i;
        for (stripe = 0; stripe <= remainder; stripe++) {
            remainder -= stripe;
        }

        row = (stripe - 1) - remainder
        col = remainder

        return [col, row]
    }


    if (layout === "diagonal-snake") {
        // first, work out which stripe we are on
        let stripe;

        let remainder = i;
        for (stripe = 0; stripe <= remainder; stripe++) {
            remainder -= stripe;
        }

        if (stripe % 2 === 1) {
            // count down along stripe from top-right corner
            row = remainder
            col = (stripe - 1) - remainder

        } else {
            // count up along stripe from lower-left corner
            row = (stripe - 1) - remainder
            col = remainder
        }

        return [col, row]
    }

    if (layout === "z-curve") {
        // N.B. - we require rows = cols  to be a power of 2
        // "A 2-dimensional space-filling curve in Z-order traverses all points in each quadrant before it enters the
        // next"
        let x = 0;
        let y = 0;
        let b = 1;

        for (let j = 0; j < i; j++) {
            // This formula for computing the next position comes from
            // Matters Computations, section 1.31.2
            b = 1;
            do {
                x ^= b;
                b &= ~x;
                y ^= b;
                b &= ~y;
                b <<= 1;
            } while (b);
        }

        return [x, y]
    }


    if (layout === "hilbert") {
        // This implementation is a literal port of the example C code on https://en.wikipedia.org/wiki/Hilbert_curve
        // N.B. - we require rows = cols  to be a power of 2

        const n = rows;
        let rx;
        let ry;
        let t = i;
        const xy = [0, 0];

        for (let s = 1; s < n; s *= 2) {
            rx = 1 & (t / 2);
            ry = 1 & (t ^ rx);
            rot(s, xy, rx, ry);

            xy[0] += (s * rx);
            xy[1] += (s * ry);
            t /= 4;
        }
        return xy;
    }
}

        function rot(n, xy, rx, ry) {
            if (ry === 0) {
                if (rx === 1) {
                    xy[0] = (n - 1 - xy[0]);
                    xy[1] = (n - 1 - xy[1]);
                }
                xy.push(xy.shift()); // swap x and y
            }
        }

export {grid};
