import { Axis, Layout } from "../types/layout";
import { max } from "d3-array";
import { applyOrderOrScale, getTicks, scale, scaleIndependently } from "./utils";

export const layoutPolarMultipleAngleScales = (data, spec, state): [Layout, Axis[]] => {
  // this has a separate scale for each concentric ring
  const layout: Layout = new Map();
  const axes: Axis[] = [];


  const rSpec = ("order" in spec.radius[0]) ? { order: spec.radius[0].order } : spec.radius[0];

  const polarLayout = new Map();
  for (const datum of data) {
    const r = applyOrderOrScale(datum, rSpec, state);

    const a = spec.angle[r];
    const angleSpec = ("order" in a) ? { order: a.order } : a;
    const n = applyOrderOrScale(datum, angleSpec, state);

    polarLayout.set(datum.id, { r, theta: n });
  }


  // TODO - check this
  const ringsWithAxes = [];
  for (const datum of data) {

    const r = applyOrderOrScale(datum, rSpec, state);
    const a = spec.angle[r];
    const angleSpec = ("order" in a) ? { order: a.order } : a;

    const maxTheta = max(Array.from(polarLayout.values()).map(l => l.theta));

    if (!ringsWithAxes.includes(r)) {
      ringsWithAxes.push(r);

      let ticks = [];
      if (!("order" in spec.radius[0])) {

        ticks = getTicks(angleSpec, state).map(t => {
          const theta = t[0] * (2 * Math.PI / maxTheta);

          return {
            x: r * Math.sin(theta),
            y: r * Math.cos(theta),
            label: t[1]
          };
        });
      }

      axes.push({
        "type": "polar",
        "x": 0,
        "y": 0,
        "r": r,
        ticks
      });
    }
  }


  scale(polarLayout, "theta", "r", ("order" in spec.angle[0]), spec.positioning, [0, 2 * Math.PI]);
  for (const datum of data) {
    const polarPos = polarLayout.get(datum.id);
    layout.set(datum.id, { x: polarPos.r * Math.sin(polarPos.theta), y: polarPos.r * Math.cos(polarPos.theta) });
  }
  return [layout, axes];
};
export const layoutPolarMultipleRadiusScales = (data, spec, state): [Layout, Axis[]] => {
  const layout: Layout = new Map();
  const axes: Axis[] = [];

  // this is old hive plot
  const angleSpec = ("order" in spec.angle[0]) ? { order: spec.angle[0].order } : spec.angle[0];

  const radialAxes = [];
  const tmp = new Map();

  const polarLayout = new Map();
  for (const datum of data) {
    const n = applyOrderOrScale(datum, angleSpec, state);

    const rn = spec.radius[n];
    const rSpec = ("scale" in rn) ? rn.scale : { order: rn.order };
    const r = applyOrderOrScale(datum, rSpec, state);

    polarLayout.set(datum.id, { r, theta: n });

    if (!radialAxes.includes(n)) {
      radialAxes.push(n);
      tmp.set(datum.id, { r: 1, theta: n, rSpec });
    }

  }

  scaleIndependently(polarLayout, "theta", ("order" in spec.radius[0]), spec.positioning, [0, 2 * Math.PI]);
  scaleIndependently(tmp, "theta", ("order" in spec.radius[0]), spec.positioning, [0, 2 * Math.PI]);


  for (const a of tmp) {
    // @ts-ignore
    const ticks = getTicks(a.rSpec, state).map(t => ({ r: t[0], theta: 0, label: t[1] }));
    scaleIndependently(ticks, "theta", ("order" in spec.radius[0]), spec.positioning, [0, 2 * Math.PI]);

    axes.push({
      type: "cartesian",
      x: 0,
      y: 0,
      // @ts-ignore
      x2: a.x,
      // @ts-ignore
      y2: a.y,
      extend: "radial",
      ticks
    });
  }

  for (const datum of data) {
    const polarPos = polarLayout.get(datum.id);
    layout.set(datum.id, { x: polarPos.r * Math.sin(polarPos.theta), y: polarPos.r * Math.cos(polarPos.theta) });
  }


  return [layout, axes];
};
export const layoutPolar = (data, spec, state): [Layout, Axis[]] => {
  const layout: Layout = new Map();
  const axes: Axis[] = [];

  const angleSpec = ("order" in spec.angle[0]) ? { order: spec.angle[0].order } : spec.angle[0];

  const ringsWithAxes = [];

  const polarLayout = new Map();
  for (const datum of data) {
    const i = applyOrderOrScale(datum, angleSpec, state);

    const rSpec = ("order" in spec.radius[0]) ? { order: spec.radius[0].order } : spec.radius[0];
    const r = applyOrderOrScale(datum, rSpec, state);

    polarLayout.set(datum.id, { r, theta: i });

    if (!ringsWithAxes.includes(r)) {
      ringsWithAxes.push(r);

      const ticks = getTicks(rSpec, state).map(t => ({ r: t[0], theta: 0, label: t[1] }));
      scaleIndependently(ticks, "theta", ("order" in spec.radius[0]), spec.positioning, [0, 2 * Math.PI]);

      axes.push({
        "type": "polar",
        "x": 0,
        "y": 0,
        "r": r,
        ticks
      });
    }
  }

  scaleIndependently(polarLayout, "theta", ("order" in spec.radius[0]), spec.positioning, [0, 2 * Math.PI]);
  for (const datum of data) {
    const polarPos = polarLayout.get(datum.id);
    layout.set(datum.id, { x: polarPos.r * Math.sin(polarPos.theta), y: polarPos.r * Math.cos(polarPos.theta) });
  }

  return [layout, axes];

};