import { MapSpec } from "../types/spec";

// TODO: need to compute a layout
// or create function that will be used for positioning


const processMapSpec = (spec: MapSpec) => {
  const settings = {
    "Mapnik": {
      url: "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
      options: {
        maxZoom: 19,
        attribution: "&copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors"

      }
    },

    "Stadia.AlidadeSmooth": {
      url: "https://tiles.stadiamaps.com/tiles/alidade_smooth/{z}/{x}/{y}{r}.png",
      options: {
        maxZoom: 20,
        attribution: "&copy; <a href=\"https://stadiamaps.com/\">Stadia Maps</a>, &copy; <a href=\"https://openmaptiles.org/\">OpenMapTiles</a> &copy; <a href=\"http://openstreetmap.org\">OpenStreetMap</a> contributors"
      }
    },

    "Stamen_Terrain": {
      url: "https://stamen-tiles-{s}.a.ssl.fastly.net/terrain/{z}/{x}/{y}{r}.{ext}",
      options: {
        attribution: "Map tiles by <a href=\"http://stamen.com\">Stamen Design</a>, <a href=\"http://creativecommons.org/licenses/by/3.0\">CC BY 3.0</a> &mdash; Map data &copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors",
        subdomains: "abcd",
        minZoom: 0,
        maxZoom: 18,
        ext: "png"
      }
    },

    "Stamen_TerrainBackground": {
      url: "https://stamen-tiles-{s}.a.ssl.fastly.net/terrain-background/{z}/{x}/{y}{r}.{ext}",
      options: {
        attribution: "Map tiles by <a href=\"http://stamen.com\">Stamen Design</a>, <a href=\"http://creativecommons.org/licenses/by/3.0\">CC BY 3.0</a> &mdash; Map data &copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors",
        subdomains: "abcd",
        minZoom: 0,
        maxZoom: 18,
        ext: "png"
      }
    },

    "Stamen_TonerBackground": {
      url: "https://stamen-tiles-{s}.a.ssl.fastly.net/toner-background/{z}/{x}/{y}{r}.{ext}",
      options: {
        attribution: "Map tiles by <a href=\"http://stamen.com\">Stamen Design</a>, <a href=\"http://creativecommons.org/licenses/by/3.0\">CC BY 3.0</a> &mdash; Map data &copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors",
        subdomains: "abcd",
        minZoom: 0,
        maxZoom: 20,
        ext: "png"
      }
    },

    "Stamen_Watercolor": {
      url: "https://stamen-tiles-{s}.a.ssl.fastly.net/watercolor/{z}/{x}/{y}.{ext}",
      options: {
        attribution: "Map tiles by <a href=\"http://stamen.com\">Stamen Design</a>, <a href=\"http://creativecommons.org/licenses/by/3.0\">CC BY 3.0</a> &mdash; Map data &copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors",
        subdomains: "abcd",
        minZoom: 1,
        maxZoom: 16,
        ext: "jpg"
      }
    }


  };


  const center = spec.center ?? [52.44, 4.83];
  const zoomLevel = spec.zoomLevel ?? 14;

  if ("tiles" in spec) {
    const options = settings[spec.tiles].options;
    options.zoomSnap = 0.5;
    return { url: settings[spec.tiles].url, options, center, zoomLevel };
  }

  const options = {};
  const fields = ["attribution", "accessToken", "minZoom", "maxZoom", "subdomains"];

  for (const field of fields) {
    if (field in spec) {
      options[field] = spec[field];
    }
  }


  return { url: spec.url, options, center, zoomLevel };

};

export { processMapSpec };