import { aggregateEdges } from "./aggregateEdges";
import { constructNetwork } from "./creation/constructNetwork";

test("aggregates edges", () => {

  const links = [
    { source: 1, target: 2, data: "link 1" },
    { source: 1, target: 2, data: "link 2" },
    { source: 4, target: 2, data: "link 3" },
    { source: 4, target: 2, data: "link 4" },
    { source: 1, target: 3, data: "link 5" }

  ];

  const nodes = [
    { id: 1 }, { id: 2 }, { id: 3 }, { id: 4 }
  ];

  const data = { nodes, links };

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: false,
    source_node: ["id", "source"], // node.id == link.source_id
    target_node: ["id", "target"]
  });


  const aggregated = aggregateEdges(network.links);
  expect(aggregated).toEqual([
      {
        source: { id: 1 },
        target: { id: 2 },
        data: ["link 1", "link 2"]
      },
      { source: { id: 1 }, target: { id: 3 }, data: ["link 5"] },
      {
        source: { id: 4 },
        target: { id: 2 },
        data: ["link 3", "link 4"]
      }
    ]
  );

});



test("use a specified aggregation function", () => {
  const links = [
    { source: 1, target: 2, data: {val: 1} },
    { source: 1, target: 2, data: {val: 2} },
    { source: 4, target: 2, data: {val: 3} },
    { source: 4, target: 2, data: {val: 4} },
    { source: 1, target: 3, data: {val: 5} }

  ];

  const nodes = [
    { id: 1 }, { id: 2 }, { id: 3 }, { id: 4 }
  ];

  const data = { nodes, links };

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: false,
    source_node: ["id", "source"], // node.id == link.source_id
    target_node: ["id", "target"]
  });

  const aggregated = aggregateEdges(network.links, { operation: "count" });

  expect(aggregated).toEqual([
      { source: { id: 1 },  target: { id: 2 }, data: 2 },
      { source: { id: 1 }, target: { id: 3 }, data: 1 },
      { source: { id: 4 }, target: { id: 2 }, data: 2 }
    ]
  );
});


