import type {NetworkLink} from "../types/network";
import type {AggFuncType} from "../types/data-transforms";
import type { AggregateEdgesSpec } from "../types/networkTransforms";

import { aggregationFunctions } from "../data/aggregation";

const defaultAggregateLinks = (edgeData: any[]) => edgeData;

const aggregateEdges = (links: NetworkLink[], spec?: AggregateEdgesSpec) => {

  type AgFuncType2 = AggFuncType | ((edgeData: any[]) => any);

  let aggregateLinks: AgFuncType2 = defaultAggregateLinks;
  let directed = false;
  if (spec) {
    aggregateLinks = spec.operation ? aggregationFunctions[spec.operation] : aggregateLinks;
    directed = spec.directed ?? directed;
  }

  const groupedLinks = new Map();

  for (const link of links) {

    let sourceId = link.source.id;
    let targetId = link.target.id;

    if (!directed) {
      const s = (sourceId <= targetId) ?  sourceId : targetId;
      const t = (sourceId <= targetId) ? targetId : sourceId;

      sourceId = s;
      targetId = t;
    }

    if (!groupedLinks.has(sourceId)) {
      groupedLinks.set(sourceId, new Map<string | number, NetworkLink[]>());
    }

    if (!groupedLinks.get(sourceId).has(targetId)) {
      groupedLinks.get(sourceId).set(targetId, []);
    }

    groupedLinks.get(sourceId).get(targetId).push(link);
  }

  const outputLinks = [];

  for (const [, linksWithSource] of groupedLinks) {
    if (!linksWithSource) {
      continue;
    }
    for (const [, linksWithSourceAndTarget] of linksWithSource) {
      if (!linksWithSourceAndTarget) {
        continue;
      }
      outputLinks.push({
        source: linksWithSourceAndTarget[0].source,
        target: linksWithSourceAndTarget[0].target,
        id: linksWithSourceAndTarget[0].id,
        data: aggregateLinks(linksWithSourceAndTarget.map(link => link.data))
      });
    }
  }

  return outputLinks;

};

export { aggregateEdges };


