import { constructNetwork } from "./creation/constructNetwork";
import { aggregateNodes } from "./aggregateNodes";

  let data = {};

  data.nodes = [
    { id: 0, name: "a", type: "FOO" },
    { id: 1, name: "b", type: "FOO" },
    { id: 2, name: "c", type: "BAR" },
    { id: 3, name: "c", type: "BAR" }
  ];

  data.links = [
    { source_id: 0, target_id: 1, weight: 6 },
    { source_id: 1, target_id: 0, weight: 3 },
    { source_id: 1, target_id: 2, weight: 7 },
    { source_id: 3, target_id: 2, weight: 7 },
    { source_id: 3, target_id: 1, weight: 7 }
  ];

test.skip("can aggregate nodes based on a single field (specified in 'field' attribute)" , () => {

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"]
  });

  aggregateNodes(network, { field: "type" });


  expect(network).toEqual({
      "nodes": [
        {
          "id": 0,
          "type": "FOO",
          "count": 2
        },
        {
          "id": 2,
          "type": "BAR",
          "count": 2
        }
      ],
      "links": [
        {
          "source_id": 0,
          "target_id": 1,
          "weight": 6,
          "source": {
            "id": 0,
            "type": "FOO",
            "count": 2
          },
          "target": {
            "id": 0,
            "type": "FOO",
            "count": 2
          }
        },
        {
          "source_id": 1,
          "target_id": 0,
          "weight": 3,
          "source": {
            "id": 0,
            "type": "FOO",
            "count": 2
          },
          "target": {
            "id": 0,
            "type": "FOO",
            "count": 2
          }
        },
        {
          "source_id": 1,
          "target_id": 2,
          "weight": 7,
          "source": {
            "id": 0,
            "type": "FOO",
            "count": 2
          },
          "target": {
            "id": 2,
            "type": "BAR",
            "count": 2
          }
        },
        {
          "source_id": 3,
          "target_id": 2,
          "weight": 7,
          "source": {
            "id": 2,
            "type": "BAR",
            "count": 2
          },
          "target": {
            "id": 2,
            "type": "BAR",
            "count": 2
          }
        },
        {
          "source_id": 3,
          "target_id": 1,
          "weight": 7,
          "source": {
            "id": 2,
            "type": "BAR",
            "count": 2
          },
          "target": {
            "id": 0,
            "type": "FOO",
            "count": 2
          }
        }
      ]
    }
  );

});


test.skip("can aggregate nodes based on a single field (specified in 'fields' array)", () => {

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"]
  });

  aggregateNodes(network, { fields: ["type"], as: "type" });


  expect(network).toEqual({
      "nodes": [
        {
          "id": 0,
          "type": ["FOO"],
          "count": 2
        },
        {
          "id": 2,
          "type": ["BAR"],
          "count": 2
        }
      ],
      "links": [
        {
          "source_id": 0,
          "target_id": 1,
          "weight": 6,
          "source": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          },
          "target": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          }
        },
        {
          "source_id": 1,
          "target_id": 0,
          "weight": 3,
          "source": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          },
          "target": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          }
        },
        {
          "source_id": 1,
          "target_id": 2,
          "weight": 7,
          "source": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          },
          "target": {
            "id": 2,
            "type": ["BAR"],
            "count": 2
          }
        },
        {
          "source_id": 3,
          "target_id": 2,
          "weight": 7,
          "source": {
            "id": 2,
            "type": ["BAR"],
            "count": 2
          },
          "target": {
            "id": 2,
            "type": ["BAR"],
            "count": 2
          }
        },
        {
          "source_id": 3,
          "target_id": 1,
          "weight": 7,
          "source": {
            "id": 2,
            "type": ["BAR"],
            "count": 2
          },
          "target": {
            "id": 0,
            "type": ["FOO"],
            "count": 2
          }
        }
      ]
    }
  );

});


test.skip("can aggregate nodes based on a multiple fields (specified in 'fields' array)", () => {

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"]
  });

  aggregateNodes(network, { fields: ["type", "name"], as: "type" });

  expect(network).toEqual({
      "nodes": [
        { "id": 0, "type": ["FOO", "a"], "count": 1 },
        { "id": 1, "type": ["FOO", "b"], "count": 1 },
        { "id": 2, "type": ["BAR", "c"], "count": 2 }
      ],
      "links": [
        {
        "source_id": 0,
        "target_id": 1,
        "weight": 6,
        "source": { "id": 0, "type": ["FOO", "a"], "count": 1 },
        "target": { "id": 1, "type": ["FOO", "b"], "count": 1 }
      }, {
        "source_id": 1,
        "target_id": 0,
        "weight": 3,
        "source": { "id": 1, "type": ["FOO", "b"], "count": 1 },
        "target": { "id": 0, "type": ["FOO", "a"], "count": 1 }
      }, {
        "source_id": 1,
        "target_id": 2,
        "weight": 7,
        "source": { "id": 1, "type": ["FOO", "b"], "count": 1 },
        "target": { "id": 2, "type": ["BAR", "c"], "count": 2 }
      }, {
        "source_id": 3,
        "target_id": 2,
        "weight": 7,
        "source": { "id": 2, "type": ["BAR", "c"], "count": 2 },
        "target": { "id": 2, "type": ["BAR", "c"], "count": 2 }
      }, {
        "source_id": 3,
        "target_id": 1,
        "weight": 7,
        "source": { "id": 2, "type": ["BAR", "c"], "count": 2 },
        "target": { "id": 1, "type": ["FOO", "b"], "count": 1 }
      }]
    }
  );
});
