import { select } from "../data/select";
import { constructExpressionFunction } from "../data/calculate";
import { NetworkNode } from "../types/network";
import { AggregateNodeSpec } from "../types/networkTransforms";

import {zip} from "d3-array";

const aggregateNodes = (network, spec: AggregateNodeSpec) => {
  let key;
  let commonFieldName;
  if ("field" in spec){
    key = (d) => select(d, spec.field);
    commonFieldName = spec.field;
  } else if ("fields" in spec) {
    key = (d) => JSON.stringify(spec.fields.map(fieldName => select(d, fieldName)));
    commonFieldName = spec.as ?? "key";
  } else {
    key = constructExpressionFunction({calculate: spec.expression});
    commonFieldName = spec.as ?? "key";
  }

  // Calculate the key value for each node
  const nodesForKey = new Map<string, NetworkNode[]>(); // store list of nodes for each value of the key

  for (const node of network.nodes){
    const keyVal = key(node);

    if (!nodesForKey.has(keyVal)){
      nodesForKey.set(keyVal, []);
    }

    nodesForKey.get(keyVal).push(node);
  }

  console.log({nodesForKey})

  // Create new node
  const newNodes = [];
  const replacement = new Map<number|string, NetworkNode>(); // mapping from old node id to new node object

  for (const [keyVal, nodeList] of nodesForKey){

    let commonFieldVal;
    if ("fields" in spec){
      // store the values of the fields used for grouping as an object
      commonFieldVal = Object.fromEntries(zip(spec.fields, JSON.parse(keyVal)));
    } else {
      commonFieldVal = keyVal;
    }


    const thisId = isNaN(parseInt(keyVal)) ? nodeList[0].id : parseInt(keyVal);

    const mergedNode = {id: thisId, [commonFieldName]: commonFieldVal, count: nodeList.length};
    newNodes.push(mergedNode);

    for (const node of nodeList){
      replacement.set(node.id, mergedNode);
    }
  }

  // Update links to point to new nodes
  for (const link of network.links){
    link.source =replacement.get(link.source.id);
    link.target = replacement.get(link.target.id)
  }

  network.nodes = newNodes;

}

export {aggregateNodes};