import createGraph from "ngraph.graph";

// @ts-ignore
import detectClusters from "ngraph.louvain";

import type { NetworkLink, NetworkNode } from "../types/network";
import type { ClusterSpec } from "../types/networkTransforms";

const calculateClusters = (linkData: NetworkLink[], nodeData: NetworkNode[], spec: ClusterSpec) => {

  // construct network once
  const g = createGraph();

  for (const link of linkData) {
    const source = typeof (link.source) === "object" ? link.source.id : link.source;
    const target = typeof (link.target) === "object" ? link.target.id : link.target;
    g.addLink(source, target);
  }

  const name = spec.as || spec.method;

  // The clustering method results in the assignment of non-contiguous cluster IDs
  // We convert these to sequential integers.
  let numClasses =0;
  const clusterNumber = {};

  if (spec.method === "louvain") {
    const clusters = detectClusters(g);
    for (const node of nodeData) {
      const clusterId: number = clusters.getClass(node.id);
      node[name] = clusterId

      if (!(clusterId in clusterNumber)) {
        clusterNumber[clusterId] = numClasses;
        numClasses++;
      }

      node[name] = clusterNumber[clusterId];
    }
  }

};

export { calculateClusters };
