import { constructNetwork } from "./creation/constructNetwork";
import { connectNodes } from "./connectNodes";

test("set link types correctly ", () => {

  let network = {
    nodes: [
      { id: 0, name: "a", type: "FOO" },
      { id: 1, name: "b", type: "FOO" },
      { id: 2, name: "c", type: "BAR" },
      { id: 3, name: "d", type: "BAR" },
      { id: 4, name: "e", type: "BAR" }
    ],
    links: []
  };

  const links = connectNodes(network.nodes, network.links, { field: "type", as: "sameType" });
  expect(links.map(l => l._type)).toEqual(["sameType", "sameType", "sameType", "sameType"]);
});

test("can link based on a field, with one link per pair of nodes ", () => {

  let network = {
    nodes: [
      { id: 0, name: "a", type: "FOO" },
      { id: 1, name: "b", type: "FOO" },
      { id: 2, name: "c", type: "BAR" },
      { id: 3, name: "d", type: "BAR" },
      { id: 4, name: "e", type: "BAR" }
    ],
    links: []
  };

  const links = connectNodes(network.nodes, network.links, { field: "type" });

  expect(links.map(l => ({ s: l.source.id, t: l.target.id }))).toEqual([
    { s: 0, t: 1 },

    { s: 2, t: 3 },
    { s: 2, t: 4 },
    { s: 3, t: 4 }
  ]);
});


test("can link based on a field, with 2 links per pair of nodes ", () => {

  let network = {
    nodes: [
      { id: 0, name: "a", type: "FOO" },
      { id: 1, name: "b", type: "FOO" },
      { id: 2, name: "c", type: "BAR" },
      { id: 3, name: "d", type: "BAR" },
      { id: 4, name: "e", type: "BAR" }
    ],
    links: []
  };

  const links = connectNodes(network.nodes, network.links, { field: "type", bothDirections: true });

  expect(links.map(l => ({ s: l.source.id, t: l.target.id }))).toEqual([
    { s: 0, t: 1 },
    { s: 1, t: 0 },
    { s: 2, t: 3 },
    { s: 2, t: 4 },
    { s: 3, t: 2 },
    { s: 3, t: 4 },
    { s: 4, t: 2 },
    { s: 4, t: 3 }
  ]);


});

test("can link based on an expression, with one link per pair of nodes ", () => {

  let network = {
    nodes: [
      { id: 0, name: "a", type: "FOO" },
      { id: 1, name: "b", type: "FOO" },
      { id: 2, name: "c", type: "BAR" },
      { id: 3, name: "d", type: "BAR" },
      { id: 4, name: "e", type: "BAR" }
    ],
    links: []
  };
  const links = connectNodes(network.nodes, network.links, { expression: "datum.type" });

  expect(links.map(l => ({ s: l.source.id, t: l.target.id }))).toEqual([
    { s: 0, t: 1 },

    { s: 2, t: 3 },
    { s: 2, t: 4 },
    { s: 3, t: 4 }
  ]);
});
