import { constructExpressionFunction } from "../data/calculate";
import { select } from "../data/select";
import type { ConnectSpec } from "../types/networkTransforms";


const connectNodes = (nodes, links, spec: ConnectSpec, state) => {
  let getVal;
  let linkType: string;

  if ("field" in spec) {
    getVal = (node) => select(node, spec.field);
    linkType = spec.as ?? spec.field;
  } else if ("expression" in spec) {
    const f = constructExpressionFunction({ calculate: spec.expression });
    getVal = node => f(node, null, state);
    linkType = spec.as ?? "commonExpression";
  }

  const vals = new Map<any, any[]>();
  for (const node of nodes) {
    const key = getVal(node);

    if (vals.has(key)) {
      vals.get(key).push(node);
    } else {
      vals.set(key, [node]);
    }
  }

  for (const key of vals.keys()) {
    for (const n1 of vals.get(key)) {
      for (const n2 of vals.get(key)) {
        if (n1.id !== n2.id && (n1.id < n2.id || spec?.bothDirections)) {
          links.push({ source: n1, target: n2, _type: linkType }); // TODO: need to set a link ID!
        }
      }
    }
  }

  return links;
};

export { connectNodes };
