import { select } from "../data/select";

import {aggregationFunctions} from "../data/aggregation";
import { ConnectivityBasedAttributesSpec } from "../types/networkTransforms";

const computeConnectivityBasedAttribute = (nodes, links, spec: ConnectivityBasedAttributesSpec) => {
  // N.B. selector can refer to either node or link data

  const attributeSelector = spec.attributeSelector;
  const direction = spec.direction ?? "both";
  const aggregationOperation = spec.operation ?? "count";
  const computedAttributeName = spec.as;

  const values = new Map();

  for (const link of links) {

    if (direction !== "out") {
      const nodeWithData = link.source;
      const otherNode = link.target;

      const value = select({ link, node: nodeWithData }, attributeSelector);

      if (!values.has(otherNode)) {
        values.set(otherNode, []);
      }

      values.get(otherNode).push(value);
    }

    if (direction !== "in") {
      const nodeWithData = link.target;
      const otherNode = link.source;
      const value = select({ link, node: nodeWithData }, attributeSelector);

      if (!values.has(otherNode)) {
        values.set(otherNode, []);
      }

      values.get(otherNode).push(value);
    }
  }

  const aggregationFn = aggregationFunctions[aggregationOperation];
  for (const node of nodes){

    if (typeof(node.data) === "undefined"){
      node.data = {};
    }

    node.data[computedAttributeName] = aggregationFn(values.get(node))
  }
};

export { computeConnectivityBasedAttribute };