import { constructNetwork } from "./constructNetwork";

test("can construct network", () => {

  let data = {};

  data.nodes = [
    { id: 0, name: "foo" },
    { id: 1, name: "bar" },
    { id: 2, name: "foo-bar" }
  ];

  data.links = [
    { source_id: 0, target_id: 1 },
    { source_id: 1, target_id: 2 },
    { source_id: 1, target_id: 0 }

  ];

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"]
  });


  expect(network).toEqual(
    {
      "nodes": [
        {
          "id": 0,
          "name": "foo"
        },
        {
          "id": 1,
          "name": "bar"
        },
        {
          "id": 2,
          "name": "foo-bar"
        }
      ],
      "links": [
        {
          "source_id": 0,
          "target_id": 1,
          "directed": true,
          "source": {
            "id": 0,
            "name": "foo"
          },
          "target": {
            "id": 1,
            "name": "bar"
          }
        },
        {
          "source_id": 1,
          "target_id": 2,
          "directed": true,
          "source": {
            "id": 1,
            "name": "bar"
          },
          "target": {
            "id": 2,
            "name": "foo-bar"
          }
        },
        {
          "source_id": 1,
          "target_id": 0,
          "directed": true,
          "source": {
            "id": 1,
            "name": "bar"
          },
          "target": {
            "id": 0,
            "name": "foo"
          }
        }
      ]
    }
  );

});

test("can construct network with calculated metrics", () => {

  let data = {};

  data.nodes = [
    { id: 0, name: "foo" },
    { id: 1, name: "bar" },
    { id: 2, name: "foo-bar" }
  ];

  data.links = [
    { source_id: 0, target_id: 1 },
    { source_id: 1, target_id: 2 },
    { source_id: 2, target_id: 0 }

  ];

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"],

    metrics: [{ metric: "in-degree" }]
  });


  expect(network).toEqual( {
          "nodes": [
              {
                  "id": 0,
                  "name": "foo",
                  "in-degree": 1
              },
              {
                  "id": 1,
                  "name": "bar",
                  "in-degree": 1
              },
              {
                  "id": 2,
                  "name": "foo-bar",
                  "in-degree": 1
              }
          ],
          "links": [
              {
                  "source_id": 0,
                  "target_id": 1,
                  "directed": true,
                  "source": {
                      "id": 0,
                      "name": "foo",
                      "in-degree": 1
                  },
                  "target": {
                      "id": 1,
                      "name": "bar",
                      "in-degree": 1
                  }
              },
              {
                  "source_id": 1,
                  "target_id": 2,
                  "directed": true,
                  "source": {
                      "id": 1,
                      "name": "bar",
                      "in-degree": 1
                  },
                  "target": {
                      "id": 2,
                      "name": "foo-bar",
                      "in-degree": 1
                  }
              },
              {
                  "source_id": 2,
                  "target_id": 0,
                  "directed": true,
                  "source": {
                      "id": 2,
                      "name": "foo-bar",
                      "in-degree": 1
                  },
                  "target": {
                      "id": 0,
                      "name": "foo",
                      "in-degree": 1
                  }
              }
          ]
      }
)

});

