
import { importPajek, parsePajek } from "./importPajek";
//import fetchMock from "jest-fetch-mock";


beforeEach(() => {
  fetchMock.resetMocks();
  fetchMock.doMock();
});

test("can parse an undirected pajek file with node labels", () => {

  const example = `
  *Vertices 4
1 "A"
2 "B"
3 "C"
4 "D"
*Edges
1 2 1
1 3 1
2 3 1
1 4 1
`;

  const network = parsePajek(example, {format: "pajek"});

  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "A"
        },
        {
          "id": "2",
          "label": "B"
        },
        {
          "id": "3",
          "label": "C"
        },
        {
          "id": "4",
          "label": "D"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": false
        },
        {
          "source": {
            "id": "2",
            "label": "B"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": false
        }
      ]
    }
  );

});

test("can parse a directed pajek file with node labels", () => {

  const example = `
  *Vertices 4
1 "A"
2 "B"
3 "C"
4 "D"
*Arcs
1 2 1
1 3 1
2 3 1
1 4 1
`;

  const network = parsePajek(example, {format: "pajek"});

  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "A"
        },
        {
          "id": "2",
          "label": "B"
        },
        {
          "id": "3",
          "label": "C"
        },
        {
          "id": "4",
          "label": "D"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": true
        },
        {
          "source": {
            "id": "2",
            "label": "B"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": true
        }
      ]
    }
  );

});


test("can parse a pajek file with no node details", () => {

  const example = `
  *Vertices 4
*Arcs
1 2 1
1 3 1
2 3 1
1 4 1
`;

  const network = parsePajek(example, {format: "pajek"});

  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "1"
        },
        {
          "id": "2",
          "label": "2"
        },
        {
          "id": "3",
          "label": "3"
        },
        {
          "id": "4",
          "label": "4"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "1"
          },
          "target": {
            "id": "2",
            "label": "2"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "1"
          },
          "target": {
            "id": "3",
            "label": "3"
          },
          "directed": true
        },
        {
          "source": {
            "id": "2",
            "label": "2"
          },
          "target": {
            "id": "3",
            "label": "3"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "1"
          },
          "target": {
            "id": "4",
            "label": "4"
          },
          "directed": true
        }
      ]
    }
  );
});

test("can parse a pajeck file with an arcslist", () => {

  const example = `
  *Vertices 5
1 "A"
2 "B"
3 "C"
4 "D"
5 "E"
*Arcslist
1 2 3 4
2 1
3 4 2
4 5
`;

  const network = parsePajek(example, {format: "pajek"});


  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "A"
        },
        {
          "id": "2",
          "label": "B"
        },
        {
          "id": "3",
          "label": "C"
        },
        {
          "id": "4",
          "label": "D"
        },
        {
          "id": "5",
          "label": "E"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": true
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": true
        },
        {
          "source": {
            "id": "2",
            "label": "B"
          },
          "target": {
            "id": "1",
            "label": "A"
          },
          "directed": true
        },
        {
          "source": {
            "id": "3",
            "label": "C"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": true
        },
        {
          "source": {
            "id": "3",
            "label": "C"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": true
        },
        {
          "source": {
            "id": "4",
            "label": "D"
          },
          "target": {
            "id": "5",
            "label": "E"
          },
          "directed": true
        }
      ]
    }
  );
});

test("can parse a pajeck file with an edgeslist", () => {

  const example = `
  *Vertices 5
1 "A"
2 "B"
3 "C"
4 "D"
5 "E"
*Edgeslist
1 2 3 4
2 1
3 4 2
4 5
`;

  const network = parsePajek(example, {format: "pajek"});


  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "A"
        },
        {
          "id": "2",
          "label": "B"
        },
        {
          "id": "3",
          "label": "C"
        },
        {
          "id": "4",
          "label": "D"
        },
        {
          "id": "5",
          "label": "E"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": false
        },
        {
          "source": {
            "id": "2",
            "label": "B"
          },
          "target": {
            "id": "1",
            "label": "A"
          },
          "directed": false
        },
        {
          "source": {
            "id": "3",
            "label": "C"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": false
        },
        {
          "source": {
            "id": "3",
            "label": "C"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": false
        },
        {
          "source": {
            "id": "4",
            "label": "D"
          },
          "target": {
            "id": "5",
            "label": "E"
          },
          "directed": false
        }
      ]
    }
  );
});

test("can parse a pajeck file with an arcslist", () => {

  const example = `
  *Vertices 5
1 "A"
2 "B"
3 "C"
4 "D"
5 "E"
*Edgeslist
1 2 3 4
2 1
3 4 2
4 5
`;

  const network = parsePajek(example, {format: "pajek"});
});


test("can fetch a pajek file from a URL and load it", async () => {

  const example = `
  *Vertices 4
1 "A"
2 "B"
3 "C"
4 "D"
*Edges
1 2 1
1 3 1
2 3 1
1 4 1
`;

  const url = "https://example.com/pajek-file";
  fetchMock.doMockIf(url, example);

  const network = await importPajek({
    url
  });

  expect(network).toEqual({
      "nodes": [
        {
          "id": "1",
          "label": "A"
        },
        {
          "id": "2",
          "label": "B"
        },
        {
          "id": "3",
          "label": "C"
        },
        {
          "id": "4",
          "label": "D"
        }
      ],
      "links": [
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "2",
            "label": "B"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": false
        },
        {
          "source": {
            "id": "2",
            "label": "B"
          },
          "target": {
            "id": "3",
            "label": "C"
          },
          "directed": false
        },
        {
          "source": {
            "id": "1",
            "label": "A"
          },
          "target": {
            "id": "4",
            "label": "D"
          },
          "directed": false
        }
      ]
    }
  );

});
