import { Network, NetworkImportPajek } from "../../types/network";

export async function importPajek(spec: NetworkImportPajek) {
  const data = await fetch(spec.url)
    .then((response) => response.text());

  return parsePajek(data, spec);

}

export function parsePajek(data: string, spec: NetworkImportPajek): Network {
  const lines = data.split("\n");

  const nodes = [];
  const nodeRefs = {};
  const links = [];
  let numVertices = 0;

  let lineType: "nodes" | "singleLinks" | "listOfLinks" | undefined = undefined;
  let line;
  let directed = false;

  for (let i = 0; i < lines.length; i++) {
    line = lines[i].trim();

    if (line.toLowerCase().startsWith("*vertices")) {
      lineType = "nodes";
      numVertices = line.split(" ")[1].trim();
      continue;
    } else if (line.toLowerCase().startsWith("*arcslist")) {
      directed = true;
      lineType = "listOfLinks";
      continue;
    } else if (line.toLowerCase().startsWith("*edgeslist")) {
      directed = false;
      lineType = "listOfLinks";
      continue;
    } else if (line.toLowerCase().startsWith("*arcs")) {
      lineType = "singleLinks";
      directed = true;
      continue;
    } else if (line.toLowerCase().startsWith("*edges")) {
      lineType = "singleLinks";
      directed = false;
      continue;
    }


    // prepare and clean line
    line = line.trim();
    line = line.split(" ");
    for (let j = 0; j < line.length; j) {
      if (line[j].length == 0) {
        line.splice(j, 1);
      } else {
        j++;
      }
    }
    if (line.length == 0) continue;

    // parse data
    if (lineType !== "nodes" && nodes.length == 0) {
      // nodes are only defined implicitly
      for (let i = 1; i <= numVertices; i++) {
        const node = { id: i.toString(), label: i.toString() };
        nodes.push(node);
        nodeRefs[i] = node;
      }
    }


    if (lineType === "nodes") {
      const node = { id: line[0], label: line[1].replace(/"/g, "") };
      nodes.push(node);
      nodeRefs[line[0]] = node;
    } else if (lineType === "singleLinks") {

      if (spec.addReverseLinks && !directed) {
        links.push({
          source: nodeRefs[line[0]],
          target: nodeRefs[line[1]],
          directed: true
        });
        links.push({
          source: nodeRefs[line[1]],
          target: nodeRefs[line[0]],
          directed: true
        });

      } else {
        links.push({
          source: nodeRefs[line[0]],
          target: nodeRefs[line[1]],
          directed
        });
      }

    } else if (lineType === "listOfLinks") {
      for (let i = 1; i < line.length; i++) {

        if (spec.addReverseLinks && !directed) {
          links.push({
            source: nodeRefs[line[0]],
            target: nodeRefs[line[i]],
            directed: true
          });

          links.push({
            source: nodeRefs[line[1]],
            target: nodeRefs[line[0]],
            directed: true
          });

        } else {
          links.push({
            source: nodeRefs[line[0]],
            target: nodeRefs[line[i]],
            directed
          });
        }

      }
    }
  }

  return { nodes, links };
}