import { filterEdges } from "./filterEdges";
import { constructNetwork } from "./creation/constructNetwork";

test("removes isolated nodes", () => {

  let data = {};

  data.nodes = [
    { id: 0, name: "foo" },
    { id: 1, name: "bar" },
    { id: 2, name: "foo-bar" }
  ];

  data.links = [
    { source_id: 0, target_id: 1, weight: 6 },
    { source_id: 1, target_id: 0, weight: 3 },
    { source_id: 1, target_id: 2, weight: 7 }

  ];

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: true,
    source_node: ["id", "source_id"], // node.id == link.source_id
    target_node: ["id", "target_id"]
  });

  expect(network.nodes).toEqual([
    { id: 0, name: "foo" },
    { id: 1, name: "bar" },
    { id: 2, name: "foo-bar" }
  ]);

  const filteredLinks = filterEdges(network.links, { "expression": "datum.weight > 4" });

  expect(filteredLinks).toEqual([
      {
        source_id: 0,
        target_id: 1,
        weight: 6,
        directed: true,
        source: { id: 0, name: "foo" },
        target: { id: 1, name: "bar" }
      },
      {
        source_id: 1,
        target_id: 2,
        weight: 7,
        directed: true,
        source: { id: 1, name: "bar" },
        target: { id: 2, name: "foo-bar" }
      }
    ]
  );

});