import { select } from "../data/select";
import { NetworkLink, NetworkNode } from "../types/network";

type NetworkGroup = {
  id: number;
  // size: number;
  nodes: NetworkNode[];
  links: NetworkLink[];

  key?: any;
}


const groupNetwork = (data: any, networkName: string, fields: string[]) => {
  const grouped: Map<string, NetworkGroup> = new Map();

  //const dataset = data[datasetName];

  const network = select(data, networkName);
  let id = 0;

  for (const d of network.nodes) {
    // we stringify as otherwise different arrays containing same values would be considered different keys
    const key = JSON.stringify(fields.map(field => select(d, field)));
    const thisId = isNaN(parseInt(key)) ? id : parseInt(key);

    if (grouped.has(key)) {
      grouped.get(key).nodes.push(d);
    } else {
      grouped.set(key, { id: thisId, nodes: [d], links: [], key: JSON.parse(key) });
      id++;
    }
  }

  const interGroupLinks = [];
  for (const l of network.links) {
    const sourceKey = JSON.stringify(fields.map(field => select(l.source, field)));
    const targetKey = JSON.stringify(fields.map(field => select(l.target, field)));

    if (sourceKey === targetKey) {
      grouped.get(sourceKey).links.push(l);
    } else {
      interGroupLinks.push(l);
    }
  }


  return {groups: Array.from(grouped.values()), interGroupLinks};

};

export { groupNetwork };

