import {calculateMetrics} from "./metrics";


const nodes = [
    {id: 'a'},
    {id: 'b'},
    {id: 'c'},
    {id: 'd'},
];

const links = [
    {source: 'a', target: 'b'},
    {source: 'a', target: 'b'},
    {source: 'a', target: 'c'},
    {source: 'a', target: 'd'},
    {source: 'a', target: 'e'},

    {source: 'b', target: 'a'},
    {source: 'b', target: 'a'},
    {source: 'c', target: 'b'},
    {source: 'c', target: 'c'},
    {source: 'd', target: 'd'},

]

test("degree", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'degree', as: 'degree'}]);

        expect(nodes).toEqual([
                {id: 'a', degree: 5},
                {id: 'b', degree: 3},
                {id: 'c', degree: 3},
                {id: 'd', degree: 2}
            ]
        )

    }
)

test("indegree", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'in-degree', as: 'indegree'}]);

        expect(nodes).toEqual([
                {id: 'a', indegree: 1},
                {id: 'b', indegree: 2},
                {id: 'c', indegree: 2},
                {id: 'd', indegree: 2}
            ]
        )

    }
)

test("outdegree", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'out-degree', as: 'outdegree'}]);

        expect(nodes).toEqual([
                {id: 'a', outdegree: 4},
                {id: 'b', outdegree: 1},
                {id: 'c', outdegree: 2},
                {id: 'd', outdegree: 1}
            ]
        )

    }
)


test("undirected-betweeness", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'undirected-betweeness', as: 'undirected-betweeness'}]);

        expect(nodes).toEqual([
                {id: 'a', 'undirected-betweeness': 5},
                {id: 'b', 'undirected-betweeness': 0},
                {id: 'c', 'undirected-betweeness': 0},
                {id: 'd', 'undirected-betweeness': 0}
            ]
        )

    }
)

test("directed-betweeness", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'directed-betweeness', as: 'directed-betweeness'}]);

        expect(nodes).toEqual([
                {id: 'a', 'directed-betweeness': 5},
                {id: 'b', 'directed-betweeness': 3},
                {id: 'c', 'directed-betweeness': 0},
                {id: 'd', 'directed-betweeness': 0}
            ]
        )

    }
)


test("closeness", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'closeness', as: 'closeness'}]);

        expect(nodes).toEqual([
                {id: 'a', closeness: 1},
                {id: 'b', closeness: 0.6666666666666666},
                {id: 'c', closeness: 0.6666666666666666},
                {id: 'd', closeness: 0.5714285714285714}
            ]
        )

    }
)


test("eccentricity", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'eccentricity', as: 'eccentricity'}]);

        expect(nodes).toEqual([
                {id: 'a', eccentricity: 1},
                {id: 'b', eccentricity: 2},
                {id: 'c', eccentricity: 2},
                {id: 'd', eccentricity: 2}
            ]
        )

    }
)


test("no metric name provided", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [{metric: 'eccentricity'}]);

        expect(nodes).toEqual([
                {id: 'a', eccentricity: 1},
                {id: 'b', eccentricity: 2},
                {id: 'c', eccentricity: 2},
                {id: 'd', eccentricity: 2}
            ]
        )

    }
)

test("calculate multiple metrics at one", () => {
        const nodes = [
            {id: 'a'},
            {id: 'b'},
            {id: 'c'},
            {id: 'd'},
        ];
        calculateMetrics(links, nodes, [
            {metric: 'in-degree', as: 'indegree'},
            {metric: 'out-degree', as: 'outdegree'}
        ]);

        expect(nodes).toEqual([
                {id: 'a', indegree: 1, outdegree: 4},
                {id: 'b', indegree: 2, outdegree: 1},
                {id: 'c', indegree: 2, outdegree: 2},
                {id: 'd', indegree: 2, outdegree: 1}
            ]
        )
    }
)
