import createGraph from 'ngraph.graph';

// @ts-ignore
// N.B. - ngraph.centrality has no type definitions, but there is an issue about adding them: https://github.com/anvaka/ngraph.centrality/issues/5
import centrality from 'ngraph.centrality';

import type {NetworkLink, NetworkNode} from "../types/network";
import { SeparateMetricSpec } from "../types/networkTransforms";

const calculateMetrics = (linkData: NetworkLink[], nodeData: NetworkNode[], metrics: SeparateMetricSpec[]) => {

    // construct network once
    const g = createGraph();

    for (const link of linkData) {
        const source = typeof(link.source) === 'object' ? link.source.id : link.source;
        const target = typeof(link.target) === 'object' ? link.target.id : link.target;
        g.addLink(source, target)
    }

    for (const metric of metrics){
        let metricValues;
        if (metric.metric === "degree"){
            metricValues = centrality.degree(g);
        } else if (metric.metric === "in-degree"){
             metricValues = centrality.degree(g, 'in');
        } else if (metric.metric === "out-degree"){
             metricValues = centrality.degree(g, 'out');
        } else if (metric.metric === "undirected-betweeness"){
             metricValues = centrality.betweenness(g);
        } else if (metric.metric === "directed-betweeness"){
             metricValues = centrality.betweenness(g, true);
        } else if (metric.metric === "closeness"){
             metricValues = centrality.closeness(g);
        } else if (metric.metric === "eccentricity"){
             metricValues = centrality.eccentricity(g);
        }

        const name = metric.as || metric.metric;
        for (const node of nodeData){
            node[name] = metricValues[node.id];
        }
    }
}

export {calculateMetrics}
