import { projectNetwork } from "./project";
import { constructNetwork } from "./creation/constructNetwork";

test("can do a bipartite projection", () => {

  // This test case is from Figure 1 of the Wikipedia article on Bipartite network projection
  // https://en.wikipedia.org/wiki/Bipartite_network_projection#/media/File:Bipartite_network_projection.png

  const nodes = [
    { name: "x1", type: "x", id: 1 },
    { name: "x2", type: "x", id: 2 },
    { name: "x3", type: "x", id: 3 },
    { name: "x4", type: "x", id: 4 },
    { name: "x5", type: "x", id: 5 },
    { name: "x6", type: "x", id: 6 },
    { name: "x7", type: "x", id: 7 },
    { name: "x8", type: "x", id: 8 },

    { name: "y1", type: "y", id: 9 },
    { name: "y2", type: "y", id: 10 },
    { name: "y3", type: "y", id: 11 },
    { name: "y4", type: "y", id: 12 },
    { name: "y5", type: "y", id: 13 },
    { name: "y6", type: "y", id: 14 }
  ];

  const links = [
    { source: 1, target: 9 },
    { source: 1, target: 10 },

    { source: 2, target: 9 },
    { source: 2, target: 10 },
    { source: 2, target: 12 },

    { source: 3, target: 11 },
    { source: 3, target: 13 },

    { source: 4, target: 9 },
    { source: 4, target: 12 },

    { source: 5, target: 11 },
    { source: 5, target: 13 },

    { source: 6, target: 11 },
    { source: 6, target: 14 },

    { source: 7, target: 13 },
    { source: 7, target: 14 },

    { source: 8, target: 13 }
  ];

  const data = { nodes, links };

  const network = constructNetwork(data, {
    nodes: "nodes",
    links: "links",
    directed: false,
    source_node: ["id", "source"], // node.id == link.source_id
    target_node: ["id", "target"]
  });


  // project to keep y and remove x
  const results = projectNetwork(network.nodes, network.links, {fieldName:  "type",  remainingNodeType: "y", removedNodeType: "x", directed: false});

  const simplifiedLinks = results.links.map(l => (`${l.source.name} -> ${l.target.name}`)).sort();

  expect(simplifiedLinks).toEqual(
    [
      "y1 -> y2",
      "y1 -> y2",
      "y1 -> y4",
      "y1 -> y4",
      "y2 -> y4",

      "y3 -> y5",
      "y3 -> y5",
      "y3 -> y6",
      "y5 -> y6"
    ]
  );

  const results2 = projectNetwork(network.nodes, network.links, { fieldName: "type", remainingNodeType: "x", removedNodeType: "y", directed: false });
  const simplifiedLinks2 = results2.links.map(l => (`${l.source.name} -> ${l.target.name}`)).sort();

  expect(simplifiedLinks2).toEqual(
    [
      "x1 -> x2",
      "x1 -> x2",
      "x1 -> x4",
      "x2 -> x4",
      "x2 -> x4",

      "x3 -> x5",
      "x3 -> x5",
      "x3 -> x6",
      "x3 -> x7",
      "x3 -> x8",
      "x5 -> x6",
      "x5 -> x7",
      "x5 -> x8",
      "x6 -> x7",
      "x7 -> x8"
    ]
  );
});