import type {NetworkNode, NetworkLink} from "../types/network";
import { ProjectNetworkSpec } from "../types/networkTransforms";

// Want to collapse:
// a - b - a
// find nodes of type B
// for each edge, select those connected to a node of type B

const defaultlinkDataReducer = (a: any, b: any) => [a,b];


// paths of length 3


const projectNetwork = (nodes: NetworkNode[], links: NetworkLink[], spec: ProjectNetworkSpec) => {

  const fieldName = spec.fieldName ?? "_type";
  const remainingNodeType = spec.remainingNodeType;
  const removedNodeType = spec.removedNodeType;
  const directed = spec.directed;
  const linkDataReducer = spec.linkDataReducer ?? defaultlinkDataReducer

  // for each node to be removed, record the connections to nodes that will remain
  const inLinks: NetworkLink[][] = [];
  const outLinks: NetworkLink[][] = [];

  const nodesToRemove: NetworkNode[] = [];

  for (const link of links) {
    // should use select rather than hard-coding .source
    if (link.source[fieldName] === removedNodeType && link.target[fieldName] === remainingNodeType) {

      if (!outLinks[link.source.id]) {
        nodesToRemove.push(link.source);
        inLinks[link.source.id] = [];
        outLinks[link.source.id] = [];
      }

      outLinks[link.source.id].push(link);

    } else if (link.source[fieldName] === remainingNodeType && link.target[fieldName] === removedNodeType) {
      if (!outLinks[link.target.id]) {
        nodesToRemove.push(link.target);
        inLinks[link.target.id] = [];
        outLinks[link.target.id] = [];
      }

      inLinks[link.target.id].push(link);
    }
  }

  const outputNodes: NetworkNode[] = [];
  const outputEdges: NetworkLink[] = [];


  for (const linkList of inLinks) {
    for (const inLink of linkList || []) {
      if (!outputNodes.includes(inLink.source)) {
        outputNodes.push(inLink.source);
      }
    }
  }

  for (const linkList of outLinks) {
    for (const outLink of linkList || []) {
      if (!outputNodes.includes(outLink.target)) {
        outputNodes.push(outLink.target);
      }
    }
  }


  if (directed) {
    // how to collapse data from two links?
    // maybe want a user-supplied merge function
    // e.g., in some cases, might want to take the latter of two dates

    for (const centralNode of nodesToRemove) {
      // add an edge between each source and each target node
      for (const inLink of inLinks[centralNode.id]) {
        for (const outLink of outLinks[centralNode.id]) {
          outputEdges.push({
            source: inLink.source,
            target: outLink.target,
            data: linkDataReducer(inLink.source.data, outLink.target.data)
          });
        }
      }
    }

  } else {

    for (const centralNode of nodesToRemove) {

      // add an edge between each source and each target node
      for (const inLink of inLinks[centralNode.id]) {
        for (const outLink of outLinks[centralNode.id]) {
          outputEdges.push({
            source: inLink.source,
            target: outLink.target,
            data: linkDataReducer(inLink.source.data, outLink.target.data)
          });
        }
      }

      // add an edge between all pairs of source nodes
      const linksToNode = inLinks[centralNode.id];

      for (let i = 0; i < linksToNode.length-1; i++) {
        for (let j = (i+1); j < linksToNode.length; j++) {
          outputEdges.push({
            source: linksToNode[i].source,
            target: linksToNode[j].source,
            data: [linksToNode[i].source.data, linksToNode[j].source.data]
          });

        }
      }


      // add an edge between all pairs of target nodes
      const linksFromNode = outLinks[centralNode.id];

      for (let i = 0; i < linksFromNode.length-1; i++) {
        for (let j = (i+1); j < linksFromNode.length; j++) {
          outputEdges.push({
            source: linksFromNode[i].target,
            target: linksFromNode[j].target,
            data: [linksFromNode[i].source.data, linksFromNode[j].source.data]
          });

        }
      }

    }

  }


  return { nodes: outputNodes, links: outputEdges };

};


export { projectNetwork };