import { promoteField } from "./promoteField";

test("can promote a field to being a node", () => {
  const nodes = [
    { id: "n0", val: 1 },
    { id: "n1", val: 2 },
    { id: "n2", val: 2 },
    { id: "n3", val: 3 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1] },
      { id: "l1", source: nodes[1], target: nodes[2] },
      { id: "l2", source: nodes[2], target: nodes[3] }
    ]
  };

  const spec = { type: "promote", field: "val", prefix: "value_" };

  const { nodes: newNodes, links: newLinks } = promoteField(network.nodes, network.links, spec, {});

  expect(newNodes).toEqual([
      { "id": "n0", "val": 1 },
      { "id": "n1", "val": 2 },
      { "id": "n2", "val": 2 },
      { "id": "n3", "val": 3 },
      { "id": "value_1" },
      { "id": "value_2" },
      { "id": "value_3" }
    ]
  );

  expect(newLinks).toEqual(
    [
      {
        "id": "l0",
        "source": { "id": "n0", "val": 1 },
        "target": { "id": "n1", "val": 2 }
      },
      {
        "id": "l1",
        "source": { "id": "n1", "val": 2 },
        "target": { "id": "n2", "val": 2 }
      },
      {
        "id": "l2",
        "source": { "id": "n2", "val": 2 },
        "target": { "id": "n3", "val": 3 }
      },
      {
        "id": "value_0",
        "source": { "id": "n0", "val": 1 },
        "target": { "id": "value_1" }
      },
      {
        "id": "value_1",
        "source": { "id": "n1", "val": 2 },
        "target": { "id": "value_2" }
      },
      {
        "id": "value_2",
        "source": { "id": "n2", "val": 2 },
        "target": { "id": "value_2" }
      },
      {
        "id": "value_3",
        "source": { "id": "n3", "val": 3 },
        "target": { "id": "value_3" }
      }
    ]
  );
});


test("can promote a field to being a node, with reversed edge drection", () => {
  const nodes = [
    { id: "n0", val: 1 },
    { id: "n1", val: 2 },
    { id: "n2", val: 2 },
    { id: "n3", val: 3 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1] },
      { id: "l1", source: nodes[1], target: nodes[2] },
      { id: "l2", source: nodes[2], target: nodes[3] }
    ]
  };

  const spec = { type: "promote", field: "val", prefix: "value_", direction: "fromValue" };

  const { nodes: newNodes, links: newLinks } = promoteField(network.nodes, network.links, spec, {});

  expect(newNodes).toEqual([
      { "id": "n0", "val": 1 },
      { "id": "n1", "val": 2 },
      { "id": "n2", "val": 2 },
      { "id": "n3", "val": 3 },
      { "id": "value_1" },
      { "id": "value_2" },
      { "id": "value_3" }
    ]
  );

  expect(newLinks).toEqual(
    [
      {
        "id": "l0",
        "source": { "id": "n0", "val": 1 },
        "target": { "id": "n1", "val": 2 }
      },
      {
        "id": "l1",
        "source": { "id": "n1", "val": 2 },
        "target": { "id": "n2", "val": 2 }
      },
      {
        "id": "l2",
        "source": { "id": "n2", "val": 2 },
        "target": { "id": "n3", "val": 3 }
      },
      {

        "id": "value_0",
        "target": { "id": "n0", "val": 1 },
        "source": { "id": "value_1" }
      },
      {
        "id": "value_1",
        "target": { "id": "n1", "val": 2 },
        "source": { "id": "value_2" }
      },
      {
        "id": "value_2",
        "target": { "id": "n2", "val": 2 },
        "source": { "id": "value_2" }
      },
      {
        "id": "value_3",
        "target": { "id": "n3", "val": 3 },
        "source": { "id": "value_3" }
      }
    ]
  );
});


test("can promote values of an expression to being a node", () => {
  const nodes = [
    { id: "n0", val: 1 },
    { id: "n1", val: 2 },
    { id: "n2", val: 2 },
    { id: "n3", val: 3 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1] },
      { id: "l1", source: nodes[1], target: nodes[2] },
      { id: "l2", source: nodes[2], target: nodes[3] }
    ]
  };

  const spec = { type: "promote", expression: "datum.val + 1 - 1", prefix: "value_" };

  const { nodes: newNodes, links: newLinks } = promoteField(network.nodes, network.links, spec, {});

  expect(newNodes).toEqual([
      { "id": "n0", "val": 1 },
      { "id": "n1", "val": 2 },
      { "id": "n2", "val": 2 },
      { "id": "n3", "val": 3 },
      { "id": "value_1" },
      { "id": "value_2" },
      { "id": "value_3" }
    ]
  );

  expect(newLinks).toEqual(
    [
      {
        "id": "l0",
        "source": { "id": "n0", "val": 1 },
        "target": { "id": "n1", "val": 2 }
      },
      {
        "id": "l1",
        "source": { "id": "n1", "val": 2 },
        "target": { "id": "n2", "val": 2 }
      },
      {
        "id": "l2",
        "source": { "id": "n2", "val": 2 },
        "target": { "id": "n3", "val": 3 }
      },
      {
        "id": "value_0",
        "source": { "id": "n0", "val": 1 },
        "target": { "id": "value_1" }
      },
      {
        "id": "value_1",
        "source": { "id": "n1", "val": 2 },
        "target": { "id": "value_2" }
      },
      {
        "id": "value_2",
        "source": { "id": "n2", "val": 2 },
        "target": { "id": "value_2" }
      },
      {
        "id": "value_3",
        "source": { "id": "n3", "val": 3 },
        "target": { "id": "value_3" }
      }
    ]
  );
});
