import { select } from "../data/select";
import { constructExpressionFunction } from "../data/calculate";
import type { PromoteSpec } from "../types/networkTransforms";


const promoteField = (nodes, links, spec: PromoteSpec, state) => {


  let getValToPromote = (_) => true;
  if ("expression" in spec) {
    const f = constructExpressionFunction({ calculate: spec.expression });
    getValToPromote = link => f(link, null, state);
  } else {
    getValToPromote = (node) => select(node, spec.field);
  }

  const newNodeMap = new Map();


  let i = 0;
  const newNodes = [];
  for (const node of nodes) {

    const val = getValToPromote(node);

    let newNode;
    if (newNodeMap.has(val)) {
      newNode = newNodeMap.get(val);
    } else {
      newNode = { id: `${spec.prefix}${val}` }; // TODO: id? type?
      newNodeMap.set(val, newNode);
      newNodes.push(newNode);
    }

    const linkId = `${(spec.linkPrefix ?? spec.prefix)}${i++}`;
    if (spec?.direction === "fromValue") {
      links.push({ id: linkId, source: newNode, target: node });// TODO: direction, id, othe rvals
    } else {
      links.push({ id: linkId, source: node, target: newNode });// TODO: direction, id, othe rvals
    }
  }

  return { nodes: [...nodes, ...newNodes], links };
};

export { promoteField };
