import { reverseEdges } from "./reverseEdges";

test("can reverse all edges", () => {
  const nodes = [
    { id: "n0" },
    { id: "n1" },
    { id: "n2" },
    { id: "n3" }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[1], target: nodes[2], directed: true },
      { id: "l2", source: nodes[2], target: nodes[3], directed: true }
    ]
  };

  const spec = { type: "reverseEdges" };

  const links = reverseEdges(network.nodes, network.links, spec, {})
    .map(l => ({ source: l.source.id, target: l.target.id }));

  expect(links).toEqual([
    { source: "n1", target: "n0" },
    { source: "n2", target: "n1" },
    { source: "n3", target: "n2" }
  ]);
});


test("can conditionally reverse edges", () => {
  const nodes = [
    { id: "n0" },
    { id: "n1" },
    { id: "n2" },
    { id: "n3" }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true, "val": 1 },
      { id: "l1", source: nodes[1], target: nodes[2], directed: true, "val": 2 },
      { id: "l2", source: nodes[2], target: nodes[3], directed: true, "val": 3 }
    ]
  };

  const spec = { type: "reverseEdges", condition: "datum.val > 1" };

  const links = reverseEdges(network.nodes, network.links, spec, {})
    .map(l => ({ source: l.source.id, target: l.target.id }));

  expect(links).toEqual([
    { source: "n0", target: "n1" },
    { source: "n2", target: "n1" },
    { source: "n3", target: "n2" }
  ]);
});