import { setDirectedness } from "./setDirectedness";

test("can make all edges directed", () => {
  const nodes = [
    { id: "n0" },
    { id: "n1" },
    { id: "n2" },
    { id: "n3" }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1] },
      { id: "l1", source: nodes[1], target: nodes[2] },
      { id: "l2", source: nodes[2], target: nodes[3] }
    ]
  };

  const spec = {type: "setDirectness", value: "directed"};

  const links = setDirectedness(network.nodes, network.links, spec, {});

  expect(links).toEqual([
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[1], target: nodes[2], directed: true },
      { id: "l2", source: nodes[2], target: nodes[3], directed: true }
  ]);
});


test("can make all edges undirected", () => {
  const nodes = [
    { id: "n0" },
    { id: "n1" },
    { id: "n2" },
    { id: "n3" }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1] },
      { id: "l1", source: nodes[1], target: nodes[2] },
      { id: "l2", source: nodes[2], target: nodes[3] }
    ]
  };

  const spec = {type: "setDirectness", value: "undirected"};

  const links = setDirectedness(network.nodes, network.links, spec, {});

  expect(links).toEqual([
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[1], target: nodes[2], directed: false },
      { id: "l2", source: nodes[2], target: nodes[3], directed: false }
  ]);
});




test("can conditionally set direction", () => {
  const nodes = [
    { id: "n0" },
    { id: "n1" },
    { id: "n2" },
    { id: "n3" }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], val: 1 },
      { id: "l1", source: nodes[1], target: nodes[2], val: 2 },
      { id: "l2", source: nodes[2], target: nodes[3], val: 3 }
    ]
  };

  const spec = { type: "setDirectness", expression: "datum.val > 1" };

  const links = setDirectedness(network.nodes, network.links, spec, {});

  expect(links).toEqual([
      { id: "l0", source: nodes[0], target: nodes[1], val: 1, directed: false },
      { id: "l1", source: nodes[1], target: nodes[2], val: 2, directed: true },
      { id: "l2", source: nodes[2], target: nodes[3], val: 3, directed: true }
  ]);
});