import { computeLineGraphDirected, computeLineGraphUndirected } from "./swapNodesEdges";


// N.B. - the networks used in these test cases are copied from the tests for the networkx line_graph function:
// https://github.com/networkx/networkx/blob/5c0b11afb4c0882a070d522ef3fa41482ba935d3/networkx/generators/tests/test_line.py

test("can run test_star", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[0], target: nodes[2], directed: false },
      { id: "l2", source: nodes[0], target: nodes[3], directed: false }
    ]
  };

  const newNetwork = computeLineGraphUndirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l1" },
    { source: "l0", target: "l2" },
    { source: "l1", target: "l2" }
  ]);
});

test("can run test_path", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 },
    { id: 4 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[1], target: nodes[2], directed: false },
      { id: "l2", source: nodes[2], target: nodes[3], directed: false },
      { id: "l3", source: nodes[3], target: nodes[4], directed: false }
    ]
  };

  const newNetwork = computeLineGraphUndirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l1" },
    { source: "l1", target: "l2" },
    { source: "l2", target: "l3" }
  ]);
});

test("can run test_cycle", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 },
    { id: 4 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[1], target: nodes[2], directed: false },
      { id: "l2", source: nodes[2], target: nodes[3], directed: false },
      { id: "l3", source: nodes[3], target: nodes[4], directed: false },
      { id: "l4", source: nodes[4], target: nodes[0], directed: false }
    ]
  };

  const newNetwork = computeLineGraphUndirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l4" },

    { source: "l0", target: "l1" },
    { source: "l1", target: "l2" },
    { source: "l2", target: "l3" },
    { source: "l3", target: "l4" }
  ]);
});


test("can run test_digraph1", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[0], target: nodes[2], directed: true },
      { id: "l2", source: nodes[0], target: nodes[3], directed: true }
    ]
  };

  const newNetwork = computeLineGraphDirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([]);
});


test("can run test_multigraph1", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 }

  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[0], target: nodes[1], directed: false },
      { id: "l2", source: nodes[1], target: nodes[0], directed: false },
      { id: "l3", source: nodes[0], target: nodes[2], directed: false },
      { id: "l4", source: nodes[2], target: nodes[0], directed: false },
      { id: "l5", source: nodes[0], target: nodes[3], directed: false }
    ]
  };

  const newNetwork = computeLineGraphUndirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { "source": "l0", "target": "l1" },
    { "source": "l0", "target": "l2" },
    { "source": "l0", "target": "l3" },
    { "source": "l0", "target": "l4" },
    { "source": "l0", "target": "l5" },
    { "source": "l1", "target": "l2" },
    { "source": "l1", "target": "l3" },
    { "source": "l1", "target": "l4" },
    { "source": "l1", "target": "l5" },
    { "source": "l2", "target": "l3" },
    { "source": "l2", "target": "l4" },
    { "source": "l2", "target": "l5" },
    { "source": "l3", "target": "l4" },
    { "source": "l3", "target": "l5" },
    { "source": "l4", "target": "l5" }
  ]);
});


test("can run test_multigraph2", () => {
  const nodes = [
    { id: 0 },
    { id: 1 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: false },
      { id: "l1", source: nodes[1], target: nodes[0], directed: false },
    ]
  };

  const newNetwork = computeLineGraphUndirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { "source": "l0", "target": "l1" },
  ]);
});


test("can run test_multidigraph1", () => {
  const nodes = [
    { id: 1 },
    { id: 2 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[1], target: nodes[0], directed: true }
    ]
  };

  const newNetwork = computeLineGraphDirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l1" },
    { source: "l1", target: "l0" }
  ]);

});

test("can run test_multidigraph2", () => {
  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[0], target: nodes[1], directed: true },
      { id: "l2", source: nodes[0], target: nodes[1], directed: true },
      { id: "l3", source: nodes[1], target: nodes[2], directed: true }
    ]
  };

  const newNetwork = computeLineGraphDirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l3" },
    { source: "l1", target: "l3" },
    { source: "l2", target: "l3" }
  ]);

});


test("can run test_digraph2", () => {

  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 }
  ];

  const network = {
    nodes,
    links: [
      { id: "l0", source: nodes[0], target: nodes[1], directed: true },
      { id: "l1", source: nodes[1], target: nodes[2], directed: true },
      { id: "l2", source: nodes[2], target: nodes[3], directed: true }
    ]
  };

  const newNetwork = computeLineGraphDirected(network.nodes, network.links, {});

  const links = newNetwork.links.map(l => ({ source: l.source.id, target: l.target.id }));
  expect(links).toEqual([
    { source: "l0", target: "l1" },
    { source: "l1", target: "l2" }
  ]);

});

