import { constructAdjacencyList } from "../selectionTransforms/constructAdjacencyList";
import { SwapNodesAndLinksSpec } from "../types/networkTransforms";

// TODO: if directionality is set to false, perhaps should set `directed` to `false` on edges before calling constructAdjacencyList

const computeLineGraphDirected = (nodes, links) => {
  const adjacency = constructAdjacencyList(nodes, links);

  const newLinks = [];
  const newNodes = [];

  let i = 0;
  for (const link of links) {
    newNodes.push(link);

    for (const l2 of adjacency.outLinks.get(link.target.id)) {
      newLinks.push({ source: link, target: l2, id: i++ });
    }
  }

  for (const link of newNodes) {
    delete link.source;
    delete link.target;
    delete link.directed;
  }

  return { nodes: newNodes, links: newLinks };
};


const computeLineGraphUndirected = (nodes, links) => {
  const adjacency = constructAdjacencyList(nodes, links);

  const newLinks = [];
  const newNodes = [];

  const addedLinks = [];

  for (const link of links) {
    newNodes.push(link);
  }

  let i = 0;
  for (const node of nodes) {
    const attachedLinks = adjacency.undirectedLinks.get(node.id);
    for (const l1 of attachedLinks) {
      for (const l2 of attachedLinks) {
        const lid = l1.id.toString() + "_" + l2.id.toString();
        if (!(addedLinks.includes(lid)) && (l1.id < l2.id)) {
          addedLinks.push(lid);
          newLinks.push({ source: l1, target: l2, id: i++ });
        }
      }
    }
  }

  for (const link of newNodes) {
    delete link.source;
    delete link.target;
    delete link.directed;
  }

  return { nodes: newNodes, links: newLinks };
};


const swapNodesEdges = (nodes, links, spec: SwapNodesAndLinksSpec) => {

  if (!spec.directed || links.every(l => !l.directed)){
    return computeLineGraphUndirected(nodes, links);
  }
  return computeLineGraphDirected(nodes, links);

}

export { computeLineGraphDirected, computeLineGraphUndirected, swapNodesEdges };