import {constructSeriation} from "./seriation";

const len = (obj) => Object.keys(obj).length;

test("sort by basic optimal-leaf-order", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];


    const perm = constructSeriation(adj_mat, {method: "optimal-leaf-order"})

    expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3] ]));
    expect(Array.from(perm).length).toEqual(4);
})


test("sort by optimal-leaf-order with specified weight function", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];


    const perm = constructSeriation(adj_mat, {method: "optimal-leaf-order", distance: "manhattan"})

    expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3] ]));
    expect(Array.from(perm).length).toEqual(5);
})


test("sort by barycentre", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];

    const perm = constructSeriation(adj_mat, {method: "barycentre"})

    // TODO: check if baycentre is always a no-op
    // expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3], [4,4] ]));
    expect(Array.from(perm).length).toEqual(5);
})


test("sort by bandwidth-reduction", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];


    const perm = constructSeriation(adj_mat, {method: "bandwidth-reduction"})

    expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3], [4,4] ]));
    expect(Array.from(perm).length).toEqual(5);
})

test("sort by spectral", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];


    const perm = constructSeriation(adj_mat, {method: "spectral"})

    expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3], [4,4] ]));
    expect(Array.from(perm).length).toEqual(5);
})

test("sort by pca", () => {

    const adj_mat =   [
        [0, 1, 0, 1, 0],
        [1, 0, 1, 0, 1],
        [0, 1, 0, 1, 1],
        [1, 1, 1, 0, 0]
    ];


    const perm = constructSeriation(adj_mat, {method: "pca"})

    expect(perm).not.toEqual(new Map([[0,0], [1,1], [2,2], [3,3], [4,4] ]));
    expect(Array.from(perm).length).toEqual(5);
})