import { constructScale } from "./scale";

test("can construct a scale", () => {
  const d = constructScale({
    type: "linear",
    domain: [5, 55],
    range: [0, 100],
    padding: 0,
    zero: false
  });

  expect(d.range()).toEqual([0, 100]);
  expect(d.domain()).toEqual([5, 55]);

  expect(d(5)).toEqual(0);
  expect(d(30)).toEqual(50);
  expect(d(55)).toEqual(100);
});


test("color scale", () => {

  const s = constructScale({
        name: "color",
        type: "linear",
      scheme: "viridis",
    //    range: {scheme: "viridis"},
        domain: [0, 20]
      })

  expect(s(10)).toEqual('rgb(33, 145, 141)')
})

test("can use rangeExpression", () => {
  const config = {
    "name": "innerPos",
    "type": "ordinal",
    "domain": [0, 10],

    "rangeExpressions": ["bounds.width/2", "bounds.width"]
    };


  const s = constructScale(config, {width: 10, height: 10})
  expect(s.range()).toEqual([5, 10]);
})

test("can use rangeExpression with a mixture of string expressions and number constants", () => {
  const config = {
    "name": "innerPos",
    "type": "ordinal",
    "domain": ["A", "B", "C"],

    "rangeExpressions": ["bounds.width/2", 2, "bounds.width"]
    };


  const s = constructScale(config, {width: 10, height: 10})
  expect(s.range()).toEqual([5, 2, 10]);
})

test("can use rangeMapFunction", () => {

  const config = {
    "name": "innerPos",
    "type": "ordinal",
    "domain": ["B", "Y", "G", "P"],

    "range": [0, 1, 2, 3],
    "rangeMapFunction": "datum * bounds.width/4",
    };


  const s = constructScale(config, {width: 12, height: 12})
  expect(s.range()).toEqual([0, 3, 6, 9]);
})

test("can use 'width' as constant in domain'", () => {

  const config = {
    "name": "innerPos",
    "type": "linear",
    "domain": [0, 10],

    "range": [0, "width"],
    };


  const s = constructScale(config, {width: 12, height: 12})
  expect(s.range()).toEqual([0, 12]);
})