import { DerivedSelectionParameter } from "../types/params";
import { shortestPath } from "./shortestPath";
import { constructAdjacencyList } from "./constructAdjacencyList";
import { prune } from "./prune";
import { filter } from "./filter";
import { project } from "./project";
import { parallelEdges } from "./parallelEdges";
import { commonNeighbours} from "./commonNeighbours";
import { connectedEdges } from "./connectedEdges";
import { connectedNodes } from "./connectedNodes";
import { edgesBetween } from "./edgesBetween";
import { findNeighbours } from "./neighbours";

import type { GraphgraSpec } from "../types/spec";
import type { Network } from "../types/network";

const applySelectionTransform = (state, spec: DerivedSelectionParameter, fullSpec: GraphgraSpec) => {
  let selection = state[spec.source];

  // tODO: find network by tracing up parents from derived network

  // find the corresponding network
  const sourceParamSpec = fullSpec.parameters.filter(p => p.name === spec.source)[0];
  let network: Network = { nodes: [], links: [] };
  if ("network" in sourceParamSpec) {
    network = state[sourceParamSpec.network];
  }

  const adjacency = constructAdjacencyList(network.nodes, network.links);

  for (const transform of spec.transform) {
    if (transform.type == "shortestPath") {
      selection = shortestPath(state, selection, adjacency, transform);
    } else if (transform.type === "prune") {
      selection = prune(state, selection, adjacency, transform);
    } else if (transform.type === "filter") {
      selection = filter(state, selection, adjacency, transform);
    } else if (transform.type === "project") {
      selection = project(state, selection, adjacency, transform);
    } else if (transform.type === "parallelEdges") {
      selection = parallelEdges(state, selection, adjacency, transform);
    } else if (transform.type === "connectedNodes") {
      selection = connectedNodes(state, selection, adjacency, transform);
    } else if (transform.type === "connectedEdges") {
      selection = connectedEdges(state, selection, adjacency, transform);
    }  else if (transform.type === "commonNeighbours") {
      selection = commonNeighbours(state, selection, adjacency, transform);
    } else if (transform.type === "edgesBetween") {
      selection = edgesBetween(state, selection, adjacency, transform);
    } else if (transform.type === "neighbours") {
      selection = findNeighbours(state, selection, adjacency, transform);
    }
  }
  return selection;
};

export { applySelectionTransform };