import type { Adjacency } from "./constructAdjacencyList";
import type { TransformProject } from "../types/params";
import type { Selection } from "../types/params";
import { select } from "../data/select";


const filterArray = (selectedArray, fullArray, spec) => {
  if (spec.semantics === "separate") {
    // construct list of values for each field
    const values = new Map<string, Set<any>>();

    for (const field of spec.fields) {
      const vals = new Set();

      for (const node of selectedArray) {
        vals.add(select(node, field));
      }
      values.set(field, vals);
    }

    return fullArray.nodes.filter(n => {
      for (const field of spec.fields) {
        const vals = values.get(field);
        if (!vals.has(select(n, field))) {
          return false;
        }
      }
      return true;
    });
  } else {
    // construct list of combinations of values
    const valCombinations = new Set<string>();

    for (const node of selectedArray) {
      const valCombination = JSON.stringify(spec.fields.map(f => select(node, f)));
      valCombinations.add(valCombination);
    }

    return fullArray.nodes.filter(node => {
      const valCombination = JSON.stringify(spec.fields.map(f => select(node, f)));
      return valCombinations.has(valCombination);
    });
  }
};


const project = (state, selection: Selection, adjacency: Adjacency, spec: TransformProject) => {
  const network = state[selection.network];

  let nodes;
  if (spec.affects === "nodes") {
    nodes = filterArray(selection.nodes, network.nodes, spec);
  } else {
    nodes = filterArray(selection.links, network.links, spec);
  }
  return { nodes, links: [] };

};

export { project };
