import { shortestPath } from "./shortestPath";
import { constructNetwork } from "../networks/creation/constructNetwork";
import {constructAdjacencyList} from "./constructAdjacencyList"

test("can find shortest path using weights", () => {

  const edges = [
    { source: 0, target: 1, weight: 10, directed: true },
    { source: 0, target: 3, weight: 5, directed: true },

    { source: 1, target: 3, weight: 2, directed: true },
    { source: 1, target: 2, weight: 1, directed: true },

    { source: 2, target: 4, weight: 4, directed: true },

    { source: 3, target: 1, weight: 3, directed: true },
    { source: 3, target: 2, weight: 9, directed: true },
    { source: 3, target: 4, weight: 2, directed: true },

    { source: 4, target: 2, weight: 6, directed: true },
    { source: 4, target: 0, weight: 7, directed: true }
  ];

  const nodes = [
    { id: 0 },
    { id: 1 },
    { id: 2 },
    { id: 3 },
    { id: 4 },
    { id: 5 }
  ];

  const network = constructNetwork({ nodes, edges }, {
    nodes: "nodes",
    links: "edges",
    directed: true,
    source_node: ["id", "source"],
    target_node: ["id", "target"]
  });

  const adjacency = constructAdjacencyList(network.nodes, network.links);

  // This network is the example in CLRS (Fig 25.5), with nodes labelled:
  //    1   2
  //  0
  //    3   4
  let path = shortestPath({networkName: network }, { nodes: [network.nodes[0], network.nodes[1]], network: "networkName" }, adjacency, { weight: "weight" });
  expect(path.nodes.map(n=> n.id)).toEqual([0, 3, 1])

  path = shortestPath({networkName: network }, { nodes: [network.nodes[0], network.nodes[2]], network: "networkName" }, adjacency, { weight: "weight" });
  expect(path.nodes.map(n=> n.id)).toEqual([0, 3, 1, 2])

  path = shortestPath({networkName: network }, { nodes: [network.nodes[0], network.nodes[3]], network: "networkName" }, adjacency, { weight: "weight" });
  expect(path.nodes.map(n=> n.id)).toEqual([0, 3])

  path = shortestPath({networkName: network }, { nodes: [network.nodes[0], network.nodes[4]], network: "networkName" }, adjacency, { weight: "weight" });
  expect(path.nodes.map(n=> n.id)).toEqual([0, 3, 4])
});
