// @ts-ignore
import { layoutD3Force } from "./layoutD3force";
// @ts-ignore
import { layoutCola } from "./layoutCola";
// @ts-ignore
import { layoutCytoscape } from "./layoutCytoscape";

import { layoutTulip } from "./layoutTulip";

import type { Axis, Layout } from "../types/layout";
import type { Network } from "../types/network";
import type { TopolgicalLayoutSpec } from "../types/topologicalLayout";
import type { Bounds } from "../types/spec";
import { rescaleLayout } from "../layouts/utils";
import { layoutBioFabric } from "./layoutBioFabric";
import { layoutSetCola } from "./layoutSetCola";
import { layoutBipartite } from "./layoutBipartite";

const computeTopologicalLayout = async (state, spec: TopolgicalLayoutSpec, bounds: Bounds, addToState): Promise<[Layout, Axis[]]> => {

  // identify correct network

  const networkName = spec.network;

  // Note that can't just do a JSON deep-copy of network, as then object identity is lost between link.source/.target and entries in nodes array
  // This means that performing a layout will add spurious fields to the nodes objects (e.g., .x,.y, velocities, etc)

  let layout: Layout = new Map();
  // const network = JSON.parse(JSON.stringify(networks[spec.network]));
  let network = state[spec.network] as Network;

  if (spec.type === "d3-force") {
    layoutD3Force(network.nodes, network.links, spec, state, bounds);
    for (const node of network.nodes) {
      layout.set(node.id, { x: node.x, y: node.y });
    }

  } else if (spec.type === "webcola") {

    // @ts-ignore
    layoutCola(network.nodes, network.links, spec);
    for (const node of network.nodes) {
      layout.set(node.id, { x: node.x, y: node.y });
    }

  } else if (spec.type === "setcola") {
    // @ts-ignore
    layoutSetCola(network.nodes, network.links, spec);
    for (const node of network.nodes) {
      layout.set(node.id, { x: node.x, y: node.y });
    }
  } else if (spec.type === "tulip") {
    layout = await layoutTulip(network.nodes, network.links, spec);
  } else if (spec.type === "bioFabric"){
    const res = layoutBioFabric(network.nodes, network.links, spec, state, addToState);
    layout = res.layout;
    network = res.reshapedNetwork;
  } else if (spec.type === "bipartite"){
      layout = layoutBipartite(network.nodes, network.links, spec, state, addToState);
  } else {
    // cytoscape

    // make a copy of nodes: this is necessary because applying a cytoscape layout will convert integer nodes ids to strings
    const nodeCopy = JSON.parse(JSON.stringify(network.nodes));
    const positions = layoutCytoscape(nodeCopy, network.links, spec);

    //console.log(positions);
    for (let i = 0; i < network.nodes.length; i++) {
      const node = network.nodes[i];
      layout.set(node.id, { x: positions[i].x, y: positions[i].y });
    }
  }

  rescaleLayout(spec, state, network.nodes, layout, bounds,  [], true);
  return [layout, []];

};

export { computeTopologicalLayout };