import { Layout } from "webcola";
//import setcola from "setcola/dist/setcola.js";
import * as setcola from "setcola/src/index.js";

import type { NetworkNode, NetworkLink } from "../types/network";
import type { LayoutSpecSetCola } from "../types/topolgicalLayoutSetcola";

// based on https://github.com/tgdwyer/WebCola/blob/2b6a7564d6e2dde0080870b051eda89e82f78ad9/WebCola/test/apitests.ts#L30

// adds index, x, y attributes to node array
// note that .source/.target attribute on links must be an index or object (not an id, which d3-force would accept)


const layoutSetCola = (nodes: NetworkNode[], links: NetworkLink[], spec: LayoutSpecSetCola) => {

  // first process the setcola spec
  const res = setcola
    .nodes(nodes)
    .links(links)
    .guides(spec.guides || [])
    .constraints(spec.constraints || [])
    .layout(); // doesn't actually perform layout - just generates nodes/links/groups/constraints ready for webcola


  //
  const linkDist = spec.linkDist ?? 115;
  const nodesize = spec.nodeSize ?? 14;
  const nodepad = 0;


  // from befaultOptions at bottom of renderer.js
  res.nodes.forEach(function(node) {
    node._width = node.width || node.size || nodesize;
    node._height = node.height || node.size || nodesize;
    node.pad = node.pad || nodepad;
    node.width = node._width + 2 * node.pad;
    node.height = node._height + 2 * node.pad;
  });

  // N.B. demo sets .size([1162, 1575])
  // and .overlaps(true)
  // layout.size([1000, 1000]).avoidOverlaps(true).convergenceThreshold(1e-3)

  // then do layout using webCola
  // @ts-ignore
  const layout = new Layout()
    // @ts-ignore
    .nodes(res.nodes)
    // @ts-ignore
    .links(res.links)
    .groups(res.groups)
    .constraints(res.constraints);

  if (spec.size){
    layout.size(spec.size);
  }
  if (spec.avoidOverlaps){
    layout.avoidOverlaps(spec.avoidOverlaps);
  }
  if (spec.convergenceThreshold){
    layout.convergenceThreshold(spec.convergenceThreshold);
  }


 // fudge with lengths of temporary edges
  layout.linkDistance(function(d) {
    let linkDistance = linkDist;
    if (d.hasOwnProperty("temp")) linkDistance = nodesize / 2;
    if (d.hasOwnProperty("length")) linkDistance = d.length;
    return linkDistance;
  });


  layout.start(spec.numIterations ?? 100);
};

export { layoutSetCola };

