import type { NetworkLink, NetworkNode } from "../types/network";
import { LayoutSpecTulip } from "../types/topologicalLayoutTulip";


const layoutTulip = async (nodes: NetworkNode[], links: NetworkLink[], spec: LayoutSpecTulip) => {
  const body = {
    network: {
      nodes: nodes,
      links: links.map(l => ({ source: l.source.id, target: l.target.id }))
    },
    options: {
      algorithm: spec.algorithm,
      ...(spec.options || [])
    }
  };

  let server = spec.server ?? "http://localhost:8080";
  server =  server.endsWith("/") ? server : server + "/";

  return await fetch(server + "layout", {
    method: "POST",
    mode: "cors",
    body: JSON.stringify(body),
    headers: {
      "Content-Type": "application/json"
    }
  })
    .then(res => res.json());
};

export { layoutTulip };

