type ModifierKeys = "alt" | "ctrl" | "meta" | "shift";

export type ModifierDefinitions = {
  modifierKeyRequired: ModifierKeys[];
  modifierKeyAbsent: ModifierKeys[];
}

type InteractionBase = {
  interaction: string;
}

type DragInteraction = InteractionBase & {
  interaction: "drag";

  /**
   * Key that must be held down whilst the interaction occurs (default is "any").
   */
  modifierKeyRequired?: ModifierKeys[];

  /**
   * Key that must be *not* held down whilst the interaction occurs (default is "any").
   */
  modifierKeyAbsent?: ModifierKeys[];
};


export type CursorInteraction = InteractionBase & {
  interaction: "select";

  /**
   * The type of event that triggers this interaction (default is "click").
   */
  event?: "click" | "mouseenter" | "mouseleave" | "mouseover" | "mouseout"

  /**
   * The interaction type to be handled.
   */
  using?: "rect" | "lasso" | "cursor";

  /**
   * The name of the selection to update
   */
  selection: string;

  /**
   * How to update the selection (default is "replace")
   */
  action: "add" | "remove" | "replace" | "toggle";

  /**
   * Key that must be held down whilst the interaction occurs (default is "any").
   */
  modifierKeyRequired?: ModifierKeys[];

  /**
   * Key that must be *not* held down whilst the interaction occurs (default is "any").
   */
  modifierKeyAbsent?: ModifierKeys[];
}


export type Interaction = DragInteraction | CursorInteraction;

// TODO: way to specify what exactly is added (nodes/links to selection, param set to value of field, ...)
// TODO: editing datum?
