type OrderDirection = "ascending" | "descending";

type Order = Map<number | string, number>;

type OrderingCriterionExpression = {
    /**
   * This expression will be evaluated for each item, and the items will be sorted based on these calculated values.
     * The expression can access an item as `datum`.
   */
  expression: string;
    /**
   * Whether to sort in ascending or descending order.
   */
  direction?: OrderDirection;
  fn: any;
}

type OrderingCriterionAttribute = {
  /**
   * The name of a field (or list of names of fields) to sort on.
   */
  field: string | string[]; // TODO: rename to fields?
  /**
   * Whether to sort in ascending or descending order.
   */
  direction?: OrderDirection;
}

type ExpandedOrderingCriterion = OrderingCriterionExpression | OrderingCriterionAttribute;
type OrderingCriterion = string | OrderingCriterionExpression | OrderingCriterionAttribute;

type OrderingAttributeSpec = {
  /**
   * The name that will be given to the ordering when it is created.
   */
  name: string;
  /**
   * The name of the data array containing the items to be laid out.
   */
  data: string;
  /**
   * If allowTies is true, then two items for which the ordering criterion are equal will be assigned the same rank.
   * If allowTies is false, then every item will be assigned a distinct rank.
   */
  allowTies?: boolean;

  /**
   * The criterion that will be used when ordering items.
   */
  orderBy: OrderingCriterion[] | OrderingCriterion;
}


type SeriationAlgorithmName = "optimal-leaf-order" | "barycentre" | "bandwidth-reduction" | "pca";
type Distances = "euclidean" | "manhattan" | "minkowski" | "chebyshev" | "hamming" | "jaccard" | "braycurtis";
type OrderingSeriationSpec = {
  /**
   * The name that will be given to the ordering.
   */
  name: string;
  /**
   * The network for which the nodes are to be ordered.
   */
  network: string;
  /**
   * The name of the seriation method.
   */
  method: SeriationAlgorithmName | { parameter: string };
  /**
   * The name of a field containing edge weights.
   */
  weightField?: string;
  /**
   * The name of the distance function to use.
   */
  distance?: Distances;
}


export type OrderingSpec = OrderingAttributeSpec | OrderingSeriationSpec;
