import type { OrderingCriterionAttribute } from "./ordering";


type LabelSpec = {
  /**
   * The field to use as the source of label text.
   */
  field: string;

  /**
   * Space (in pixels) to allow for labels.
   */
  width?: number;

  /**
   * Other properties valid for a Vega text mark can also be assigned.
   */
}

type TableSpec = {
  /**
   * The name of the table (mandatory, for implementation reasons).
   */
  name: string;

  /**
   * Ordering for the rows.
   */
  rowOrder: { order: string, field: string } | OrderingCriterionAttribute;

  /**
   * Ordering for the columns.
   */
  colOrder: { order: string, field: string } | OrderingCriterionAttribute;

  /**
   * Name of the data array from which this table should be constructed.
   */
  data: string;

  /**
   * Name of group. This is generally not needed, as the required grouping can be identified from the row and column orderings.
   * However, this enables users to provide a group that has been subsequently modified by a transformation.
   */
  group?: string;

  /**
   * Specification for row labels.
   */
  rowLabels?: LabelSpec;

  /**
   * Specification for column labels.
   */
  colLabels?: LabelSpec;

  /**
   * If true, dragging the row/column labels will update the underlying order.
   */
  dragToReorder?: boolean;

  /**
   * If true, then every value that appears as a row will also appear as a column (even if every cell in that column is empty),
   * and vice-versa. This is useful for adjacency matrices of directed networks, as source/sink nodes may otherwise appear only
   * as rows or columns.
   */
  symmetric?: boolean;
};

// by default we want to disallow ties in these orderings


// export type TableSpec = TableInlineOrderSpec | TablePredefinedOrderSpec;
