import type { LayoutSpecCytoscape } from "./topologicalLayoutCytoscape";
import type { LayoutSpecCola } from "./topologicalLayoutCola";
import type {LayoutSpecD3Force} from "./topologicalLayoutD3Force";
import type { LayoutSpecTulip } from "./topologicalLayoutTulip";
import type { Val } from "./layout";
import { LayoutSpecSetCola } from "./topolgicalLayoutSetcola";
import { DimensionSpec, OrderWithOptionalField } from "./layout";

type LayoutSpecBioFabric = LayoutSpecBase & {
  /**
   * A layout in which nodes are represented by rows, and edges are vertical lines.
   * As each link attached to a node has a different horizontal position, this can be considered to be a layout of a transformed network,
   * in which each (node, link) combination in the original network has been replaced by a new node.
   */
  type: "bioFabric";

    /**
   * The name that will be given to the layout when it is created.
   */
  name: string;

  /**
   * Used to order links between the same pair of nodes.
   */
  linkTypeColName?: string;

  /**
   * The transformed network will contain new nodes, with different ids, but the ids of the original nodes that they were
   * generated from will be saved as an attribute with this name (default: "_originalId").
   */
  saveIdAs?: string;

  /**
   * Name for the reshaped network (defaults to the name of the original network followed by "#BioFabric").
   */
  newNetworkName?: string;
};


export interface LayoutSpecBase {
  /**
   * The name that will be given to the layout when it is created.
   */
  name?: string; // optional if there is only one layout

  /**
   * The name of the network for which the layout is being calculated.
   */
  network: string; // the name of the network

  /**
   * The layout type.
   */
  type: "d3-force" | "webcola" | "cytoscape" | "tulip" | "bioFabric" | "setcola" | "bipartite";

  /**
   * Width of individual elements in the layout: field, expression, or number
   */
  width?: Val;

  /**
   * Height of individual elements in the layout: field, expression, or number
   */
  height?: Val;
}

export type LayoutSpecBipartiteBase = LayoutSpecBase & {
  /**
   * The bipartite layout assigns each node to one of two layers. The nodes in one layer are assigned positions by
   * applying a scale or ordering. The nodes in the other layer are positioned at the barycenter of the fixed nodes that
   * they are connected to.
   */
  type: "bipartite";

  /**
   * If this expression evaluates to true for a node, then it is assigned to the first layer.
   */
  inFirstLayer: string;


  /**
   * Specifies whether the nodes in the first or second layer should be held fixed.
   * The nodes in the other layer are repositioned to be at the barycenter of the nodes they are connected to.
   */
  fixedLayer: 1 | 2;
}

type LayoutSpecBipartiteVertical = LayoutSpecBipartiteBase & {
  /**
   * Determines positioning of nodes in fixed layer.
   */
  y: DimensionSpec;
}

type LayoutSpecBipartiteHorizontal = LayoutSpecBipartiteBase & {
  /**
   * Determines positioning of nodes in fixed layer.
   */
  x: DimensionSpec;
}

type LayoutSpecBipartite = LayoutSpecBipartiteVertical | LayoutSpecBipartiteHorizontal;

export type TopolgicalLayoutSpec = LayoutSpecD3Force | LayoutSpecCola | LayoutSpecCytoscape | LayoutSpecTulip | LayoutSpecBioFabric | LayoutSpecSetCola | LayoutSpecBipartite;

// For Cola
// TODO: link distance custom for each edge
