import { extent } from "d3-array";

import { select } from "../data/select";
import { Parameter } from "../types/params";
import { createBrushInput } from "./brushInput";

export const createInputs = (state, inputsDiv: HTMLDivElement, paramatersSpec: Parameter[], setParam: (paramName: string, newVal: string | number) => void) => {

  const paramInputElements = {};

  for (const param of paramatersSpec) {

    if (!("bind" in param)) {
      continue;
    }

    const specialInputTypes = ["select", "radio", "brush"];

    if (param.bind.input === "brush"){
      //
      const label = document.createElement("label");
      label.innerText = ("label" in param.bind) ? param.bind.label : param.name;

      label.style.display = "inline-flex";
      label.style.alignItems = "center";

      inputsDiv.appendChild(label);

      const data = state[param.bind.network][param.bind.for];
      const xExtent = extent(data.map(d => select(d, param.bind.field)));

      const updateVal = (val) => setParam(param.name, val);

      const input = createBrushInput(xExtent, "linear", updateVal, param.bind.updateContinuously);
      paramInputElements[param.name] = input;
      inputsDiv.appendChild(input);

    } else if ("element" in param.bind) {
      // binding to an external element

      const eventName = ("event" in param.bind) ? param.bind.event : "oninput";

      const el = document.getElementById(param.bind.element);
      el[eventName] = (ev) => setParam(param.name, ev.target.value);
      paramInputElements[param.name] = el;

    } else if (!specialInputTypes.includes(param.bind.input)) {
      // a single input element of some type

      const label = document.createElement("label");
      label.innerText = ("label" in param.bind) ? param.bind.label : param.name;

      label.style.display = "inline-flex";
      label.style.alignItems = "center";
      label.style.userSelect = "none";

      inputsDiv.appendChild(label);

      const input = document.createElement("input");
      input.type = param.bind.input;
      label.appendChild(input);

      paramInputElements[param.name] = input;

      let valSpan;
      if (param.bind.input === "range") {
        valSpan = document.createElement("span");
        valSpan.innerText = param.bind.value ?? "";
        label.appendChild(valSpan);
      }

      if ("min" in param.bind) {
        input.min = param.bind.min.toString();
      }

      if ("max" in param.bind) {
        input.max = param.bind.max.toString();
      }

      if ("step" in param.bind) {
        input.step = param.bind.step.toString();
      }

      if ("value" in param.bind) {
        input.value = param.bind.value;
      }

      // pass-through other attributes
      for (const tag of Object.keys(param.bind)) {
        const notAttributes = ["input", "label", "labels", "options", "min", "max", "step", "element", "event"];
        if (!notAttributes.includes(tag)) {
          input[tag] = param.bind[tag];
        }
      }

      if (param.bind.input === "checkbox") {
        // @ts-ignore
        input.onchange = (ev) => setParam(param.name, ev.target.checked);
      } else if (param.bind.input === "range") {
        input.onchange = (ev) => {
          // @ts-ignore
          setParam(param.name, ev.target.value);
          // @ts-ignore
          valSpan.innerText = ev.target.value;
        };
      } else {
        // @ts-ignore
        input.onchange = (ev) => setParam(param.name, ev.target.value);
      }

    } else if (param.bind.input === "radio") {
      // a set of checkboxes acting together as adio buttons

      const label = document.createElement("label");
      label.innerText = ("label" in param.bind) ? param.bind.label : param.name;

      label.style.display = "inline-flex";
      label.style.alignItems = "center";

      inputsDiv.appendChild(label);

      for (const i in param.bind.options) {
        const l = document.createElement("label");
        l.style.display = "inline-flex";
        l.style.alignItems = "center";
        l.innerText = (param.bind.labels && param.bind.labels[i]) ?? param.bind.options[i].toString();

        const input = document.createElement("input");
        input.type = "radio";
        input.name = param.name;

        input.oninput = (ev) => {
          // @ts-ignore
          if (ev.target.checked) {
            setParam(param.name, param.bind.options[i]);
          }
        };

        label.appendChild(l);
        l.appendChild(input);
      }

    } else if (param.bind.input === "select") {
      // A select element containing option elements
      const label = document.createElement("label");
      label.innerText = ("label" in param.bind) ? param.bind.label : param.name;

      label.style.display = "inline-flex";
      label.style.alignItems = "center";

      inputsDiv.appendChild(label);

      const input = document.createElement("select");
      label.appendChild(input);

      paramInputElements[param.name] = input;

      for (const i in param.bind.options) {
        const l = document.createElement("option");
        l.value = param.bind.options[i];
        l.innerText = (param.bind.labels && param.bind.labels[i]) ?? param.bind.options[i].toString();
        input.appendChild(l);
      }

      if ("value" in param.bind) {
        input.value = param.bind.value;
      }

      // @ts-ignore
      input.oninput = (ev) => setParam(param.name, ev.target.value);

    }

    if ("value" in param){
        (paramInputElements[param.name] ?? {}).value = param.value;
    }

  }
  return paramInputElements;
};