import { create } from "d3-selection";
import { scaleLinear, scaleTime } from "d3-scale";
import { axisBottom } from "d3-axis";
import { arc } from "d3-shape";
import { brushX } from "d3-brush";

const createBrushInput = (xExtent, scaleType: "linear" | "date", updateVal, updateContinuously?) => {
  const height = 60;
  const width = 500;

  const margin = { top: 10, right: 20, bottom: 20, left: 20 };

  let x;
  if (scaleType == "linear") {
    x = scaleLinear(xExtent, [margin.left, width - margin.right]);
  } else {
    x = scaleTime(xExtent, [margin.left, width - margin.right]);
  }

  const xAxis = g => g
    .attr("transform", `translate(0,${height - margin.bottom})`)
    .call(axisBottom(x));

  const arcHandle = arc()
    .innerRadius(0)
    .outerRadius((height - margin.top - margin.bottom) / 5)
    .startAngle(0)
    .endAngle((d, i) => i ? Math.PI : -Math.PI);

  const brushHandle = (g, selection) => g
    .selectAll(".handle--custom")
    .data([{ type: "w" }, { type: "e" }])
    .join(
      enter => enter.append("path")
        .attr("class", "handle--custom")
        .attr("fill", "#666")
        .attr("fill-opacity", 0.8)
        .attr("stroke", "#000")
        .attr("stroke-width", 1.5)
        .attr("cursor", "ew-resize")
        .attr("d", arcHandle)
    )
    .attr("display", selection === null ? "none" : null)
    .attr("transform", selection === null ? null : (d, i) => `translate(${selection[i]},${(height + margin.top - margin.bottom) / 2})`);


  const svg = create("svg").attr("viewBox", [0, 0, width, height]);
  const brush = brushX()
    .extent([[margin.left, margin.top], [width - margin.right, height - margin.bottom]])
    .on("start brush end", brushed)
    .on("end", (ev) => !updateContinuously && updateVal(ev.selection.map(v => x.invert(v))));

  svg.append("g")
    .call(xAxis);

  const g = svg.append("g");
  g.call(brush)
    .call(brush.move, xExtent.map(x));

  function brushed({ selection }) {
    g.call(brushHandle, selection);
    updateContinuously && updateVal(selection.map(v => x.invert(v)));
  }

  return svg.node();
};

export { createBrushInput };