
const isArray = (x) => Array.isArray(x);
const isDate = (_) => Object.prototype.toString.call(_) === '[object Date]';
const isObject = (_) =>  _ === Object(_);



const formatTooltip = (value) => {

  if (value == null) {
    return "";
  } else if (isArray(value)) {
    return formatArray(value);
  } else if (isObject(value) && !isDate(value)) {
    return formatObject(value);
  }
  return value + "";
};

const formatObject = (obj) => {
  let html = "<table><tbody>";
  const keys = Object.keys(obj);
  html += keys.map(k => `<tr><td class="key" style="text-align: right">${k}:</td><td class="value" style="text-align: left">${formatValue(obj[k])}</td></tr>`).join("\n");
  html += "</tbody></table>";
  return html;
};


const formatArray = (value) => `[${value.map(formatValue).join(", ")}]`;

const formatValue = (value) => {
  // U+2026 is the "horizontal ellipsis" character
  if (isArray(value)) {
    return "[\u2026]";
  } else if (isObject(value) && !isDate(value)) {
    return "{\u2026}";
  }
  return value;
};


export { formatTooltip };